@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -12,13 +12,37 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/types.h>
-#include <inttypes.h>
+#include <stdint.h>
+#include "GetDPConfig.h"
 #include "MallocUtils.h"
 #include "ListUtils.h"
 #include "TreeUtils.h"
 #include "Message.h"
 
-static char *startptr;
+#if !defined(HAVE_GMSH)
+
+int fcmp_int(const void *a, const void *b)
+{
+  return (*(int *)a - *(int *)b);
+}
+
+int fcmp_absint(const void *a, const void *b)
+{
+  return (abs(*(int *)a) - abs(*(int *)b));
+}
+
+int fcmp_double(const void *a, const void *b)
+{
+  double cmp;
+
+  cmp = *(double *)a - *(double *)b;
+  if(cmp > 1.e-16)
+    return 1;
+  else if(cmp < -1.e-16)
+    return -1;
+  else
+    return 0;
+}
 
 List_T *List_Create(int n, int incr, int size)
 {
@@ -82,64 +106,23 @@ int List_Nbr(List_T * liste)
   return (liste) ? liste->n : 0;
 }
 
-void List_Insert(List_T *liste, void *data,
-		 int (*fcmp)(const void *a, const void *b))
-{
-  if (List_Search(liste,data,fcmp) == 0)
-    List_Add(liste,data);
-}
-
-int List_Replace(List_T *liste, void *data,
-		 int (*fcmp)(const void *a, const void *b))
-{
-  void *ptr;
-
-  if (liste->isorder != 1) List_Sort(liste,fcmp);
-  liste->isorder = 1;
-  ptr = (void *) bsearch(data,liste->array,liste->n,liste->size,fcmp);
-  if (ptr == NULL) {
-    List_Add(liste,data);
-    return(0);
-  }
-  else {
-    memcpy(ptr,data,liste->size);
-    return (1);
-  }
-}
-
 void List_Read(List_T * liste, int index, void *data)
 {
   if((index < 0) || (index >= liste->n))
-    Msg::Fatal("Wrong list index (read)");
+    Message::Fatal("Wrong list index (read)");
   memcpy(data, &liste->array[index * liste->size], liste->size);
 }
 
 void List_Write(List_T * liste, int index, void *data)
 {
   if((index < 0) || (index >= liste->n))
-    Msg::Error("Wrong list index (write)");
+    Message::Error("Wrong list index (write)");
   else {
     liste->isorder = 0;
     memcpy(&liste->array[index * liste->size], data, liste->size);
   }
 }
 
-void List_Put(List_T * liste, int index, void *data)
-{
-  if(index < 0)
-    Msg::Error("Wrong list index (put)");
-  else {
-    if(index >= liste->n) {
-      liste->n = index + 1;
-      List_Realloc(liste, liste->n);
-      List_Write(liste, index, data);
-    }
-    else {
-      List_Write(liste, index, data);
-    }
-  }
-}
-
 void List_Pop(List_T * liste)
 {
   if(liste->n > 0)
@@ -149,7 +132,7 @@ void List_Pop(List_T * liste)
 void *List_Pointer(List_T * liste, int index)
 {
   if((index < 0) || (index >= liste->n))
-    Msg::Fatal("Wrong list index (pointer)");
+    Message::Fatal("Wrong list index (pointer)");
 
   liste->isorder = 0;
   return (&liste->array[index * liste->size]);
@@ -158,7 +141,7 @@ void *List_Pointer(List_T * liste, int index)
 void *List_Pointer_NoChange(List_T * liste, int index)
 {
   if((index < 0) || (index >= liste->n))
-    Msg::Fatal("Wrong list index (pointer)");
+    Message::Fatal("Wrong list index (pointer)");
 
   return (&liste->array[index * liste->size]);
 }
@@ -197,20 +180,6 @@ int List_Search(List_T * liste, void *data,
   return (1);
 }
 
-int List_ISearch(List_T * liste, void *data,
-                 int (*fcmp) (const void *a, const void *b))
-{
-  void *ptr;
-
-  if(liste->isorder != 1)
-    List_Sort(liste, fcmp);
-  liste->isorder = 1;
-  ptr = (void *)bsearch(data, liste->array, liste->n, liste->size, fcmp);
-  if(ptr == NULL)
-    return (-1);
-  return (((intptr_t)ptr - (intptr_t)liste->array) / liste->size);
-}
-
 int List_ISearchSeq(List_T * liste, void *data,
                     int (*fcmp) (const void *a, const void *b))
 {
@@ -226,74 +195,6 @@ int List_ISearchSeq(List_T * liste, void *data,
   return i;
 }
 
-int  List_ISearchSeqPartial(List_T *liste, void * data, int i_Start,
-			    int (*fcmp)(const void *a, const void *b))
-{
-  int i ;
-
-  if (!liste)  return -1 ;
-  i = i_Start ;
-  while ((i < List_Nbr(liste)) &&
-         fcmp(data, (void *)List_Pointer(liste, i)) )  i++ ;
-  if (i == List_Nbr(liste))  i = -1 ;
-  return i ;
-}
-
-int List_Query(List_T *liste, void *data,
-		 int (*fcmp)(const void *a, const void *b))
-{
-  void *ptr;
-
-  if (liste->isorder != 1) List_Sort(liste,fcmp);
-  liste->isorder = 1;
-  ptr = (void *) bsearch(data,liste->array,liste->n,liste->size,fcmp);
-  if (ptr == NULL) return(0);
-
-  memcpy(data,ptr,liste->size);
-  return (1);
-}
-
-static void *lolofind(void *data, void *array, int n, int size,
-		      int (*fcmp)(const void *a, const void *b) )
-{
-  char *ptr;
-  int i;
-
-  ptr = (char*)array;
-  for (i = 0; i < n; i++) {
-    if (fcmp(ptr,data) == 0) break;
-    ptr += size;
-  }
-  if (i < n) return(ptr);
-  return(NULL);
-}
-
-int List_LQuery(List_T *liste, void *data,
-		 int (*fcmp)(const void *a, const void *b), int first)
-{
-  char *ptr;
-  
-  if (first == 1) { 
-    ptr = (char *) lolofind(data,liste->array,liste->n,liste->size,fcmp);
-  }
-  else {
-    if (startptr != NULL)
-      ptr = (char *) lolofind(data,startptr,
-			      liste->n - (startptr-liste->array)/liste->size,
-			      liste->size,fcmp);
-    else
-      return(0);
-  }
-
-  if (ptr == NULL) return(0);
-
-  startptr =  ptr + liste->size;
-  if ( startptr >= ( liste->array + liste->n * liste->size))
-    startptr = NULL;
-  memcpy(data,ptr,liste->size);
-  return (1);
-}
-
 void *List_PQuery(List_T * liste, void *data,
                   int (*fcmp) (const void *a, const void *b))
 {
@@ -306,21 +207,6 @@ void *List_PQuery(List_T * liste, void *data,
   return (ptr);
 }
 
-int List_Suppress(List_T *liste, void *data,
-		 int (*fcmp)(const void *a, const void *b))
-{
-  char *ptr;
-  int len;
-  
-  ptr = (char*)List_PQuery(liste,data,fcmp) ;
-  if (ptr == NULL) return(0);
-  
-  liste->n--;
-  len = liste->n - (((intptr_t)ptr - (intptr_t)liste->array) / liste->size);
-  if (len > 0) memmove(ptr, ptr + liste->size, len * liste->size);
-  return(1);
-}
-
 int List_PSuppress(List_T * liste, int index)
 {
   char *ptr;
@@ -370,123 +256,22 @@ void List_Copy(List_T * a, List_T * b)
   }
 }
 
-List_T *List_Copy(List_T *src)
+List_T *ListOfDouble2ListOfInt(List_T *dList)
 {
-  List_T *dest = (List_T *)Malloc(sizeof(List_T));
-  dest->nmax = src->nmax;
-  dest->incr = src->incr;
-  dest->size = src->size;
-  dest->n = src->n;
-  dest->isorder = src->isorder;
-  dest->array = (char *)Malloc(src->nmax * src->size);
-  memcpy(dest->array, src->array, src->nmax * src->size);
-  return dest;
+  int n = List_Nbr(dList); 
+  List_T *iList = List_Create(n, n, sizeof(int));
+  for(int i = 0; i < n; i++){
+    double d;
+    List_Read(dList, i, &d);
+    int j = (int)d;
+    List_Add(iList, &j);
+  }
+  return iList;
 }
 
-void List_Merge(List_T * a, List_T * b)
-{
-  int i;
+#endif
 
-  if(!a || !b) return;
-  for(i = 0; i < List_Nbr(a); i++) {
-    List_Add(b, List_Pointer_Fast(a, i));
-  }
-}
-
-void swap_bytes(char *array, int size, int n)
-{
-  int i, c;
-  char *x, *a;
-
-  x = (char *)Malloc(size);
-
-  for(i = 0; i < n; i++) {
-    a = &array[i * size];
-    memcpy(x, a, size);
-    for(c = 0; c < size; c++)
-      a[size - 1 - c] = x[c];
-  }
-
-  Free(x);
-}
-
-List_T *List_CreateFromFile(int n, int incr, int size, FILE * file, int format,
-                            int swap)
-{
-  int i, error = 0;
-  List_T *liste;
-
-  if(!n){
-    liste = List_Create(1, incr, size);
-    return liste;
-  }
-
-  liste = List_Create(n, incr, size);
-  liste->n = n;
-  switch (format) {
-  case LIST_FORMAT_ASCII:
-    if(size == sizeof(double)){
-      for(i = 0; i < n; i++){
-        if(!fscanf(file, "%lf", (double *)&liste->array[i * size])){
-          error = 1;
-          break;
-        }
-      }
-    }
-    else if(size == sizeof(float)){
-      for(i = 0; i < n; i++){
-        if(!fscanf(file, "%f", (float *)&liste->array[i * size])){
-          error = 1;
-          break;
-        }
-      }
-    }
-    else if(size == sizeof(int)){
-      for(i = 0; i < n; i++){
-        if(!fscanf(file, "%d", (int *)&liste->array[i * size])){
-          error = 1;
-          break;
-        }
-      }
-    }
-    else if(size == sizeof(char)){
-      for(i = 0; i < n; i++){
-        char c = (char)fgetc(file);
-        if(c == EOF){
-          error = 1;
-          break;
-        }
-        else{
-          liste->array[i * size] = c;
-        }
-      }
-    }
-    else{
-      Msg::Error("Bad type of data to create list from (size = %d)", size);
-      error = 1;
-    }
-    break;
-  case LIST_FORMAT_BINARY:
-    if(!fread(liste->array, size, n, file)){
-      error = 1;
-      break;
-    }
-    if(swap)
-      swap_bytes(liste->array, size, n);
-    break;
-  default:
-    Msg::Error("Unknown list format");
-    error = 1;
-    break;
-  }
-
-  if(error){
-    Msg::Error("Read error");
-    liste->n = 0;
-  }
-
-  return liste;
-}
+// These are not defined in Gmsh:
 
 static int safe_fwrite(const void *ptr, size_t size, size_t nmemb, FILE * stream)
 {
@@ -494,13 +279,13 @@ static int safe_fwrite(const void *ptr, size_t size, size_t nmemb, FILE * stream
 
   if(result < nmemb) {
     if(result >= 0)     /* Partial write */
-      Msg::Error("Disk full");
+      Message::Error("Disk full");
     else
-      Msg::Error(strerror(errno));
+      Message::Error(strerror(errno));
     if(fflush(stream) < 0)
-      Msg::Error("EOF reached");
+      Message::Error("EOF reached");
     if(fclose(stream) < 0)
-      Msg::Error(strerror(errno));
+      Message::Error(strerror(errno));
     return 1;
   }
   return 0;
@@ -528,131 +313,167 @@ void List_WriteToFile(List_T * liste, FILE * file, int format)
       for(i = 0; i < n; i++)
         fputc(*((char *)&liste->array[i * liste->size]), file);
     else
-      Msg::Error("Bad type of data to write list to file (size = %d)",
+      Message::Error("Bad type of data to write list to file (size = %d)",
           liste->size);
     break;
   case LIST_FORMAT_BINARY:
     safe_fwrite(liste->array, liste->size, n, file);
     break;
   default:
-    Msg::Error("Unknown list format");
+    Message::Error("Unknown list format");
     break;
   }
 }
 
-// For backward compatibility
-
-List_T *List_CreateFromFileOld(int n, int incr, int size, FILE * file, int format,
-                               int swap)
+int List_Query(List_T *liste, void *data,
+		 int (*fcmp)(const void *a, const void *b))
 {
-  int i, error = 0;
-  List_T *liste;
+  void *ptr;
 
-  if(!n){
-    liste = List_Create(1, incr, size);
-    return liste;
+  if (liste->isorder != 1) List_Sort(liste,fcmp);
+  liste->isorder = 1;
+  ptr = (void *) bsearch(data,liste->array,liste->n,liste->size,fcmp);
+  if (ptr == NULL) return(0);
+
+  memcpy(data,ptr,liste->size);
+  return (1);
+}
+
+int List_Suppress(List_T *liste, void *data,
+		 int (*fcmp)(const void *a, const void *b))
+{
+  char *ptr;
+  int len;
+  
+  ptr = (char*)List_PQuery(liste,data,fcmp) ;
+  if (ptr == NULL) return(0);
+  
+  liste->n--;
+  len = liste->n - (((intptr_t)ptr - (intptr_t)liste->array) / liste->size);
+  if (len > 0) memmove(ptr, ptr + liste->size, len * liste->size);
+  return(1);
+}
+
+void List_Insert(List_T *liste, void *data,
+		 int (*fcmp)(const void *a, const void *b))
+{
+  if (List_Search(liste,data,fcmp) == 0)
+    List_Add(liste,data);
+}
+
+List_T *List_Copy(List_T *src)
+{
+  List_T *dest = (List_T *)Malloc(sizeof(List_T));
+  dest->nmax = src->nmax;
+  dest->incr = src->incr;
+  dest->size = src->size;
+  dest->n = src->n;
+  dest->isorder = src->isorder;
+  dest->array = (char *)Malloc(src->nmax * src->size);
+  memcpy(dest->array, src->array, src->nmax * src->size);
+  return dest;
+}
+
+int List_ISearch(List_T * liste, void *data,
+                 int (*fcmp) (const void *a, const void *b))
+{
+  void *ptr;
+
+  if(liste->isorder != 1)
+    List_Sort(liste, fcmp);
+  liste->isorder = 1;
+  ptr = (void *)bsearch(data, liste->array, liste->n, liste->size, fcmp);
+  if(ptr == NULL)
+    return (-1);
+  return (((intptr_t)ptr - (intptr_t)liste->array) / liste->size);
+}
+
+int List_ISearchSeqPartial(List_T *liste, void * data, int i_Start,
+                           int (*fcmp)(const void *a, const void *b))
+{
+  int i ;
+
+  if (!liste)  return -1 ;
+  i = i_Start ;
+  while ((i < List_Nbr(liste)) &&
+         fcmp(data, (void *)List_Pointer(liste, i)) )  i++ ;
+  if (i == List_Nbr(liste))  i = -1 ;
+  return i ;
+}
+
+int List_Replace(List_T *liste, void *data,
+		 int (*fcmp)(const void *a, const void *b))
+{
+  void *ptr;
+
+  if (liste->isorder != 1) List_Sort(liste,fcmp);
+  liste->isorder = 1;
+  ptr = (void *) bsearch(data,liste->array,liste->n,liste->size,fcmp);
+  if (ptr == NULL) {
+    List_Add(liste,data);
+    return(0);
+  }
+  else {
+    memcpy(ptr,data,liste->size);
+    return (1);
+  }
+}
+
+static void *lolofind(void *data, void *array, int n, int size,
+		      int (*fcmp)(const void *a, const void *b) )
+{
+  char *ptr;
+  int i;
+
+  ptr = (char*)array;
+  for (i = 0; i < n; i++) {
+    if (fcmp(ptr,data) == 0) break;
+    ptr += size;
+  }
+  if (i < n) return(ptr);
+  return(NULL);
+}
+
+static char *startptr = NULL;
+
+int List_LQuery(List_T *liste, void *data,
+		 int (*fcmp)(const void *a, const void *b), int first)
+{
+  char *ptr;
+  
+  if (first == 1) { 
+    ptr = (char *) lolofind(data,liste->array,liste->n,liste->size,fcmp);
+  }
+  else {
+    if (startptr != NULL)
+      ptr = (char *) lolofind(data,startptr,
+			      liste->n - (startptr-liste->array)/liste->size,
+			      liste->size,fcmp);
+    else
+      return(0);
   }
 
-  liste = List_Create(n, incr, size);
-  liste->n = n;
-  switch (format) {
-  case LIST_FORMAT_ASCII:
-    if(size == sizeof(double)){
-      for(i = 0; i < n; i++){
-        if(!fscanf(file, "%lf", (double *)&liste->array[i * size])){
-          error = 1;
-          break;
-        }
-      }
-    }
-    else if(size == sizeof(float)){
-      for(i = 0; i < n; i++){
-        if(!fscanf(file, "%f", (float *)&liste->array[i * size])){
-          error = 1;
-          break;
-        }
-      }
-    }
-    else if(size == sizeof(int)){
-      for(i = 0; i < n; i++){
-        if(!fscanf(file, "%d", (int *)&liste->array[i * size])){
-          error = 1;
-          break;
-        }
-      }
-    }
-    else if(size == sizeof(char)){
-      for(i = 0; i < n; i++){
-        if(!fscanf(file, "%c", (char *)&liste->array[i * size])){
-          error = 1;
-          break;
-        }
-        if(liste->array[i * size] == '^')
-          liste->array[i * size] = '\0';
-      }
+  if (ptr == NULL) return(0);
+
+  startptr =  ptr + liste->size;
+  if ( startptr >= ( liste->array + liste->n * liste->size))
+    startptr = NULL;
+  memcpy(data,ptr,liste->size);
+  return (1);
+}
+
+void List_Put(List_T * liste, int index, void *data)
+{
+  if(index < 0)
+    Message::Error("Wrong list index (put)");
+  else {
+    if(index >= liste->n) {
+      liste->n = index + 1;
+      List_Realloc(liste, liste->n);
+      List_Write(liste, index, data);
     }
     else {
-      Msg::Error("Bad type of data to create list from (size = %d)", size);
-      error = 1;
+      List_Write(liste, index, data);
     }
-    return liste;
-  case LIST_FORMAT_BINARY:
-    if(!fread(liste->array, size, n, file)){
-      error = 1;
-      break;
-    }
-    if(swap)
-      swap_bytes(liste->array, size, n);
-    return liste;
-  default:
-    Msg::Error("Unknown list format");
-    error = 1;
-    break;
   }
-
-  if(error){
-    Msg::Error("Read error");
-    liste->n = 0;
-  }
-
-  return liste;
 }
-
-// Comparison functions
-
-int fcmp_int(const void *a, const void *b)
-{
-  return (*(int *)a - *(int *)b);
-}
-
-int fcmp_absint(const void *a, const void *b)
-{
-  return (abs(*(int *)a) - abs(*(int *)b));
-}
-
-int fcmp_double(const void *a, const void *b)
-{
-  double cmp;
-
-  cmp = *(double *)a - *(double *)b;
-  if(cmp > 1.e-16)
-    return 1;
-  else if(cmp < -1.e-16)
-    return -1;
-  else
-    return 0;
-}
-
-List_T *ListOfDouble2ListOfInt(List_T *dList)
-{
-  int n = List_Nbr(dList); 
-  List_T *iList = List_Create(n, n, sizeof(int));
-  for(int i = 0; i < n; i++){
-    double d;
-    List_Read(dList, i, &d);
-    int j = (int)d;
-    List_Add(iList, &j);
-  }
-  return iList;
-}
-
