@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -16,7 +16,7 @@ void *Malloc(size_t size)
     return (NULL);
   ptr = malloc(size);
   if(ptr == NULL)
-    Msg::Fatal("Out of memory (buy some more RAM!)");
+    Message::Fatal("Out of memory (buy some more RAM!)");
   return (ptr);
 }
 
@@ -28,7 +28,7 @@ void *Calloc(size_t num, size_t size)
     return (NULL);
   ptr = calloc(num, size);
   if(ptr == NULL)
-    Msg::Fatal("Out of memory (buy some more RAM!)");
+    Message::Fatal("Out of memory (buy some more RAM!)");
   return (ptr);
 }
 
@@ -38,7 +38,7 @@ void *Realloc(void *ptr, size_t size)
     return (NULL);
   ptr = realloc(ptr, size);
   if(ptr == NULL)
-    Msg::Fatal("Out of memory (buy some more RAM!)");
+    Message::Fatal("Out of memory (buy some more RAM!)");
   return (ptr);
 }
 
