@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -8,9 +8,14 @@
 
 #include <map>
 #include <string>
+#include <vector>
 #include <stdarg.h>
 
 class GmshClient;
+namespace onelab{ class client; }
+class Constant;
+class Expression;
+class Group;
 
 // a class to manage messages
 class Message {
@@ -25,6 +30,10 @@ class Message {
   static std::map<std::string, double> _timers;
   // communication with Gmsh
   static GmshClient *_client;
+  // communication with onelab server
+  static onelab::client *_onelabClient;
+  //parallelization
+  static int _isCommWorld;
  public:
   Message() {}
   static void Init(int argc, char **argv);
@@ -34,6 +43,8 @@ class Message {
   static void SetCommRank(int val){ _commRank = val; }
   static void SetCommSize(int val){ _commSize = val; }
   static void Barrier();
+  static int GetIsCommWorld(){return _isCommWorld; }
+  static void SetIsCommWorld(int val){ _isCommWorld = val; }
   static int GetNumThreads();
   static int GetMaxThreads();
   static int GetThreadNum();
@@ -54,13 +65,24 @@ class Message {
   static double &Timer(std::string str){ return _timers[str]; }
   static void PrintTimers();
   static void InitializeSocket(std::string sockname);
+  static void FinalizeSocket();
+  static bool UseSocket(){ return _client ? true : false; }
   static void SendFileOnSocket(std::string filename);
   static void SendOptionOnSocket(int num, std::string option);
   static void TestSocket();
-  static void FinalizeSocket();
-  static bool UseSocket(){ return _client ? true : false; }
+  static void InitializeOnelab(std::string name, std::string sockname);
+  static void FinalizeOnelab();
+  static bool UseOnelab(){ return _onelabClient ? true : false; }
+  static std::string GetOnelabClientName();
+  static void GetOnelabString(std::string name, char **val);
+  static std::string GetOnelabAction();
+  static void AddOnelabNumberChoice(std::string name, double val);
+  static void AddOnelabStringChoice(std::string name, std::string kind, std::string choice);
+  typedef std::map<std::string, std::vector<double> > fmap;
+  typedef std::map<std::string, std::vector<std::string> > cmap;
+  static void ExchangeOnelabParameter(Constant *c, fmap &fopt, cmap &copt);
+  static void ExchangeOnelabParameter(Group *p, fmap &fopt, cmap &copt);
+  static void ExchangeOnelabParameter(Expression *p, fmap &fopt, cmap &copt);
 };
 
-typedef Message Msg;
-
 #endif
