@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -23,15 +23,8 @@ int     Tree_Search(Tree_T *Tree, void *data);
 int     Tree_Query(Tree_T *Tree, void *data);
 void   *Tree_PQuery(Tree_T *Tree, void *data);
 int     Tree_Suppress(Tree_T *Tree, void *data);
-int     Tree_Size(Tree_T *tree) ;
-
-inline void Tree_Action(Tree_T *tree, void (*action) (void *data, void *dummy))
-{
-  if(!tree) return;
-
-  avl_foreach(tree->root, action, AVL_FORWARD);
-}
-
+int     Tree_Size(Tree_T *tree);
+void    Tree_Action(Tree_T *tree, void (*action) (void *data, void *dummy));
 List_T *Tree2List(Tree_T *pTree);
 
 #endif
