@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -33,7 +33,7 @@
 // Modified for Gmsh (C++, 64 bits, ...)
 
 #include <stdio.h>
-#include <inttypes.h>
+#include <stdint.h>
 
 #include "avl.h"
 #include "MallocUtils.h"
