@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -6,6 +6,7 @@
 #ifndef _PRO_DATA_H_
 #define _PRO_DATA_H_
 
+#include <string>
 #include "GetDPConfig.h"
 #include "ListUtils.h"
 
@@ -46,7 +47,7 @@
 #if defined(HAVE_MULTIHARMONIC)
 #define NBR_MAX_HARMONIC    40
 #else
-#define NBR_MAX_HARMONIC    2   
+#define NBR_MAX_HARMONIC    2
 #endif
 
 #define MAX_STACK_SIZE      40
@@ -63,7 +64,7 @@ struct TwoInt { int  Int1, Int2; };
 /* ------------------------------------------------------------------------ */
 
 struct Problem {
-  List_T *Group         , *Expression; 
+  List_T *Group         , *Expression;
   List_T *FunctionSpace , *Constraint        , *Formulation;
   List_T *JacobianMethod, *IntegrationMethod;
   List_T *Resolution    , *PostProcessing    , *PostOperation;
@@ -78,11 +79,13 @@ struct Group {
   int     Num,  Type, FunctionType, SuppListType;
   List_T *InitialList, *InitialSuppList;
   List_T *ExtendedList, *ExtendedSuppList;
-  struct  MovingBand2D *MovingBand2D; 
+  bool    IsExtendedListMultiValued;
+  struct  MovingBand2D *MovingBand2D;
 };
 
 struct MovingBand2D {
-  List_T *InitialList1, *ExtendedList1, *InitialList2, *ExtendedList2;
+  List_T *InitialList1,  *InitialList2;
+  List_T *ExtendedList1, *ExtendedList2;
   int NbrNodes1, *NumNodes1, NbrNodes2, *NumNodes2;
   double *x1, *y1, *z1, *x2, *y2, *z2, Area;
   int  Period2, ntr1, ntr2, Closed1, Closed2;
@@ -330,6 +333,7 @@ struct BasisFunction {
   int     SupportIndex, EntityIndex;
   double  Order;
   int     ElementType;
+  int     Orient;
 };
 
 struct GlobalBasisFunction {
@@ -355,7 +359,7 @@ struct GlobalBasisFunction {
 #define FORM2P      12
 #define FORM3P      13
 
-#define SCALAR      20 
+#define SCALAR      20
 #define VECTOR      21
 #define TENSOR      22  /* second-rank tensor of order 3 */
 #define TENSOR_SYM  23
@@ -405,7 +409,7 @@ struct ConstraintInFS {
 /* ------------------------------------------------------------------------ */
 
 struct Function {
-  void    (*Fct)(); /* ANSI C++ forbids data member `Function' with 
+  void    (*Fct)(); /* ANSI C++ forbids data member `Function' with
 			same name as enclosing class */
   int     TypeOfValue, NbrArguments, NbrParameters;
   double *Para;
@@ -422,9 +426,17 @@ struct FunctionActive {
     struct {
       double  Value;
     } SurfaceArea;
+    struct {
+      double  Value;
+    } GetVolume;
     struct {
       List_T *Table;
     } ValueFromIndex;
+    struct {
+      int     NbrLines, NbrColumns;
+      double *x, *y ;
+      double *data ;
+    } ListMatrix;
   } Case;
 };
 
@@ -451,7 +463,7 @@ struct IntegralQuantity {
   int      DefineQuantityIndexNoDof;
 
   int      NbrQuantityIndex, *QuantityIndexTable;
-  int    *QuantityTraceGroupIndexTable;
+  int    *QuantityTraceGroupIndexTable ;
 
   int      InIndex;
   int      IntegrationMethodIndex, JacobianMethodIndex;
@@ -464,7 +476,7 @@ struct IntegralQuantity {
 struct IntegralQuantityActive {
   int     Type_FormDof, Type_ValueDof;
 
-  List_T  *IntegrationCase_L; 
+  List_T  *IntegrationCase_L;
   struct  IntegrationCase *IntegrationCase_P;
   int     CriterionIndex;
   void    (*Get_IntPoint)();
@@ -477,7 +489,7 @@ struct IntegralQuantityActive {
 };
 
 
-struct FirstElement { 
+struct FirstElement {
   struct Dof *Equ;
   struct Dof *Dof;
   double Value;
@@ -495,14 +507,15 @@ struct DefineQuantity {
   List_T  *DummyFrequency;
 
   /* for multiple DofData vs. one FunctionSpace */
-  int      DofDataIndex; 
+  int      DofDataIndex;
   struct   DofData  *DofData;
 
-  /* for integral quantities */ 
+  /* for integral quantities */
   struct   IntegralQuantity  IntegralQuantity;
 };
 
-#define NBR_MAX_BASISFUNCTIONS         60
+// second order hex with 3 BFs per node for elasticity
+#define NBR_MAX_BASISFUNCTIONS         81
 
 struct QuantityStorage {
 
@@ -548,7 +561,7 @@ struct EquationTerm {
 	int     DofIndexInWholeQuantity;
 	int     CanonicalWholeQuantity, ExpressionIndexForCanonical;
 	struct  Function  FunctionForCanonical;
-	int     CanonicalWholeQuantity_Equ, ExpressionIndexForCanonical_Equ, 
+	int     CanonicalWholeQuantity_Equ, ExpressionIndexForCanonical_Equ,
 	          OperatorTypeForCanonical_Equ;
 	void    (*BuiltInFunction_Equ)();
 
@@ -560,10 +573,12 @@ struct EquationTerm {
 	int     DefineQuantityIndexNoDof, DofInTrace;
       } Term;
 
-      int  InIndex; 
+      int  InIndex;
       int  Full_Matrix;
       int  IntegrationMethodIndex, JacobianMethodIndex;
-    
+
+      int  ExpressionIndexForMetricTensor;
+
       int  MatrixIndex;
       struct FemLocalTermActive  *Active;
     } LocalTerm;
@@ -581,7 +596,7 @@ struct EquationTerm {
 	int     CanonicalWholeQuantity, ExpressionIndexForCanonical;
 
 	int     NbrQuantityIndex, *QuantityIndexTable;
-	int     *QuantityTraceGroupIndexTable;
+	int     *QuantityTraceGroupIndexTable ;
 
 	int     TypeOperatorEqu, DefineQuantityIndexEqu;
 	int     TypeOperatorDof, DefineQuantityIndexDof;
@@ -627,7 +642,7 @@ struct FemLocalTermActive {
 
   int Full_Matrix;
   int NbrEqu, NbrHar, *NumEqu, *NumDof;
-  struct Dof *Equ, *Dof; 
+  struct Dof *Equ, *Dof;
   List_T *FirstElements;
   double **Matrix;
 };
@@ -639,7 +654,7 @@ struct GlobalQuantityStorage {
   int     CodeGlobalQuantity, CodeAssociateBasisFunction;
 
   int     CodeEntity;
-  int     Constraint;  
+  int     Constraint;
   double  Value[NBR_MAX_HARMONIC];
   int     TimeFunctionIndex;
 };
@@ -680,9 +695,9 @@ struct GlobalEquationTerm {
 #define CURLINV    7
 #define DIVINV     8
 
-/* Tous ces operateurs de trace ne servent a RIEN pour le moment 
-   De plus, les 'x' sont ambigus. Il faut penser a definir des 
-   operateurs de trace (T ou T*), qui doivent avoir, outre 
+/* Tous ces operateurs de trace ne servent a RIEN pour le moment
+   De plus, les 'x' sont ambigus. Il faut penser a definir des
+   operateurs de trace (T ou T*), qui doivent avoir, outre
    \Gamma=\partial\Omega, l'info concernant \Omega.
  */
 #define NPx        9
@@ -714,7 +729,7 @@ struct GlobalEquationTerm {
 #define CWQ_GF_PSCA_EXP    7
 #define CWQ_GF_PVEC_DOF    8
 #define CWQ_DOF_PVEC_GF    9
-#define CWQ_GF_PVEC_EXP   10 
+#define CWQ_GF_PVEC_EXP   10
 #define CWQ_EXP_PVEC_GF   11
 
 #define CWQ_EXP_TIME_GF_PSCA_DOF  12
@@ -748,14 +763,14 @@ struct WholeQuantity {
     struct { int  Index; }                                       ShowValue;
     struct { int  Index; }                                       ValueSaved;
     struct { int  TypeOperator; void  (*Function)(); }           Operator; /* binary or unary */
-    struct { List_T *WholeQuantity; 
+    struct { List_T *WholeQuantity;
              int FunctionSpaceIndexForType, NbrHar; }            Cast;
     struct { List_T *WholeQuantity ; }                           ChangeCurrentPosition ;
-    struct { List_T *WholeQuantity; 
+    struct { List_T *WholeQuantity ;
              int InIndex, DofIndexInWholeQuantity; }             Trace;
-    struct { char *SystemName; int DefineSystemIndex; 
-             int DofNumber; }                                    DofValue; 
-    struct { List_T *WholeQuantity; 
+    struct { char *SystemName; int DefineSystemIndex;
+             int DofNumber; }                                    DofValue;
+    struct { List_T *WholeQuantity;
              int Index, NbrPoints; }                             MHTransform;
     struct { int Index, NbrPoints, FreqOffSet; }                 MHJacNL;
   } Case;
@@ -844,7 +859,7 @@ struct DefineSystem {
 #define VAL_COMPLEX  2
 
 struct Operation {
-  int  Type, DefineSystemIndex;
+  int  Type, DefineSystemIndex, Rank;
 
   union {
     struct {
@@ -853,6 +868,9 @@ struct Operation {
     struct {
       char *String;
     } SystemCommand;
+    struct {
+      char *FileName;
+    } GmshRead;
     struct {
       int     ExpressionIndex;
     } SetTime;
@@ -867,7 +885,7 @@ struct Operation {
     } SetFrequency;
     struct {
       List_T  *Frequency;
-      int     DefineSystemIndex[2]; 
+      int     DefineSystemIndex[2];
     } FourierTransform;
     struct {
       int     DefineSystemIndex[2];
@@ -875,16 +893,16 @@ struct Operation {
       double *Scales;
     } FourierTransform2;
     struct {
-      int     Size; 
+      int     Size;
       List_T  *Save;
-      double  Shift; 
+      double  Shift;
     } Lanczos;
     struct {
-      int     NumEigenvalues; 
-      double  Shift_r, Shift_i; 
+      int     NumEigenvalues;
+      double  Shift_r, Shift_i;
     } EigenSolve;
     struct {
-      int     ExpressionIndex; 
+      int     ExpressionIndex;
     } Evaluate;
     struct {
       int     Iteration ;
@@ -896,7 +914,7 @@ struct Operation {
       double  Alpha ;
     } MultiplySolution ;
     struct {
-      int     Size; 
+      int     Size;
       List_T  *Save;
       double  Shift;
       int     PertFreq;
@@ -912,6 +930,15 @@ struct Operation {
       int     DTimeIndex;
       List_T  *Operation;
     } TimeLoopNewmark;
+    struct {
+      double  Time0, TimeMax;
+      List_T *ButcherA, *ButcherB, *ButcherC;
+      int     DTimeIndex;
+    } TimeLoopRungeKutta;
+    struct {
+      double  Time0, TimeMax, DTimeMin, DTimeMax, MaxOrder;
+      List_T  *DefineSystemIndices, *Operation, *OperationEnd;
+    } TimeLoopAdaptive;
     struct {
       double  Criterion;
       int     NbrMaxIteration, RelaxationFactorIndex, Flag;
@@ -923,13 +950,22 @@ struct Operation {
       List_T  *ChangeOfState;
       List_T  *Operation, *OperationEnd;
     } IterativeTimeReduction;
+    struct {
+      char   *Type;
+      double Tolerance;
+      int    MaxIter;
+      int    Restart;
+      List_T *FieldIndices;
+//      List_T *DeflationIndices;
+      List_T *Operations_Ax, *Operations_Mx;
+    } IterativeLinearSolver;
     struct {
       int     ExpressionIndex;
       List_T  *Operation_True, *Operation_False;
     } Test;
     struct {
       List_T  *DofNumber, *TimeStep, *Expression;
-      char    *FileOut;       
+      char    *FileOut;
     } Print;
     struct {
       int     GroupIndex, ExpressionIndex;
@@ -973,14 +1009,11 @@ struct Operation {
       List_T  *Operation;
     } Generate_MH_Moving_S;
     struct {
-      double  dummy; 
+      double  dummy;
     } Add_MH_Moving;
     struct {
       int     GroupIndex;
     } Generate;
-    struct {
-      int     SolverIndex;
-    } Solve;
     struct {
       int     GroupIndex;
       char    *FileName;
@@ -994,8 +1027,8 @@ struct Operation {
     } DeformeMesh;
     // FIXME: Roman
     struct {
-      List_T *SystemIndex, *ExpectationIndex; 
-      List_T *LocalMatrixIndex; 
+      List_T *SystemIndex, *ExpectationIndex;
+      List_T *LocalMatrixIndex;
       List_T *ExpansionCoef;
     } TensorProductSolve;
 
@@ -1003,7 +1036,6 @@ struct Operation {
 
 };
 
-
 struct ChangeOfState {
   int     Type;
   int     QuantityIndex, InIndex, FormulationIndex;
@@ -1012,14 +1044,15 @@ struct ChangeOfState {
   int     ExpressionIndex, ExpressionIndex2, FlagIndex;
 };
 
-
 /* Operation.Type */
 #define OPERATION_NONE                      0
 
 #define OPERATION_GENERATE                  1
 #define OPERATION_SOLVE                     2
-#define OPERATION_SOLVEAGAIN              223  
+#define OPERATION_SOLVEAGAIN              223
+#define OPERATION_SOLVENL                 233
 #define OPERATION_GENERATEJAC               3
+#define OPERATION_GENERATERHS             100
 #define OPERATION_GENERATEONLY            101
 #define OPERATION_GENERATEONLYJAC         103
 #define OPERATION_SOLVEJAC                  4
@@ -1028,7 +1061,6 @@ struct ChangeOfState {
 #define OPERATION_GENERATESEPARATE          5
 #define OPERATION_UPDATE                    6
 #define OPERATION_UPDATECONSTRAINT          7
-#define OPERATION_UPDATETRANSLATION       104
 #define OPERATION_LANCZOS                   8
 #define OPERATION_PERTURBATION              9
 #define OPERATION_EIGENSOLVE               16
@@ -1053,12 +1085,10 @@ struct ChangeOfState {
 #define OPERATION_ADD_MH_MOVING          9992
 #define OPERATION_DUMMYDOFS              9993
 
-
 #define OPERATION_SAVEMESH                333
 #define OPERATION_DEFORMEMESH             334
 #define OPERATION_READSOLUTION             12
 #define OPERATION_TRANSFERSOLUTION         13
-#define OPERATION_TRANSFERINITSOLUTION     14
 #define OPERATION_INITSOLUTION             15
 #define OPERATION_SETCURRENTSYSTEM         70
 
@@ -1077,6 +1107,9 @@ struct ChangeOfState {
 #define OPERATION_TIMELOOPNEWMARK          31
 #define OPERATION_ITERATIVELOOP            32
 #define OPERATION_ITERATIVETIMEREDUCTION   33
+#define OPERATION_TIMELOOPRUNGEKUTTA       34
+#define OPERATION_ITERATIVELINEARSOLVER    35
+#define OPERATION_TIMELOOPADAPTIVE         36
 
 #define OPERATION_CHANGEOFCOORDINATES      40
 #define OPERATION_CHANGEOFCOORDINATES2    400
@@ -1084,6 +1117,12 @@ struct ChangeOfState {
 #define OPERATION_SYSTEMCOMMAND            50
 
 #define OPERATION_POSTOPERATION            60
+#define OPERATION_GMSHREAD                 61
+#define OPERATION_GMSHCLEARALL             62
+
+#define OPERATION_SETCOMMSELF              63
+#define OPERATION_SETCOMMWORLD             64
+#define OPERATION_BARRIER                  65
 
 /* ChangeOfState.Type */
 #define CHANGEOFSTATE_NOCHANGE              0
@@ -1112,10 +1151,11 @@ struct PreResolutionInfo {
 
 struct PostProcessing {
   char    *Name;
-  int      FormulationIndex; 
+  int      FormulationIndex;
   List_T  *OriginSystemIndex;
   char    *NameOfSystem;
   List_T  *PostQuantity;
+  int	   Rank;
 };
 
 struct PostQuantity {
@@ -1134,8 +1174,8 @@ struct PostQuantityTerm {
 };
 
 /* PostQuantityTerm.Type */
-/* LOCALQUANTITY  
-   GLOBALQUANTITY    
+/* LOCALQUANTITY
+   GLOBALQUANTITY
    INTEGRALQUANTITY      */
 
 /* PostQuantityTerm.EvaluationType */
@@ -1150,18 +1190,25 @@ struct PostOperation {
   char    *Name, *AppendString;
   int      PostProcessingIndex, Format;
   List_T  *PostSubOperation;
+  int Rank;
 };
 
 struct PostSubOperation {
-  int    PostQuantityIndex[2], PostQuantitySupport[2];  
+  int    PostQuantityIndex[2], PostQuantitySupport[2];
   int    Type, SubType, CombinationType;
   int    Depth, Skin, Smoothing, Dimension, Comma, HarmonicToTime, CatFile;
   int    Format, Adapt, Sort, Iso, NoNewLine, DecomposeInSimplex;
-  int    ChangeOfCoordinates[3], LastTimeStepOnly; 
+  int    NewCoordinates;
+  char  *NewCoordinatesFile;
+  int    ValueIndex;
+  int    ChangeOfCoordinates[3], LastTimeStepOnly, AppendTimeStepToFileName;
+  int    OverrideTimeStepValue, NoMesh;
   int    StoreInRegister;
+  char  *SendToServer;
   int    Legend, FrozenTimeStepList;
   double LegendPosition[3];
   double Target;
+  char   *ValueName;
   List_T *HeaderChar_L, *HeaderTag_L;
   List_T *FormatChar_L, *FormatTag_L;
   List_T *FooterChar_L, *FooterTag_L;
@@ -1233,13 +1280,16 @@ struct PostSubOperation {
 #define FORMAT_TIME_TABLE             2
 #define FORMAT_SIMPLE_SPACE_TABLE     3
 #define FORMAT_FREQUENCY_TABLE        4
+#define FORMAT_VALUE_ONLY             5
 #define FORMAT_ADAPT                  9
 #define FORMAT_GMSH                  10
 #define FORMAT_GMSH_PARSED           11
 #define FORMAT_MATLAB                14
 #define FORMAT_GNUPLOT               15
 #define FORMAT_REGION_TABLE          16
-#define FORMAT_UNV                   17
+#define FORMAT_REGION_VALUE          17
+#define FORMAT_UNV                   18
+#define FORMAT_NODE_TABLE            19
 
 /* PostSubOperation.Sort */
 #define SORT_BY_POSITION      1
@@ -1259,6 +1309,7 @@ struct CurrentData {
   char   *Name;
 
   int     NbrSystem, NbrCpu, RankCpu;
+  struct DefineSystem   *DefineSystem_P ;
   struct DofData  *DofData_P0;
 
   struct DofData  *DofData;
@@ -1281,7 +1332,7 @@ struct CurrentData {
 
   int     flagAssDiag;
 
-  /* All values below must be of double type */
+  /* All values below must be double */
 
   double  x, y, z;
   double  u, v, w;
@@ -1388,7 +1439,7 @@ struct Geo_Node {
 
 struct Geo_Element {
   int  Num ;
-  int  Type, Region ; 
+  int  Type, Region, ElementaryRegion ;
   int  NbrNodes , * NumNodes ;
   int  NbrEdges , * NumEdges ;
   int  NbrFacets, * NumFacets ;
@@ -1402,7 +1453,7 @@ struct Entity2XEntity1 {
 struct EntityXVector {
   int     Num;
   double  Vector[3];
-} ; 
+} ;
 
 struct EntityInTree {
   int  Num, Index ;
@@ -1433,10 +1484,12 @@ int fcmp_PostQuantity_Name        (const void *a, const void *b);
 int fcmp_PostOperation_Name       (const void *a, const void *b);
 
 void Init_ProblemStructure();
-void Read_ProblemStructure(char *name);
+void Read_ProblemStructure(const char *fileName);
+void Finalize_ProblemStructure();
 void Print_ProblemStructure();
+std::string Get_AbsolutePath(const char *fileName);
 
 void Print_ListResolution(int choice, int flag_lres, char **name);
-void Print_ListPostOperation(int choice, int flag_lpos, char **name);
+void Print_ListPostOperation(int choice, int flag_lpos, char *name[NBR_MAX_POS]);
 
 #endif
