@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -35,7 +35,7 @@ extern int getdp_yyerrorlevel;
 int getdp_yyparse();
 void getdp_yyrestart(FILE*);
 
-char *strSave(char *string);
+char *strSave(const char *string);
 void cStyleComments();
 void cxxStyleComments();
 void parseString(char endchar);
