@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -7,10 +7,15 @@
 #include "Message.h"
 #include "EigenSolve.h"
 
-void EigenSolve(struct DofData * DofData_P, int NumEigenvalues, 
+#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)))
+#define PetscTruth PetscBool
+#define PetscOptionsGetTruth PetscOptionsGetBool
+#endif
+
+void EigenSolve(struct DofData * DofData_P, int NumEigenvalues,
 		double shift_r, double shift_i)
 {
-#if defined(HAVE_ARPACK) && defined(HAVE_SLEPC) 
+#if defined(HAVE_ARPACK) && defined(HAVE_SLEPC)
   // if both Arpack and SLEPC are available, use Arpack by default
   // (set "-slepc" on the command line to force SLEPC)
   PetscTruth slepc = PETSC_FALSE, set;
@@ -24,6 +29,6 @@ void EigenSolve(struct DofData * DofData_P, int NumEigenvalues,
 #elif defined(HAVE_SLEPC)
   EigenSolve_SLEPC(DofData_P, NumEigenvalues, shift_r, shift_i);
 #else
-  Msg::Error("EigenSolve not available without SLEPC or ARPACK");
+  Message::Error("EigenSolve not available without SLEPC or ARPACK");
 #endif
 }
