@@ -1,8 +1,9 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
 
+#include <set>
 #include <stdlib.h>
 #include "ProData.h"
 #include "ProDefine.h"
@@ -50,7 +51,8 @@ int Check_IsEntityInExtendedGroup(struct Group * Group_P, int Entity, int Flag)
     if (!Group_P->ExtendedList) Generate_ExtendedGroup(Group_P) ;
     return( List_Search(Group_P->ExtendedList, &Entity, fcmp_int) ) ;
 
-  case GROUPSOFNODESOF :  case GROUPSOFEDGESOF :  case REGION :  case GLOBAL :
+  case GROUPSOFNODESOF :  case GROUPSOFEDGESOF : case GROUPSOFFACETSOF :
+  case REGION :  case GLOBAL :
     return( (Flag)? List_Search(Group_P->InitialList, &Entity, fcmp_int) : 1 ) ;
 
   case GROUPSOFEDGESONNODESOF :
@@ -60,7 +62,7 @@ int Check_IsEntityInExtendedGroup(struct Group * Group_P, int Entity, int Flag)
     return(! List_Search(Group_P->ExtendedSuppList, &Entity, fcmp_int)) ;
 
   default :
-    Msg::Error("Unknown function type for Group '%s'", Group_P->Name);
+    Message::Error("Unknown function type for Group '%s'", Group_P->Name);
     return(-1) ;
   }
 }
@@ -72,8 +74,8 @@ int Check_IsEntityInExtendedGroup(struct Group * Group_P, int Entity, int Flag)
 void Generate_ExtendedGroup(struct Group * Group_P)
 {
 
-  Msg::Info("  Generate ExtendedGroup '%s' (%s)", Group_P->Name,
-      Get_StringForDefine(FunctionForGroup_Type, Group_P->FunctionType)) ;
+  Message::Info("  Generate ExtendedGroup '%s' (%s)", Group_P->Name,
+            Get_StringForDefine(FunctionForGroup_Type, Group_P->FunctionType)) ;
 
   switch (Group_P->FunctionType) {
 
@@ -107,6 +109,10 @@ void Generate_ExtendedGroup(struct Group * Group_P)
 			   &Group_P->ExtendedList) ;
     break ;
 
+  case GROUPSOFFACETSOF :
+    Generate_GroupsOfFacets(Group_P->InitialList, &Group_P->ExtendedList) ;
+    break ;
+
   case EDGESOFTREEIN :
     Geo_GenerateEdgesOfTree(Group_P->InitialList, Group_P->InitialSuppList,
 			    &Group_P->ExtendedList) ;
@@ -120,9 +126,29 @@ void Generate_ExtendedGroup(struct Group * Group_P)
     break ;
 
   default :
-    Msg::Error("Unknown function type for Group '%s'", Group_P->Name) ;
+    Message::Error("Unknown function type for Group '%s'", Group_P->Name) ;
     break;
   }
+
+  // Check if ExtendedList only contains single entries with identical
+  // first keys (in absolute value): this allows to dramatically
+  // speed-up Get_DofOfElement() for large global basis functions
+  if(List_Nbr(Group_P->ExtendedList) && (Group_P->FunctionType == GROUPSOFNODESOF ||
+                                         Group_P->FunctionType == GROUPSOFEDGESOF ||
+                                         Group_P->FunctionType == GROUPSOFFACETSOF)) {
+    Group_P->IsExtendedListMultiValued = false;
+    std::set<int> s;
+    for(int i = 0; i < List_Nbr(Group_P->ExtendedList); i++){
+      TwoInt k;
+      List_Read(Group_P->ExtendedList, i, &k);
+      std::pair<std::set<int>::iterator, bool> ret = s.insert(abs(k.Int1));
+      if(ret.second){
+        Group_P->IsExtendedListMultiValued = true;
+        Message::Info("  Extended group is multivalued (search will be slow...)");
+        break;
+      }
+    }
+  }
 }
 
 /* ------------------------------------------------------------------------ */
@@ -189,7 +215,8 @@ void Generate_GroupsOfNodes(List_T * InitialList, List_T ** ExtendedList)
 
   Entity_Tr = Tree_Create(sizeof (struct TwoInt), fcmp_int2) ;
 
-  Nbr_Element = Geo_GetNbrGeoElements() ;  /*  Msg::Info("  Add Node :"); */
+  Nbr_Element = Geo_GetNbrGeoElements() ;
+  // Message::Info("  Add Node :");
   for (i_Element = 0 ; i_Element < Nbr_Element ; i_Element++) {
     GeoElement = Geo_GetGeoElement(i_Element) ;
 
@@ -203,13 +230,14 @@ void Generate_GroupsOfNodes(List_T * InitialList, List_T ** ExtendedList)
 
 	if ( ! Tree_Search(Entity_Tr, &Num_GroupOfNodes) ) {
 	  Tree_Add(Entity_Tr, &Num_GroupOfNodes) ;
-	  /* Msg::Info(" (%d, %d)", Num_GroupOfNodes.Int1, Num_GroupOfNodes.Int2); */
+          // Message::Info(" (%d, %d)", Num_GroupOfNodes.Int1, Num_GroupOfNodes.Int2);
 	}
       }
     }
   }
 
-  *ExtendedList = Tree2List(Entity_Tr) ;  Tree_Delete(Entity_Tr) ;
+  *ExtendedList = Tree2List(Entity_Tr) ;
+  Tree_Delete(Entity_Tr) ;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -227,13 +255,12 @@ void Generate_GroupsOfEdges(List_T * InitialList,
   struct TwoInt  Num_GroupOfEdges, * Key1_P, * Key2_P ;
   List_T  * ExtendedAuxList ;
   struct Group  * GroupForSupport_P ;
-
-  Entity_Tr = Tree_Create(sizeof (struct TwoInt), fcmp_absint2) ;
+  int MultiValuedGroup = 0;
 
   switch (Type_SuppList) {
 
   case SUPPLIST_INSUPPORT :
-
+    Entity_Tr = Tree_Create(sizeof (struct TwoInt), fcmp_absint2) ;
     if (List_Nbr(InitialList)) {
       Generate_GroupsOfNodes(InitialList, &ExtendedAuxList) ;
 
@@ -285,46 +312,167 @@ void Generate_GroupsOfEdges(List_T * InitialList,
 	    if ( ! Tree_Search(Entity_Tr, &Num_GroupOfEdges) )
 	      Tree_Add(Entity_Tr, &Num_GroupOfEdges) ;
 	  }
-
 	  else {
 	    if (Key1_P && Key2_P && Key1_P->Int2 != Key2_P->Int2) {
-
 	      Num_GroupOfEdges.Int1 = - GeoElement->NumEdges[i_Entity] ;
 	      Num_GroupOfEdges.Int2 = Key1_P->Int2 ;
-	      if ( ! Tree_Search(Entity_Tr, &Num_GroupOfEdges) )
-		{
+	      if ( ! Tree_Search(Entity_Tr, &Num_GroupOfEdges) ){
 		Tree_Add(Entity_Tr, &Num_GroupOfEdges) ;
-		fprintf(stderr, "ADD 1 <========= %d %d %d\n", GeoElement->NumNodes[abs(Num_Nodes[0])-1], Num_GroupOfEdges.Int1, Num_GroupOfEdges.Int2);
-		}
-
+		fprintf(stderr, "ADD 1 <========= %d %d %d\n",
+                        GeoElement->NumNodes[abs(Num_Nodes[0])-1],
+                        Num_GroupOfEdges.Int1, Num_GroupOfEdges.Int2);
+              }
 	      Num_GroupOfEdges.Int1 = GeoElement->NumEdges[i_Entity] ;
 	      Num_GroupOfEdges.Int2 = Key2_P->Int2 ;
-	      if ( ! Tree_Search(Entity_Tr, &Num_GroupOfEdges) )
-		{
+	      if ( ! Tree_Search(Entity_Tr, &Num_GroupOfEdges) ){
 		Tree_Add(Entity_Tr, &Num_GroupOfEdges) ;
-		fprintf(stderr, "ADD 2 <========= %d %d %d \n", GeoElement->NumNodes[abs(Num_Nodes[1])-1], Num_GroupOfEdges.Int1, Num_GroupOfEdges.Int2);
-		}
-
+		fprintf(stderr, "ADD 2 <========= %d %d %d \n",
+                        GeoElement->NumNodes[abs(Num_Nodes[1])-1],
+                        Num_GroupOfEdges.Int1, Num_GroupOfEdges.Int2);
+              }
 	    }
 	  }
 	  */
-
 	}
       }
 
       List_Delete(ExtendedAuxList) ;
     }
+    *ExtendedList = Tree2List(Entity_Tr) ;
+    Tree_Delete(Entity_Tr) ;
     break ;
 
+  case SUPPLIST_NONE :
   default :
-    Msg::Error("Bad GroupsOfEdges (supplementary list missing)") ;
+    MultiValuedGroup = 1;
+    *ExtendedList = List_Create(10,10,sizeof (struct TwoInt)) ;
+    if (List_Nbr(InitialList)) {
+      Generate_GroupsOfNodes(InitialList, &ExtendedAuxList) ;
+      Nbr_Element = Geo_GetNbrGeoElements() ;
+      for (i_Element = 0 ; i_Element < Nbr_Element ; i_Element++) {
+        GeoElement = Geo_GetGeoElement(i_Element) ;
+        if (List_Search(InitialList, &GeoElement->Region, fcmp_int) ) {
+          // when generating edges of line elements, we assume that we want to
+          // keep any multiple copy that might arise, with its sign; this is
+          // required by the cohomology solver
+          if(GeoElement->Type != LINE && GeoElement->Type != LINE_2)
+            MultiValuedGroup = 0;
+          if (GeoElement->NbrEdges == 0) Geo_CreateEdgesOfElement(GeoElement) ;
+          for (i_Entity = 0 ; i_Entity < GeoElement->NbrEdges ; i_Entity++) {
+            Num_Nodes = Geo_GetNodesOfEdgeInElement(GeoElement, i_Entity) ;
+            Num_Node = GeoElement->NumNodes[abs(Num_Nodes[0])-1] ;
+            Key1_P = (struct TwoInt*)List_PQuery(ExtendedAuxList, &Num_Node, fcmp_int) ;
+            Num_Node = GeoElement->NumNodes[abs(Num_Nodes[1])-1] ;
+            Key2_P = (struct TwoInt*)List_PQuery(ExtendedAuxList, &Num_Node, fcmp_int) ;
+            if (Key1_P && Key2_P) {
+              Num_GroupOfEdges.Int1 = GeoElement->NumEdges[i_Entity] ;
+              Num_GroupOfEdges.Int2 = GeoElement->Region ;
+              List_Add(*ExtendedList, &Num_GroupOfEdges);
+            }
+          }
+        }
+      }
+      List_Delete(ExtendedAuxList) ;
+    }
+    // prune list if we are not in the "forced" multivalued case
+    if(!MultiValuedGroup) {
+      Entity_Tr = Tree_Create(sizeof (struct TwoInt), fcmp_absint2) ;
+      for (i_Entity = 0; i_Entity < List_Nbr(*ExtendedList); i_Entity++) {
+        List_Read(*ExtendedList, i_Entity, &Num_GroupOfEdges) ;
+        if (!Tree_Search(Entity_Tr, &Num_GroupOfEdges))
+          Tree_Add(Entity_Tr, &Num_GroupOfEdges) ;
+      }
+      List_Delete(*ExtendedList) ;
+      *ExtendedList = Tree2List(Entity_Tr);
+      Tree_Delete(Entity_Tr) ;
+    }
+    break;
+
   }
 
-  *ExtendedList = Tree2List(Entity_Tr) ;  Tree_Delete(Entity_Tr) ;
   /*
   for (i_Entity = 0 ; i_Entity < List_Nbr(*ExtendedList) ; i_Entity++) {
     List_Read(*ExtendedList, i_Entity, &Num_GroupOfEdges) ;
-    Msg::Info(" (%d, %d)", Num_GroupOfEdges.Int1, Num_GroupOfEdges.Int2) ;
+    Message::Info(" (%d, %d)", Num_GroupOfEdges.Int1, Num_GroupOfEdges.Int2) ;
+  }
+  */
+}
+
+/* ------------------------------------------------------------------------ */
+/*  G e n e r a t e _ G r o u p s O f F a c e s                             */
+/* ------------------------------------------------------------------------ */
+
+void Generate_GroupsOfFacets(List_T * InitialList,
+                             List_T ** ExtendedList)
+{
+  Tree_T  * Entity_Tr ;
+  struct Geo_Element  * GeoElement ;
+  int     Nbr_Element, i_Element,  i_Entity ;
+  int     * Num_Nodes, Num_Node ;
+  struct TwoInt  Num_GroupOfFacets ;
+  List_T  * ExtendedAuxList ;
+
+  int MultiValuedGroup = 1;
+
+  *ExtendedList = List_Create(10,10,sizeof (struct TwoInt)) ;
+
+  if (List_Nbr(InitialList)) {
+    Generate_GroupsOfNodes(InitialList, &ExtendedAuxList) ;
+    Nbr_Element = Geo_GetNbrGeoElements() ;
+    for (i_Element = 0 ; i_Element < Nbr_Element ; i_Element++) {
+      GeoElement = Geo_GetGeoElement(i_Element) ;
+      if (List_Search(InitialList, &GeoElement->Region, fcmp_int) ) {
+        // when generating facets of surface elements, we assume that we want to
+        // keep any multiple copy that might arise, with its sign; this is
+        // required by the cohomology solver
+        if(GeoElement->Type != TRIANGLE &&
+           GeoElement->Type != TRIANGLE_2 &&
+           GeoElement->Type != QUADRANGLE &&
+           GeoElement->Type != QUADRANGLE_2 &&
+           GeoElement->Type != QUADRANGLE_2_8N)
+          MultiValuedGroup = 0;
+        if (GeoElement->NbrEdges == 0)  Geo_CreateEdgesOfElement(GeoElement) ;
+        if (GeoElement->NbrFacets == 0) Geo_CreateFacetsOfElement(GeoElement) ;
+        for (i_Entity = 0 ; i_Entity < GeoElement->NbrFacets ; i_Entity++) {
+          Num_Nodes = Geo_GetNodesOfFacetInElement(GeoElement, i_Entity) ;
+          bool found = true;
+          int i = 0;
+          while(Num_Nodes[i]){
+            Num_Node = GeoElement->NumNodes[abs(Num_Nodes[i])-1] ;
+            if(!List_PQuery(ExtendedAuxList, &Num_Node, fcmp_int)){
+              found = false;
+              break;
+            }
+            i++;
+          }
+          if(found){
+            Num_GroupOfFacets.Int1 = GeoElement->NumFacets[i_Entity] ;
+            Num_GroupOfFacets.Int2 = GeoElement->Region ;
+            List_Add(*ExtendedList, &Num_GroupOfFacets);
+          }
+        }
+      }
+    }
+    List_Delete(ExtendedAuxList) ;
+  }
+
+  // prune list if we are not in the "forced" multivalued case
+  if(!MultiValuedGroup) {
+    Entity_Tr = Tree_Create(sizeof (struct TwoInt), fcmp_absint2) ;
+    for(i_Entity = 0; i_Entity < List_Nbr(*ExtendedList); i_Entity++) {
+      List_Read(*ExtendedList, i_Entity, &Num_GroupOfFacets) ;
+      if(!Tree_Search(Entity_Tr, &Num_GroupOfFacets))
+        Tree_Add(Entity_Tr, &Num_GroupOfFacets) ;
+    }
+    List_Delete(*ExtendedList) ;
+    *ExtendedList = Tree2List(Entity_Tr) ;
+    Tree_Delete(Entity_Tr) ;
+  }
+
+  /*
+  for (i_Entity = 0 ; i_Entity < List_Nbr(*ExtendedList) ; i_Entity++) {
+    List_Read(*ExtendedList, i_Entity, &Num_GroupOfFacets) ;
+    Message::Info(" (%d, %d)", Num_GroupOfFacets.Int1, Num_GroupOfFacets.Int2) ;
   }
   */
 }
@@ -381,23 +529,23 @@ void  Generate_Elements(List_T * InitialList,
       if (List_Search(InitialSuppList, &GeoElement->Region, fcmp_int))
 	List_Add(ExtendedSuppList, &i_Element);
     }
-    
+
     for (i_Element = 0 ; i_Element < Nbr_Element ; i_Element++) {
       GeoElement = Geo_GetGeoElement(i_Element) ;
 
       if (List_Search(InitialList, &GeoElement->Region, fcmp_int)){
 	for(i_Element2 = 0 ; i_Element2 < List_Nbr(ExtendedSuppList) ; i_Element2++){
 	  GeoElement2 = Geo_GetGeoElement(*(int*)List_Pointer(ExtendedSuppList, i_Element2)) ;
-	  
+
 	  k = 0 ;
 	  for(i_Node2 = 0 ; i_Node2 < GeoElement2->NbrNodes ; i_Node2++){
 	    for(i_Node = 0 ; i_Node < GeoElement->NbrNodes ; i_Node++){
-	      if(GeoElement2->NumNodes[i_Node2] == GeoElement->NumNodes[i_Node]) k++;	      
+	      if(GeoElement2->NumNodes[i_Node2] == GeoElement->NumNodes[i_Node]) k++;
 	    }
 	  }
 	  if(k == GeoElement2->NbrNodes){
 	    Pair.Int1 = GeoElement2->Num ; /* Number of the the element on the boundary */
-	    Pair.Int2 = i_Element ; /* Index of the element connected to all the nodes of 
+	    Pair.Int2 = i_Element ; /* Index of the element connected to all the nodes of
 				       the element on the boundary */
 	    Tree_Add(Entity_Tr, &Pair);
 	  }
@@ -420,5 +568,6 @@ void  Generate_Elements(List_T * InitialList,
 
   }
 
-  *ExtendedList = Tree2List(Entity_Tr) ;  Tree_Delete(Entity_Tr) ;
+  *ExtendedList = Tree2List(Entity_Tr) ;
+  Tree_Delete(Entity_Tr) ;
 }
