@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -29,7 +29,7 @@ void F_InterpolationLinear(F_ARG)
   xp = A->Val[0] ;
 
   if (xp < x[0]) {
-    Msg::Error("Bad argument for linear interpolation (%g < %g)", xp, x[0]) ;
+    Message::Error("Bad argument for linear interpolation (%g < %g)", xp, x[0]) ;
   }
   else if (xp > x[N-1]) {
     a = (y[N-1] - y[N-2]) / (x[N-1] - x[N-2]) ;
@@ -41,7 +41,15 @@ void F_InterpolationLinear(F_ARG)
     yp =  y[up] + ( xp - x[up] ) * a ;
   }
 
-  V->Val[0] = yp ;
+  if (Current.NbrHar == 1)
+    V->Val[0] = yp ;
+  else if (Current.NbrHar == 2) {
+    V->Val[0] = yp ;
+    V->Val[1] = 0. ;
+  }
+  else {
+    Message::Error("Function 'Interpolation' not valid for Complex");
+  }
   V->Type = SCALAR ;
 }
 
@@ -59,7 +67,7 @@ void F_dInterpolationLinear(F_ARG)
   xp = A->Val[0] ;
 
   if (xp < x[0]) {
-    Msg::Error("Bad argument for linear Interpolation (%g < %g)", xp, x[0]) ;
+    Message::Error("Bad argument for linear Interpolation (%g < %g)", xp, x[0]) ;
   }
   else if (xp > x[N-1]) {
     dyp = (y[N-1] - y[N-2]) / (x[N-1] - x[N-2]) ;
@@ -69,7 +77,15 @@ void F_dInterpolationLinear(F_ARG)
     dyp = (y[up] - y[lo]) / (x[up] - x[lo]) ;
   }
 
-  V->Val[0] = dyp ;
+  if (Current.NbrHar == 1)
+    V->Val[0] = dyp ;
+  else if (Current.NbrHar == 2) {
+    V->Val[0] = dyp ;
+    V->Val[1] = 0. ;
+  }
+  else {
+    Message::Error("Function 'dInterpolation' not valid for Complex");
+  }
   V->Type = SCALAR ;
 }
 
@@ -82,7 +98,7 @@ void F_dInterpolationLinear2(F_ARG)
   if (!Fct->Active) {
     Fi_InitListXY (Fct, A, V) ;
     Fi_InitListXY2 (Fct, A, V) ;
-  }    
+  }
 
   D = Fct->Active ;  N = D->Case.Interpolation.NbrPoint ;
   x = D->Case.Interpolation.xc ;  y = D->Case.Interpolation.yc ;
@@ -90,7 +106,7 @@ void F_dInterpolationLinear2(F_ARG)
   xp = A->Val[0] ;
 
   if (xp < x[0]) {
-    Msg::Error("Bad argument for linear interpolation (%g < %g)", xp, x[0]) ;
+    Message::Error("Bad argument for linear interpolation (%g < %g)", xp, x[0]) ;
   }
   else if (xp > x[N-1]) {
     a = (y[N-1] - y[N-2]) / (x[N-1] - x[N-2]) ;
@@ -104,8 +120,12 @@ void F_dInterpolationLinear2(F_ARG)
 
   if (Current.NbrHar == 1)
     V->Val[0] = yp ;
+  else if (Current.NbrHar == 2) {
+    V->Val[0] = yp ;
+    V->Val[1] = 0. ;
+  }
   else {
-    Msg::Error("Function 'Interpolation' not valid for Complex");
+    Message::Error("Function 'dInterpolation' not valid for Complex");
   }
   V->Type = SCALAR ;
 }
@@ -127,7 +147,7 @@ void F_InterpolationAkima(F_ARG)
   xp = A->Val[0] ;
 
   if (xp < x[0]) {
-    Msg::Error("Bad argument for linear interpolation (%g < %g)", xp, x[0]) ;
+    Message::Error("Bad argument for linear interpolation (%g < %g)", xp, x[0]) ;
   }
   else if (xp > x[N-1]) {
     a = (y[N-1] - y[N-2]) / (x[N-1] - x[N-2]) ;
@@ -144,8 +164,12 @@ void F_InterpolationAkima(F_ARG)
 
   if (Current.NbrHar == 1)
     V->Val[0] = yp ;
+  else if (Current.NbrHar == 2) {
+    V->Val[0] = yp ;
+    V->Val[1] = 0. ;
+  }
   else {
-    Msg::Error("Function 'Interpolation' not valid for Complex");
+    Message::Error("Function 'Interpolation' not valid for Complex");
   }
   V->Type = SCALAR ;
 }
@@ -167,14 +191,14 @@ void F_dInterpolationAkima(F_ARG)
   xp = A->Val[0] ;
 
   if (xp < x[0]) {
-    Msg::Error("Bad argument for linear interpolation (%g < %g)", xp, x[0]) ;
+    Message::Error("Bad argument for linear interpolation (%g < %g)", xp, x[0]) ;
   }
   else if (xp > x[N-1]) {
     dyp = (y[N-1] - y[N-2]) / (x[N-1] - x[N-2]) ;
   }
   else {
     up = 0 ;  while (x[++up] < xp) ;  lo = up - 1 ;
-    a = xp - x[lo] ; a2 = a*a ; 
+    a = xp - x[lo] ; a2 = a*a ;
     dyp = D->Case.Interpolation.bi[lo]
       + D->Case.Interpolation.ci[lo] * 2. * a
       + D->Case.Interpolation.di[lo] * 3. * a2 ;
@@ -182,12 +206,165 @@ void F_dInterpolationAkima(F_ARG)
 
   if (Current.NbrHar == 1)
     V->Val[0] = dyp ;
+  else if (Current.NbrHar == 2) {
+    V->Val[0] = dyp ;
+    V->Val[1] = 0. ;
+  }
   else {
-    Msg::Error("Function 'Interpolation' not valid for Complex");
+    Message::Error("Function 'dInterpolation' not valid for Complex");
   }
   V->Type = SCALAR ;
 }
 
+
+bool Fi_InterpolationBilinear(double *x, double *y, double *M, int NL, int NC,
+                              double xp, double yp, double *zp)
+{
+  double a11, a12, a21, a22;
+  int i, j;
+
+  // Interpolate point (xp,yp) in a regular grid
+  // x[i] <= xp < x[i+1]
+  // y[j] <= yp < y[j+1]
+
+  *zp = 0.0 ;
+
+  // When (xp,yp) lays outside the boundaries of the table:
+  // the nearest border is taken
+  if (xp < x[0]) xp = x[0];
+  else if (xp > x[NL-1]) xp = x[NL-1];
+  for (i=0 ; i<NL-1 ; ++i) if (x[i+1] >= xp  &&  xp >= x[i]) break;
+  i = (i >= NL) ? NL-1 : i;
+
+  if (yp < y[0]) yp = y[0];
+  else if (yp > y[NC-1]) yp = y[NC-1];
+  for (j=0 ; j<NC-1 ; ++j) if (y[j+1] >= yp  &&  yp >= y[j]) break;
+  j = (j >= NC) ? NC-1 : j;
+
+
+  a11 = M[   i  + NL * j    ];
+  a21 = M[(i+1) + NL * j    ];
+  a12 = M[   i  + NL * (j+1)];
+  a22 = M[(i+1) + NL * (j+1)];
+
+  *zp = 1/((x[i+1]-x[i])*(y[j+1]-y[j])) *
+    ( a11 * ( x[i+1]-xp) * ( y[j+1]-yp) +
+      a21 * (-x[i  ]+xp) * ( y[j+1]-yp) +
+      a12 * ( x[i+1]-xp) * (-y[j  ]+yp) +
+      a22 * (-x[i  ]+xp) * (-y[j  ]+yp) );
+
+  return true ;
+}
+
+
+void F_InterpolationBilinear(F_ARG)
+{
+/*
+    It performs a bilinear interpolation at point (xp,yp) based
+    on a two-dimensional table (sorted grid).
+
+    Input parameters:
+    NL  Number of lines
+    NC  Number of columns
+    x   values (ascending order) linked to the NL lines of the table
+    y   values (ascending order) linked to the NC columns of the table
+    M   Matrix M(x,y) = M[x+NL*y]
+
+    xp  x coordinate of interpolation point
+    yp  y coordinate of interpolation point
+
+    R. Scorretti
+*/
+
+  int     NL, NC;
+  double  xp, yp, zp = 0., *x, *y, *M;
+  struct FunctionActive  * D;
+
+  if( (A+0)->Type != SCALAR || (A+1)->Type != SCALAR)
+    Message::Error("Two Scalar arguments required!");
+
+  if (!Fct->Active)  Fi_InitListMatrix (Fct, A, V) ;
+
+  D = Fct->Active ;
+  NL = D->Case.ListMatrix.NbrLines ;
+  NC = D->Case.ListMatrix.NbrColumns ;
+
+  x = D->Case.ListMatrix.x ;
+  y = D->Case.ListMatrix.y ;
+  M = D->Case.ListMatrix.data ;
+
+  xp = (A+0)->Val[0] ;
+  yp = (A+1)->Val[0] ;
+
+  bool IsInGrid = Fi_InterpolationBilinear (x, y, M, NL, NC, xp, yp, &zp);
+  if (!IsInGrid) Message::Error("Extrapolation not allowed (xp=%g ; yp=%g)", xp, yp) ;
+
+  V->Type = SCALAR ;
+  V->Val[0] = zp ;
+}
+
+void Fi_InitListMatrix(F_ARG)
+{
+  int     i=0, k, NL, NC, sz ;
+  struct FunctionActive  * D ;
+
+  /*
+    The original table structure:
+          |  y(1)         y(2)         ...     y(NC)
+    ------+--------------------------------------------
+    x(1)  |  data(1)    data(NL+1)     ...      .
+    x(2)  |  data(2)    data(NL+2)              .
+    .     .             .              .
+    .     .             .
+    x(NL) |  data(NL)   data(2*NL)  ...     data(NL*NC)
+
+    is furnished with the following format:
+    [ NL, NC, x(1..NL), y(1..NC), data(1..NL*NC) ]
+
+    R. Scorretti
+ */
+
+  D = Fct->Active =
+    (struct FunctionActive *)Malloc(sizeof(struct FunctionActive)) ;
+
+  NL = Fct->Para[i++];
+  NC = Fct->Para[i++];
+
+  sz = 2 + NL + NC + NL*NC ; // expected size of list matrix
+  if (Fct->NbrParameters != sz)
+    Message::Error("Bad size of input data (expected = %d ; found = %d). "
+                   "List with format: x(NbrLines=%d), y(NbrColumns=%d), "
+                   "matrix(NbrLines*NbrColumns=%d)",
+                   sz, Fct->NbrParameters, NL, NC, NL*NC);
+
+  // Initialize structure and allocate memory
+  D->Case.ListMatrix.NbrLines = NL;
+  D->Case.ListMatrix.NbrColumns = NC;
+  D->Case.ListMatrix.x = (double *) malloc (sizeof(double)*NL);
+  D->Case.ListMatrix.y = (double *) malloc (sizeof(double)*NC);
+  D->Case.ListMatrix.data = (double *) malloc (sizeof(double)*NL*NC);
+
+  // Assign values
+  for (k=0 ; k<NL ; ++k) D->Case.ListMatrix.x[k] = Fct->Para[i++];
+  for (k=0 ; k<NC ; ++k) D->Case.ListMatrix.y[k] = Fct->Para[i++];
+  for (k=0 ; k<NL*NC ; ++k) D->Case.ListMatrix.data[k] = Fct->Para[i++];
+}
+
+void Fi_InitListX(F_ARG)
+{
+  int     i, N ;
+  double  *x ;
+  struct FunctionActive  * D ;
+
+  D = Fct->Active =
+    (struct FunctionActive *)Malloc(sizeof(struct FunctionActive)) ;
+  N = D->Case.Interpolation.NbrPoint = Fct->NbrParameters ;
+  x = D->Case.Interpolation.x = (double *)Malloc(sizeof(double)*N) ;
+
+  for (i = 0 ; i < N ; i++)
+    x[i] = Fct->Para[i] ;
+}
+
 void Fi_InitListXY(F_ARG)
 {
   int     i, N ;
@@ -267,37 +444,6 @@ void Fi_InitAkima(F_ARG)
   }
 }
 
-void F_InterpolationMatrix(F_ARG)
-{
-  int i, j, N, NbrLines, NbrColumns;
-  double xp;
-  double * Matrix;
-
-  N = Fct->NbrParameters;
-  if (N <= 2) Msg::Error("Bad number of parameters for matrix interpolation (%d)", N) ;
-
-  NbrLines   = (int)(Fct->Para[0]+0.5);
-  NbrColumns = (int)(Fct->Para[1]+0.5);
-  if (N-2 != NbrLines*NbrColumns)
-    Msg::Error("Bad number of parameters for matrix interpolation (%d+2 instead of %d+2)",
-	       N-2, NbrLines*NbrColumns) ;
-
-  Matrix = Fct->Para+2;
-
-  xp = A->Val[0] ;
- 
-  fprintf(stderr, "\n");
-  for (i = 0 ; i < NbrLines ; i++) {
-    fprintf(stderr, "  Line %d :", i);
-    for (j = 0 ; j < NbrColumns ; j++) {
-      fprintf(stderr, " %g", *(Matrix+i*NbrColumns+j));
-    }
-    fprintf(stderr, "\n");
-  }
-
-  V->Val[0] = 1. ;
-  V->Type = SCALAR ;
-}
 
 struct IntDouble { int Int; double Double; } ;
 struct IntVector { int Int; double Double[3]; } ;
@@ -315,7 +461,7 @@ void F_ValueFromIndex (F_ARG)
     List_PQuery(D->Case.ValueFromIndex.Table, &Current.NumEntity, fcmp_int);
 
   if (!IntDouble_P)
-    Msg::Error("Unknown Entity Index (%d) in ValueFromIndex Table",
+    Message::Error("Unknown Entity Index (%d) in ValueFromIndex Table",
                Current.NumEntity);
   /*
   printf("==> search %d --> found %g\n", Current.NumEntity, IntDouble_P->Double);
@@ -332,7 +478,7 @@ void F_VectorFromIndex(F_ARG)
   if (!Fct->Active)  Fi_InitVectorFromIndex (Fct, A, V) ;
 
   D = Fct->Active ;
-  
+
   /* printf("%d \n", List_Nbr(D->Case.ValueFromIndex.Table)); */
 
   if (List_Nbr(D->Case.ValueFromIndex.Table)){
@@ -340,8 +486,8 @@ void F_VectorFromIndex(F_ARG)
     IntVector_P = (struct IntVector *)
       List_PQuery(D->Case.ValueFromIndex.Table, &Current.NumEntity, fcmp_int);
 
-    /* 
-       if (!IntVector_P)  Msg::Error("Unknown Entity Index in VectorFromIndex Table");
+    /*
+       if (!IntVector_P)  Message::Error("Unknown Entity Index in VectorFromIndex Table");
        printf("==> search %d --> found %g\n", Current.NumEntity, IntVector_P->Double);
     */
 
@@ -349,15 +495,15 @@ void F_VectorFromIndex(F_ARG)
     V->Val[1] = IntVector_P->Double[1] ;
     V->Val[2] = IntVector_P->Double[2] ;
     V->Type = VECTOR;
-  }    
+  }
   else{
     V->Val[0] = 0.;
     V->Val[1] = 0.;
     V->Val[2] = 0.;
-    V->Type = VECTOR; 
+    V->Type = VECTOR;
     /* WARNING, "Table empty: Uninitialized data or Unknown Entity
        Index in VectorFromIndex Table  */
-  }    
+  }
 }
 
 void Fi_InitValueFromIndex(F_ARG)
@@ -367,7 +513,7 @@ void Fi_InitValueFromIndex(F_ARG)
   struct FunctionActive  * D ;
 
   N = (int)Fct->Para[0];
-  
+
   D = Fct->Active =
     (struct FunctionActive *)Malloc(sizeof(struct FunctionActive)) ;
 
@@ -389,12 +535,12 @@ void Fi_InitVectorFromIndex(F_ARG)
   if ((Fct->NbrParameters)){
 
     N = (int)Fct->Para[0];
-      
+
     D = Fct->Active =
       (struct FunctionActive *)Malloc(sizeof(struct FunctionActive)) ;
 
     D->Case.ValueFromIndex.Table = List_Create(N, 1, sizeof(struct IntVector[3]));
-    
+
     for (i = 0 ; i < N ; i++) {
       IntVector_s.Int = (int)(Fct->Para[i*4+1]+0.1);
       IntVector_s.Double[0] = Fct->Para[i*4+2];
