@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -111,12 +111,12 @@ void GF_LaplacexForm(GF_ARGX)
 	Val->Val[0] = - ONE_OVER_FOUR_PI * I1 ; 
       }
       else {
-	Msg::Error("Unknown Basis Function Type for 'GF_LaplacexForm'");
+	Message::Error("Unknown Basis Function Type for 'GF_LaplacexForm'");
       }
       break ;
       
     default :
-      Msg::Error("Unknown Element Type (%s) for 'GF_LaplacexForm'",
+      Message::Error("Unknown Element Type (%s) for 'GF_LaplacexForm'",
 		 Get_StringForDefine(Element_Type, Element->ElementSource->Type));
     }
     
@@ -224,7 +224,7 @@ void GF_LaplacexForm(GF_ARGX)
 	if(r00 <= EPSILON*(fabs(s0m)+fabs(s0p)) ){ f20 = log(s0m/s0p) ; B0 = 0; }
 	else{
 	  if (!(r0m + s0m)) 
-	    Msg::Error("1/0 in GF_LaplacexForm (case _3D TRIANGLE) Num %d Obs %.15e %.15e %.15e",
+	    Message::Error("1/0 in GF_LaplacexForm (case _3D TRIANGLE) Num %d Obs %.15e %.15e %.15e",
 		       Element->ElementSource->Num, x, y, z) ;
 	  f20 = log((r0p + s0p) / (r0m + s0m));
 	  B0  = atan(t00*s0p/(r00_2+fabs(zl)*r0p))-atan(t00*s0m/(r00_2+fabs(zl)*r0m));
@@ -233,7 +233,7 @@ void GF_LaplacexForm(GF_ARGX)
 	if(r10 <= EPSILON*(fabs(s1m)+fabs(s1p)) ){ f21 = log(s1m/s1p); B1 = 0; }
 	else{
 	  if(!(r0p + s1m)) 
-	    Msg::Error("1/0 in GF_LaplacexForm (case _3D TRIANGLE) Num %d Obs %.15e %.15e %.15e",
+	    Message::Error("1/0 in GF_LaplacexForm (case _3D TRIANGLE) Num %d Obs %.15e %.15e %.15e",
 		       Element->ElementSource->Num, x, y, z) ;
 	  f21 = log((r1p + s1p) / (r0p + s1m));
 	  B1 =  atan(t10*s1p/(r10_2+fabs(zl)*r1p))-atan(t10*s1m/(r10_2+fabs(zl)*r0p));
@@ -242,7 +242,7 @@ void GF_LaplacexForm(GF_ARGX)
 	if(r20 <= EPSILON*(fabs(s2m)+fabs(s2p)) ){ f22 = log(s2m/s2p); B2 = 0; }
 	else{
 	  if(!(r1p+s2m))
-	    Msg::Error("1/0 in GF_LaplacexForm (case _3D TRIANGLE) Num %d Obs %.15e %.15e %.15e",
+	    Message::Error("1/0 in GF_LaplacexForm (case _3D TRIANGLE) Num %d Obs %.15e %.15e %.15e",
 		       Element->ElementSource->Num, x, y, z) ;
 	  f22 = log((r0m + s2p) / (r1p + s2m));
 	  B2 = atan(t20*s2p/(r20_2+fabs(zl)*r0m))-atan(t20*s2m/(r20_2+fabs(zl)*r1p));
@@ -262,7 +262,7 @@ void GF_LaplacexForm(GF_ARGX)
 
       case 2 : /* BF_Node */
 	if (!v2) 	
-	  Msg::Error("1/0 in GF_LaplacexForm (case _3D TRIANGLE) v2 %e", v2);
+	  Message::Error("1/0 in GF_LaplacexForm (case _3D TRIANGLE) v2 %e", v2);
 
 	f30 = (s0p*r0p-s0m*r0m) + r00_2 * f20 ; /* f3i */
 	f31 = (s1p*r1p-s1m*r0p) + r10_2 * f21 ;
@@ -305,7 +305,7 @@ void GF_LaplacexForm(GF_ARGX)
 	}
 	break;
       default :
-	Msg::Error("Unknown Basis Function Type for 'GF_LaplacexForm'");
+	Message::Error("Unknown Basis Function Type for 'GF_LaplacexForm'");
       }
 
       Val->Val[0] *= ONE_OVER_FOUR_PI ;
@@ -314,13 +314,13 @@ void GF_LaplacexForm(GF_ARGX)
       break ;
     
     default :
-      Msg::Error("Unknown Element Type (%s) for 'GF_LaplacexForm'",
+      Message::Error("Unknown Element Type (%s) for 'GF_LaplacexForm'",
 	  Get_StringForDefine(Element_Type, Element->ElementSource->Type));
     }
     break ;
 
   default :
-    Msg::Error("Unknown Dimension (%d) for 'GF_LaplacexForm'", 
+    Message::Error("Unknown Dimension (%d) for 'GF_LaplacexForm'", 
 	(int)Fct->Para[0]);
     
   }
@@ -375,7 +375,7 @@ void GF_GradLaplacexForm(GF_ARGX)
       break ;
       
     default :
-      Msg::Error("Unknown Element Type (%s) for 'GF_GradLaplacexForm'",
+      Message::Error("Unknown Element Type (%s) for 'GF_GradLaplacexForm'",
 	  Get_StringForDefine(Element_Type, Element->ElementSource->Type));
     }
     break ;
@@ -530,7 +530,7 @@ void GF_GradLaplacexForm(GF_ARGX)
     
       case 2 : /* BF_Node */
 	if (!v2 ) 	
-	  Msg::Error("1/0 in GF_LaplacexForm (case _3D TRIANGLE) v2 %e", v2);
+	  Message::Error("1/0 in GF_LaplacexForm (case _3D TRIANGLE) v2 %e", v2);
 
 	f0[0] = s0[0] * t00 * f20 - m0[0]*(r0p-r0m) ; /* fi */
 	f0[1] = s0[1] * t00 * f20 - m0[1]*(r0p-r0m) ;
@@ -601,13 +601,13 @@ void GF_GradLaplacexForm(GF_ARGX)
       break ;
       
     default :
-      Msg::Error("Unknown Element Type (%s) for 'GF_GradLaplacexForm'",
+      Message::Error("Unknown Element Type (%s) for 'GF_GradLaplacexForm'",
 	  Get_StringForDefine(Element_Type, Element->ElementSource->Type));
     }
     break ;
     
   default :
-    Msg::Error("Unknown Dimension (%d) for 'GF_GradLaplacexForm'",
+    Message::Error("Unknown Dimension (%d) for 'GF_GradLaplacexForm'",
 	(int)Fct->Para[0]);
     
   }
@@ -641,7 +641,7 @@ void GF_NPxGradLaplacexForm(GF_ARGX)
 
     case LINE :	  
       if (Element->Type != LINE)
-	Msg::Error("GF_NPxGradLaplacexForm not ready for mixed geometrical elements");
+	Message::Error("GF_NPxGradLaplacexForm not ready for mixed geometrical elements");
 
       xs[0] = Element->ElementSource->x[0] ; ys[0] = Element->ElementSource->y[0] ;
       xs[1] = Element->ElementSource->x[1] ; ys[1] = Element->ElementSource->y[1] ;
@@ -665,14 +665,14 @@ void GF_NPxGradLaplacexForm(GF_ARGX)
 	switch (Type_Int) {	  
 	case 1 :  
 	case 2 :  
-	  Msg::Error("Degenerate case not done in 'GF_NPxGradLaplacexForm'");
+	  Message::Error("Degenerate case not done in 'GF_NPxGradLaplacexForm'");
 	  break ;	
 	case 3 :  
 	  if (fabs(d) < EPSILON2) {
 	    I1 = 0.0 ;
 	  }
 	  else {
-	    if(d<0) Msg::Error("Unexpected value in 'GF_NPxGradLaplacexForm'");
+	    if(d<0) Message::Error("Unexpected value in 'GF_NPxGradLaplacexForm'");
 	    i1 = sqrt(d) ;
 	    Is = 2. / i1 * (atan((2.*a+b)/i1) -  atan(b/i1)) ;
 	    Jp = sqrt(SQU(xp[0]-xp[1])+SQU(yp[0]-yp[1])) ;
@@ -685,12 +685,12 @@ void GF_NPxGradLaplacexForm(GF_ARGX)
 	Val->Val[0] = - ONE_OVER_TWO_PI * I1 ; 
       }
       else {	
-	Msg::Error("Unknown Basis Function Type for 'GF_NPxGradLaplacexForm'");
+	Message::Error("Unknown Basis Function Type for 'GF_NPxGradLaplacexForm'");
       }
       break ;
 
     default :
-      Msg::Error("Unknown Element Type (%s) for 'GF_NPxGradLaplacexForm'",
+      Message::Error("Unknown Element Type (%s) for 'GF_NPxGradLaplacexForm'",
 	  Get_StringForDefine(Element_Type, Element->ElementSource->Type));     
     }
     break ;
@@ -706,13 +706,13 @@ void GF_NPxGradLaplacexForm(GF_ARGX)
       Val->Val[0] = N[0]*ValGrad.Val[0] + N[1]*ValGrad.Val[1] + N[2]*ValGrad.Val[2] ;
       break ;
     default :
-      Msg::Error("Unknown Element Type (%s) for 'GF_NPxGradLaplacexForm'",
+      Message::Error("Unknown Element Type (%s) for 'GF_NPxGradLaplacexForm'",
 	  Get_StringForDefine(Element_Type, Element->ElementSource->Type));  
     }
     break ;
 
   default :
-    Msg::Error("Unknown Dimension (%d) for 'GF_NPxGradLaplacexForm'",
+    Message::Error("Unknown Dimension (%d) for 'GF_NPxGradLaplacexForm'",
 	(int)Fct->Para[0]);
   }
 }
@@ -725,7 +725,7 @@ void GF_NPxGradLaplacexForm(GF_ARGX)
 
 void GF_NSxGradLaplacexForm(GF_ARGX)
 {
-  Msg::Error("Not done: 'GF_NSxGradLaplacexForm'");
+  Message::Error("Not done: 'GF_NSxGradLaplacexForm'");
 }
 
 /* ------------------------------------------------------------------------ */
@@ -741,7 +741,7 @@ void GF_ApproximateLaplacexForm(GF_ARGX)
     break ;
 
   default :
-    Msg::Error("Bad Parameter Value in 'GF_ApproximateLaplacexForm'");
+    Message::Error("Bad Parameter Value in 'GF_ApproximateLaplacexForm'");
     break;
 
   }
