@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -18,8 +18,8 @@ void Gauss_Pyramid(int Nbr_Points, int Num,
     break ;
 
   default : 
-    Msg::Error("Wrong number of Gauss points for Pyramid");
-    Msg::Error("Valid choice: 8");
+    Message::Error("Wrong number of Gauss points for Pyramid: "
+                   "valid choice: 8");
     break;
   }
 }
