@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -10,7 +10,7 @@
 #include "MallocUtils.h"
 #include "Message.h"
 
-#define SQU(a)     ((a)*(a)) 
+#define SQU(a)     ((a)*(a))
 
 extern double Flag_ORDER ;
 
@@ -21,7 +21,7 @@ struct GeoData  * CurrentGeoData ;
 static void swapBytes(char *array, int size, int n)
 {
   int i, c;
-  
+
   char *x = (char *)Malloc(size * sizeof(char)) ;
   for(i = 0; i < n; i++) {
     char *a = &array[i * size];
@@ -32,6 +32,18 @@ static void swapBytes(char *array, int size, int n)
   Free(x);
 }
 
+static std::string ExtractDoubleQuotedString(const char *str, int len)
+{
+  char *c = strstr((char*)str, "\"");
+  if(!c) return "";
+  std::string ret;
+  for(int i = 1; i < len; i++) {
+    if(c[i] == '"' || c[i] == EOF || c[i] == '\n' || c[i] == '\r') break;
+    ret.push_back(c[i]);
+  }
+  return ret;
+}
+
 /* ------------------------------------------------------------------------ */
 /*  G e o _ A d d G e o D a t a                                             */
 /* ------------------------------------------------------------------------ */
@@ -46,7 +58,7 @@ int Geo_AddGeoData(List_T * GeoData_L,
   if (!Name_MshFile)  Name_MshFile = Name_DefaultMshFile ;
 
   if ((i = List_ISearchSeq(GeoData_L, Name_MshFile, fcmp_GeoData_Name)) < 0) {
-    Msg::Info("Loading Geometric data '%s'", Name_MshFile) ;
+    Message::Info("Loading Geometric data '%s'", Name_MshFile) ;
     i = List_Nbr(GeoData_L) ;
     Geo_InitGeoData(&GeoData_S, i, Name_MshFile) ;
     Geo_OpenFile(Name_MshFile, "r") ;
@@ -55,7 +67,7 @@ int Geo_AddGeoData(List_T * GeoData_L,
 
     if (!Name_AdaptFile) Name_AdaptFile = Name_DefaultAdaptFile ;
     if (Name_AdaptFile) {
-      Msg::Info("Loading Adaptation data '%s'", Name_AdaptFile) ;
+      Message::Info("Loading Adaptation data '%s'", Name_AdaptFile) ;
       Geo_OpenFile(Name_AdaptFile, "r") ;
       Geo_SetCurrentGeoData(&GeoData_S) ;
       Geo_ReadFileAdapt(&GeoData_S) ;
@@ -119,7 +131,7 @@ void Geo_OpenFile(char * Name, const char * Mode)
 {
   File_GEO = fopen(Name, Mode) ;
 
-  if (!File_GEO) Msg::Error("Unable to open file '%s'", Name);
+  if (!File_GEO) Message::Error("Unable to open file '%s'", Name);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -144,24 +156,24 @@ int Geo_GetElementType(int Format, int Type)
     case 1  : return(LINE) ;
     case 2  : return(TRIANGLE) ;
     case 3  : return(QUADRANGLE) ;
-    case 4  : return(TETRAHEDRON) ;      
+    case 4  : return(TETRAHEDRON) ;
     case 5  : return(HEXAHEDRON) ;
     case 6  : return(PRISM) ;
     case 7  : return(PYRAMID) ;
     case 8  : return(LINE_2) ;
     case 9  : return(TRIANGLE_2) ;
     case 10 : return(QUADRANGLE_2) ;
-    case 11 : return(TETRAHEDRON_2) ;      
+    case 11 : return(TETRAHEDRON_2) ;
     case 12 : return(HEXAHEDRON_2) ;
     case 13 : return(PRISM_2) ;
     case 14 : return(PYRAMID_2) ;
     case 16 : return(QUADRANGLE_2_8N) ;
-    default : Msg::Error("Unkown type of Element in Gmsh format (%d)", 
+    default : Message::Error("Unkown type of Element in Gmsh format (%d)",
 			 FORMAT_GMSH) ; return(-1) ;
     }
     break ;
   default :
-    Msg::Error("Unkown mesh format (%d)", Format) ;
+    Message::Error("Unkown mesh format (%d)", Format) ;
     return(-1) ;
   }
 }
@@ -175,23 +187,23 @@ int Geo_GetElementTypeInv(int Format, int Type)
     case LINE          : return(1) ;
     case TRIANGLE      : return(2) ;
     case QUADRANGLE    : return(3) ;
-    case TETRAHEDRON   : return(4) ;      
+    case TETRAHEDRON   : return(4) ;
     case HEXAHEDRON    : return(5) ;
     case PRISM         : return(6) ;
     case PYRAMID       : return(7) ;
     case LINE_2        : return(8) ;
     case TRIANGLE_2    : return(9) ;
     case QUADRANGLE_2  : return(10) ;
-    case TETRAHEDRON_2 : return(11) ;      
+    case TETRAHEDRON_2 : return(11) ;
     case HEXAHEDRON_2  : return(12) ;
     case PRISM_2       : return(13) ;
     case PYRAMID_2     : return(14) ;
     case QUADRANGLE_2_8N: return(16) ;
-    default : Msg::Error("Unkown type of Element in Gmsh format") ; return(-1) ;
+    default : Message::Error("Unkown type of Element in Gmsh format") ; return(-1) ;
     }
     break ;
   default :
-    Msg::Error("Unkown mesh format") ;
+    Message::Error("Unkown mesh format") ;
     return(-1) ;
   }
 
@@ -204,19 +216,19 @@ int Geo_GetNbNodesPerElement(int Type)
   case LINE          : return(2) ;
   case TRIANGLE      : return(3) ;
   case QUADRANGLE    : return(4) ;
-  case TETRAHEDRON   : return(4) ;      
+  case TETRAHEDRON   : return(4) ;
   case HEXAHEDRON    : return(8) ;
   case PRISM         : return(6) ;
   case PYRAMID       : return(5) ;
   case LINE_2        : return(3) ;
   case TRIANGLE_2    : return(6) ;
   case QUADRANGLE_2  : return(9) ;
-  case TETRAHEDRON_2 : return(10) ;      
+  case TETRAHEDRON_2 : return(10) ;
   case HEXAHEDRON_2  : return(20) ;
   case PRISM_2       : return(15) ;
   case PYRAMID_2     : return(13) ;
   case QUADRANGLE_2_8N: return(8) ;
-  default : Msg::Error("Unkown type of Element") ; return(-1) ;
+  default : Message::Error("Unkown type of Element") ; return(-1) ;
   }
 }
 
@@ -227,7 +239,7 @@ void Geo_SaveMesh(struct GeoData * GeoData_P, List_T * InitialList, char * FileN
   struct Geo_Node   * Geo_Node_P ;
   struct Geo_Element  Geo_Element ;
   struct GeoData GeoData ;
-  int i, j, Type, iDummy=0;
+  int i, j, Type;
   int  fcmp_int(const void * a, const void * b);
 
   GeoData.Nodes    = List_Create(1000, 1000, sizeof(struct Geo_Node)) ;
@@ -238,15 +250,16 @@ void Geo_SaveMesh(struct GeoData * GeoData_P, List_T * InitialList, char * FileN
     if (List_Search(InitialList, &Geo_Element.Region, fcmp_int) ) {
       List_Add(GeoData.Elements, &Geo_Element) ;
       for (j = 0 ; j < Geo_Element.NbrNodes ; j++)
-	if (!List_Search(GeoData.Nodes, 
-	 		 Geo_Node_P = Geo_GetGeoNodeOfNum(Geo_Element.NumNodes[j]), fcmp_Nod) ) 
+	if (!List_Search(GeoData.Nodes,
+	 		 Geo_Node_P = Geo_GetGeoNodeOfNum(Geo_Element.NumNodes[j]),
+                         fcmp_Nod) )
 	  List_Add(GeoData.Nodes, Geo_Node_P) ;
     }
   }
- 
+
   file = fopen(FileName,"w");
-  Msg::Info("Saving mesh in file \"%s\" (%d nodes, %d elements)", 
-	    FileName, List_Nbr(GeoData.Nodes), List_Nbr(GeoData.Elements));
+  Message::Info("Saving mesh in file \"%s\" (%d nodes, %d elements)",
+                FileName, List_Nbr(GeoData.Nodes), List_Nbr(GeoData.Elements));
 
   fprintf(file, "$NOD\n%d\n", List_Nbr(GeoData.Nodes));
 
@@ -263,12 +276,12 @@ void Geo_SaveMesh(struct GeoData * GeoData_P, List_T * InitialList, char * FileN
     Type = Geo_GetElementTypeInv(FORMAT_GMSH, Geo_Element.Type) ;
     fprintf(file, "%d %d %d %d %d ",
 	    Geo_Element.Num, Type, Geo_Element.Region,
-	    iDummy, Geo_Element.NbrNodes) ;
+	    Geo_Element.Region, Geo_Element.NbrNodes) ;
     for (j = 0 ; j < Geo_Element.NbrNodes ; j++)
       fprintf(file, "%d ", Geo_Element.NumNodes[j]) ;
     fprintf(file, "\n") ;
   }
-    
+
   fprintf(file, "$ENDELM\n");
 
   fclose(file);
@@ -284,36 +297,54 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
   struct Geo_Element  Geo_Element ;
   char                String[256] ;
   int                 binary = 0, swap = 0;
-  
+
   while (1) {
 
-    do { 
-      fgets(String, sizeof(String), File_GEO) ; 
+    do {
+      fgets(String, sizeof(String), File_GEO) ;
       if (feof(File_GEO))  break ;
-    } while (String[0] != '$') ;  
-    
+    } while (String[0] != '$') ;
+
     if (feof(File_GEO))  break ;
 
     /*  F O R M A T  */
 
     if(!strncmp(&String[1], "MeshFormat", 10)) {
 
-      fscanf(File_GEO, "%lf %d %d\n", &Version, &Format, &Size);
+      fgets(String, sizeof(String), File_GEO) ;
+      if(sscanf(String, "%lf %d %d", &Version, &Format, &Size) != 3)
+        return;
       if(Version < 2.0 || Version >= 3.0){
-	Msg::Error("Unknown mesh file version (%g)", Version);
+	Message::Error("Unknown mesh file version (%g)", Version);
 	return;
       }
 
       if(Format){
 	binary = 1;
-	Msg::Info("Mesh is in binary format");
+	Message::Info("Mesh is in binary format");
 	int one;
 	if(fread(&one, sizeof(int), 1, File_GEO) != 1) return;
 	if(one != 1){
 	  swap = 1;
-	  Msg::Info("Swapping bytes from binary file");
+	  Message::Info("Swapping bytes from binary file");
 	}
       }
+    }//=====Ruth
+    else if(!strncmp(&String[1], "PhysicalNames", 13)) {
+
+      fgets(String, sizeof(String), File_GEO) ;
+      int numNames;
+      if(sscanf(String, "%d", &numNames) != 1) return ;
+      for(int i = 0; i < numNames; i++) {
+        int dim = -1, num;
+        if(Version > 2.0){
+          if(fscanf(File_GEO, "%d", &dim) != 1) return ;
+        }
+        if(fscanf(File_GEO, "%d", &num) != 1) return ;
+        fgets(String, sizeof(String), File_GEO) ;
+        std::string name = ExtractDoubleQuotedString(String, 256);
+        //if(name.size()) setPhysicalName(name, dim, num);
+      }
     }
 
     /*  N O D E S  */
@@ -322,11 +353,10 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
 	     !strncmp(&String[1], "NOD", 3) ||
 	     !strncmp(&String[1], "Nodes", 5)) {
 
-      fscanf(File_GEO, "%d", &Nbr) ;
-      if(binary) {
-	char c;
-	if(fread(&c, sizeof(char), 1, File_GEO) != 1) return;
-      }
+      fgets(String, sizeof(String), File_GEO) ;
+      if(sscanf(String, "%d", &Nbr) != 1) return;
+      Message::Debug("%d nodes", Nbr);
+
       if (GeoData_P->Nodes == NULL)
 	GeoData_P->Nodes = List_Create(Nbr, 1000, sizeof(struct Geo_Node)) ;
 
@@ -334,7 +364,7 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
 	if (!binary){
 	  fscanf(File_GEO, "%d %lf %lf %lf",
 		 &Geo_Node.Num, &Geo_Node.x, &Geo_Node.y, &Geo_Node.z) ;
-	} 
+	}
 	else {
 	  if(fread(&Geo_Node.Num, sizeof(int), 1, File_GEO) != 1) return;
 	  if(swap) swapBytes((char*)&Geo_Node.Num, sizeof(int), 1);
@@ -342,27 +372,27 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
 	  if(fread(xyz, sizeof(double), 3, File_GEO) != 3) return;
 	  if(swap) swapBytes((char*)xyz, sizeof(double), 3);
 	  Geo_Node.x = xyz[0];
-	  Geo_Node.y = xyz[1]; 
+	  Geo_Node.y = xyz[1];
 	  Geo_Node.z = xyz[2];
 	}
 	List_Add(GeoData_P->Nodes, &Geo_Node) ;
 	if(!i){
 	  GeoData_P->Xmin = GeoData_P->Xmax = Geo_Node.x;
 	  GeoData_P->Ymin = GeoData_P->Ymax = Geo_Node.y;
-	  GeoData_P->Zmin = GeoData_P->Zmax = Geo_Node.z; 
+	  GeoData_P->Zmin = GeoData_P->Zmax = Geo_Node.z;
 	}
 	else{
 	  GeoData_P->Xmin = std::min(GeoData_P->Xmin, Geo_Node.x);
 	  GeoData_P->Xmax = std::max(GeoData_P->Xmax, Geo_Node.x);
 	  GeoData_P->Ymin = std::min(GeoData_P->Ymin, Geo_Node.y);
 	  GeoData_P->Ymax = std::max(GeoData_P->Ymax, Geo_Node.y);
-	  GeoData_P->Zmin = std::min(GeoData_P->Zmin, Geo_Node.z); 
-	  GeoData_P->Zmax = std::max(GeoData_P->Zmax, Geo_Node.z); 
+	  GeoData_P->Zmin = std::min(GeoData_P->Zmin, Geo_Node.z);
+	  GeoData_P->Zmax = std::max(GeoData_P->Zmax, Geo_Node.z);
 	}
       }
 
-      if(GeoData_P->Xmin != GeoData_P->Xmax && 
-	 GeoData_P->Ymin != GeoData_P->Ymax && 
+      if(GeoData_P->Xmin != GeoData_P->Xmax &&
+	 GeoData_P->Ymin != GeoData_P->Ymax &&
 	 GeoData_P->Zmin != GeoData_P->Zmax)
 	GeoData_P->Dimension = _3D;
       else if(GeoData_P->Xmin != GeoData_P->Xmax && GeoData_P->Ymin != GeoData_P->Ymax)
@@ -379,20 +409,20 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
 	GeoData_P->Dimension = _1D;
       else
 	GeoData_P->Dimension = _0D;
-      
-      GeoData_P->CharacteristicLength = 
-	sqrt(SQU(GeoData_P->Xmax - GeoData_P->Xmin) + 
-	     SQU(GeoData_P->Ymax - GeoData_P->Ymin) + 
+
+      GeoData_P->CharacteristicLength =
+	sqrt(SQU(GeoData_P->Xmax - GeoData_P->Xmin) +
+	     SQU(GeoData_P->Ymax - GeoData_P->Ymin) +
 	     SQU(GeoData_P->Zmax - GeoData_P->Zmin));
-      if(!GeoData_P->CharacteristicLength) 
+      if(!GeoData_P->CharacteristicLength)
 	GeoData_P->CharacteristicLength = 1.;
     }
 
     else if (!strncmp(&String[1], "ParametricNodes", 15)) {
-      
-      Msg::Error("Cannot read parametric nodes: please save mesh file with standard "
+
+      Message::Error("Cannot read parametric nodes: please save mesh file with standard "
                  "nodes instead!");
-      
+
     }
 
     /*  E L E M E N T S  */
@@ -400,11 +430,9 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
     else if (!strncmp(&String[1], "ELM", 3) ||
 	     !strncmp(&String[1], "Elements", 8)) {
 
-      fscanf(File_GEO, "%d", &Nbr) ;
-      if(binary) {
-	char c;
-	if(fread(&c, sizeof(char), 1, File_GEO) != 1) return;
-      }
+      fgets(String, sizeof(String), File_GEO) ;
+      if(sscanf(String, "%d", &Nbr) != 1) return;
+      Message::Debug("%d elements", Nbr);
 
       if (GeoData_P->Elements == NULL)
 	GeoData_P->Elements =
@@ -418,16 +446,18 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
 	  if(Version == 1.0){
 	    fscanf(File_GEO, "%d %d %d %d %d",
 		   &Geo_Element.Num, &Type, &Geo_Element.Region,
-		   &iDummy, &Geo_Element.NbrNodes) ;
+		   &Geo_Element.ElementaryRegion, &Geo_Element.NbrNodes) ;
 	    Geo_Element.Type = Geo_GetElementType(FORMAT_GMSH, Type) ;
 	  }
 	  else{
 	    fscanf(File_GEO, "%d %d %d", &Geo_Element.Num, &Type, &NbTags);
-	    Geo_Element.Region = 1;
+	    Geo_Element.Region = Geo_Element.ElementaryRegion = 1;
 	    for(j = 0; j < NbTags; j++){
 	      fscanf(File_GEO, "%d", &iDummy);
 	      if(j == 0)
 		Geo_Element.Region = iDummy;
+              else if(j == 1)
+                Geo_Element.ElementaryRegion = iDummy;
 	      /* ignore any other tags for now */
 	    }
 	    Geo_Element.Type = Geo_GetElementType(FORMAT_GMSH, Type) ;
@@ -439,7 +469,7 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
 
 	  List_Add(GeoData_P->Elements, &Geo_Element) ;
 	}
-      } 
+      }
       else {
 	int numElementsPartial = 0;
 	while(numElementsPartial < Nbr){
@@ -457,27 +487,25 @@ void Geo_ReadFile(struct GeoData * GeoData_P)
 	    if(fread(data, sizeof(int), n, File_GEO) != n) return;
 	    if(swap) swapBytes((char*)data, sizeof(int), n);
 	    Geo_Element.Num = data[0];
-	    Geo_Element.Region = (numTags > 0) ? data[4 - numTags] : 0;
-	    //int elementary = (numTags > 1) ? data[4 - numTags + 1] : 0;
-	    Geo_Element.NumNodes = &data[numTags + 1];
+	    Geo_Element.Region = (numTags > 0) ? data[1] : 0;
+	    Geo_Element.ElementaryRegion = (numTags > 1) ? data[2] : 0;
 	    Geo_Element.NumNodes = (int *)Malloc(Geo_Element.NbrNodes * sizeof(int)) ;
 	    for (j = 0 ; j < Geo_Element.NbrNodes ; j++)
 	      Geo_Element.NumNodes[j] = data[numTags + 1 + j] ;
-
 	    List_Add(GeoData_P->Elements, &Geo_Element) ;
 	  }
 	  Free(data);
 	  numElementsPartial += numElms;
 	}
       }
-      
+
       List_Sort(GeoData_P->Elements, fcmp_Elm) ;
-      
+
     }
 
     do {
       fgets(String, sizeof(String), File_GEO) ;
-      if (feof(File_GEO)) Msg::Error("Prematured end of file");
+      if (feof(File_GEO)) Message::Error("Prematured end of file");
     } while (String[0] != '$') ;
 
   }   /* while 1 ... */
@@ -504,11 +532,11 @@ void Geo_ReadFileAdapt(struct GeoData * GeoData_P)
 
   while (1) {
 
-    do { 
-      fgets(String, sizeof(String), File_GEO) ; 
+    do {
+      fgets(String, sizeof(String), File_GEO) ;
       if (feof(File_GEO))  break ;
-    } while (String[0] != '$') ;  
-    
+    } while (String[0] != '$') ;
+
     if (feof(File_GEO))  break ;
 
     if (!strncmp(&String[1], "Adapt", 5)) {
@@ -517,7 +545,7 @@ void Geo_ReadFileAdapt(struct GeoData * GeoData_P)
 	fscanf(File_GEO, "%d %lf %lf %lf", &Geo_Element.Num, &E, &H, &P) ;
 	if(!(Geo_Element_P = (struct Geo_Element *)
 	     List_PQuery(GeoData_P->Elements, &Geo_Element, fcmp_Elm)))
-	  Msg::Error("Element %d not found in database", Geo_Element.Num) ;
+	  Message::Error("Element %d not found in database", Geo_Element.Num) ;
 	Index_GeoElement = Geo_GetGeoElementIndex(Geo_Element_P) ;
 	GeoData_P->H[Index_GeoElement+1] = H ;
 	GeoData_P->P[Index_GeoElement+1] = P ;
@@ -527,14 +555,14 @@ void Geo_ReadFileAdapt(struct GeoData * GeoData_P)
 
     do {
       fgets(String, sizeof(String), File_GEO) ;
-      if (feof(File_GEO)) Msg::Error("Prematured end of file");
+      if (feof(File_GEO)) Message::Error("Prematured end of file");
     } while (String[0] != '$') ;
 
   }   /* while 1 ... */
 
   if(Flag_ORDER < 0) Flag_ORDER = Max_Order ;
- 
-  Msg::Info("Maximum interpolation order = %g", Flag_ORDER) ;
+
+  Message::Info("Maximum interpolation order = %g", Flag_ORDER) ;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -566,7 +594,7 @@ int Geo_GetNbrGeoElements(void)
 
 struct Geo_Element *Geo_GetGeoElement(int Index_Element)
 {
-  return((struct Geo_Element *)List_Pointer(CurrentGeoData->Elements, 
+  return((struct Geo_Element *)List_Pointer(CurrentGeoData->Elements,
 						  Index_Element)) ;
 }
 
@@ -576,7 +604,7 @@ struct Geo_Element *Geo_GetGeoElement(int Index_Element)
 
 int Geo_GetGeoElementIndex(struct Geo_Element * GeoElement)
 {
-  return(GeoElement - 
+  return(GeoElement -
 	       (struct Geo_Element*)List_Pointer(CurrentGeoData->Elements, 0)) ;
 }
 
@@ -608,7 +636,7 @@ int Geo_GetNbrGeoNodes(void){
 
 struct Geo_Node *Geo_GetGeoNode(int Index_Node)
 {
-  return((struct Geo_Node *)List_Pointer(CurrentGeoData->Nodes, 
+  return((struct Geo_Node *)List_Pointer(CurrentGeoData->Nodes,
 					       Index_Node)) ;
 }
 
@@ -641,8 +669,8 @@ void Geo_GetNodesCoordinates(int Nbr_Node, int * Num_Node,
 
     if(!(Geo_Node_P = (struct Geo_Node*)
 	 List_PQuery(CurrentGeoData->Nodes, &Geo_Node, fcmp_Nod)))
-      Msg::Error("Node %d does not exist", Geo_Node.Num) ;
-    
+      Message::Error("Node %d does not exist", Geo_Node.Num) ;
+
     x[i] = Geo_Node_P->x ;  y[i] = Geo_Node_P->y ;  z[i] = Geo_Node_P->z ;
   }
 }
@@ -662,8 +690,8 @@ void Geo_SetNodesCoordinates(int Nbr_Node, int * Num_Node,
 
     if(!(Geo_Node_P = (struct Geo_Node*)
 	 List_PQuery(CurrentGeoData->Nodes, &Geo_Node, fcmp_Nod)))
-      Msg::Error("Node %d does not exist", Geo_Node.Num) ;
-    
+      Message::Error("Node %d does not exist", Geo_Node.Num) ;
+
     Geo_Node_P->x = x[i] ;  Geo_Node_P->y = y[i] ;  Geo_Node_P->z = z[i] ;
   }
 }
@@ -679,8 +707,8 @@ void Geo_SetNodesCoordinatesX(int Nbr_Node, int * Num_Node,
 
     if(!(Geo_Node_P = (struct Geo_Node*)
 	 List_PQuery(CurrentGeoData->Nodes, &Geo_Node, fcmp_Nod)))
-      Msg::Error("Node %d does not exist", Geo_Node.Num) ;
-    
+      Message::Error("Node %d does not exist", Geo_Node.Num) ;
+
     Geo_Node_P->x = x[i] ;
   }
 }
@@ -696,8 +724,8 @@ void Geo_SetNodesCoordinatesY(int Nbr_Node, int * Num_Node,
 
     if(!(Geo_Node_P = (struct Geo_Node*)
 	 List_PQuery(CurrentGeoData->Nodes, &Geo_Node, fcmp_Nod)))
-      Msg::Error("Node %d does not exist", Geo_Node.Num) ;
-    
+      Message::Error("Node %d does not exist", Geo_Node.Num) ;
+
     Geo_Node_P->y = y[i] ;
   }
 }
@@ -713,8 +741,8 @@ void Geo_SetNodesCoordinatesZ(int Nbr_Node, int * Num_Node,
 
     if(!(Geo_Node_P = (struct Geo_Node*)
 	 List_PQuery(CurrentGeoData->Nodes, &Geo_Node, fcmp_Nod)))
-      Msg::Error("Node %d does not exist", Geo_Node.Num) ;
-    
+      Message::Error("Node %d does not exist", Geo_Node.Num) ;
+
     Geo_Node_P->z = z[i] ;
   }
 }
