@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -30,8 +30,7 @@ void  Treatment_ConstraintForElement(struct FunctionSpace    * FunctionSpace_P,
 				     int Num_Entity[], int i_Entity,
 				     int i_BFunction, int TypeConstraint)
 {
-  int                    Nbr_Constraint, i_Constraint, k, Index_GeoElement, dummy ;
-  double                *uvw;
+  int                    Nbr_Constraint, i_Constraint, k, Index_GeoElement ;
   List_T                      * Constraint_L ;
   struct ConstraintInFS       * Constraint_P ;
   struct ConstraintPerRegion  * ConstraintPerRegion_P ;
@@ -42,22 +41,22 @@ void  Treatment_ConstraintForElement(struct FunctionSpace    * FunctionSpace_P,
 
   Constraint_L = FunctionSpace_P->Constraint ;
   Nbr_Constraint = List_Nbr(Constraint_L) ;
-  
+
   for (i_Constraint = 0 ;
        i_Constraint < Nbr_Constraint &&
 	 ! QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].Constraint ;
        i_Constraint++) {
-    
+
     Constraint_P =
       (struct ConstraintInFS*)List_Pointer(Constraint_L, i_Constraint) ;
     ConstraintPerRegion_P = Constraint_P->ConstraintPerRegion ;
 
     switch(ConstraintPerRegion_P->Type) {
-	
+
     case ASSIGN :                case INIT :
     case ASSIGNFROMRESOLUTION :  case INITFROMRESOLUTION :
     case CST_LINK : case CST_LINKCPLX :
-	
+
       switch(Constraint_P->QuantityType) {
 
       case LOCALQUANTITY :
@@ -74,21 +73,28 @@ void  Treatment_ConstraintForElement(struct FunctionSpace    * FunctionSpace_P,
 		ConstraintPerRegion_P->Type == INIT) {
 
 	      switch (TypeConstraint) {
-	      case NODESOF :  
+	      case NODESOF :
 	      case GROUPSOFEDGESONNODESOF :
 		Current.NumEntity = abs(Num_Entity[i_Entity]) ;
-		Geo_GetNodesCoordinates( 1, &Current.NumEntity,
-					 &Current.x, &Current.y, &Current.z) ;
-		/* This is necessary if we want CoordXYZ[] functions
-		   to work in prepro for nodal contraints */
-		uvw = Geo_GetNodes_uvw(Current.Element->Type, &dummy) ;
-		Current.u = uvw[3 * i_Entity] ;
-		Current.v = uvw[3 * i_Entity + 1] ;
-		Current.w = uvw[3 * i_Entity + 2] ;
+		Geo_GetNodesCoordinates(1, &Current.NumEntity,
+                                        &Current.x, &Current.y, &Current.z) ;
+                /* We used to do the follwoing to get the correct u,v,w
+                   coordinates so we can use CoordXYZ[] functions in
+                   preprocessing for nodal constraints:
+                   int dummy;
+                   double *uvw = Geo_GetNodes_uvw(Current.Element->Type, &dummy) ;
+                   Current.u = uvw[3 * i_Entity] ;
+                   Current.v = uvw[3 * i_Entity + 1] ;
+                   Current.w = uvw[3 * i_Entity + 2] ;
+
+                   This is not necessary anymore: we now set Current.Element=0
+                   in Get_{Value,Link}ForConstraint, and thus use
+                   Current.{x,y,z} directly in CoordXYZ[].
+                */
 		break ;
 	      case EDGESOF :
 		Current.NumEntity = abs(Num_Entity[i_Entity]) ;
-		Current.NumEntityInElement = i_Entity ;  
+		Current.NumEntityInElement = i_Entity ;
 		break ;
 	      }
 
@@ -105,6 +111,7 @@ void  Treatment_ConstraintForElement(struct FunctionSpace    * FunctionSpace_P,
 		 abs(Num_Entity[i_Entity]),
 		 &QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].
 		 CodeEntity_Link,
+                 QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].BasisFunction->Orient,
 		 QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].Value) ;
 	      if (abs(Num_Entity[i_Entity]) ==
 		  QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].
@@ -122,21 +129,21 @@ void  Treatment_ConstraintForElement(struct FunctionSpace    * FunctionSpace_P,
 	  }
 	}
 	break ; /* LOCALQUANTITY */
-	
+
       case GLOBALQUANTITY :
-	
+
 	GlobalQuantity_P = (struct GlobalQuantity*)
 	  List_Pointer(FunctionSpace_P->GlobalQuantity,
 		       Constraint_P->ReferenceIndex) ;
 	if ((GlobalQuantity_P->Type == ALIASOF) &&
 	    (GlobalQuantity_P->ReferenceIndex == i_BFunction)) {
-    
+
 	  GroupEntity_Pr = (struct Group*)
 	    List_Pointer(Problem_S.Group, Constraint_P->EntityIndex) ;
-	  
+
 	  if (List_Search(GroupEntity_Pr->InitialList,
 			  &Num_Entity[i_Entity], fcmp_int)) {
-	    
+
 	    QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].Constraint =
 	      ConstraintPerRegion_P->Type ;
 
@@ -150,15 +157,14 @@ void  Treatment_ConstraintForElement(struct FunctionSpace    * FunctionSpace_P,
 	    }
 	    else if (ConstraintPerRegion_P->Type == CST_LINK ||
 		     ConstraintPerRegion_P->Type == CST_LINKCPLX) {
-	      /*
-	      Msg::Error("CST_LINK for GlobalQuantity not done yet") ;
-	      */
-	      /* to be validated */
+	      // Message::Error("CST_LINK for GlobalQuantity not done yet") ;
+	      // FIXME: to be validated
 	      Get_LinkForConstraint
 		(Constraint_P,
 		 abs(Num_Entity[i_Entity]),
 		 &QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].
 		 CodeEntity_Link,
+                 QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].BasisFunction->Orient,
 		 QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].Value) ;
 	      if (abs(Num_Entity[i_Entity]) ==
 		  QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].
@@ -174,30 +180,30 @@ void  Treatment_ConstraintForElement(struct FunctionSpace    * FunctionSpace_P,
 		 TimeFunctionIndex) ;
 	    }
 	  }
-	}		
+	}
 	break ; /* GLOBALQUANTITY */
 
       default :
-	Msg::Error("Unknown type of Quantity in Constraint of type Fixed");
+	Message::Error("Unknown type of Quantity in Constraint of type Fixed");
 	break;
 
       }
-      
+
       break ;  /* ASSIGN || INIT || ASSIGNFROMRESOLUTION || INITFROMRESOLUTION */
 
     default :
-      Msg::Error("Unknown type of Constraint");
+      Message::Error("Unknown type of Constraint");
       break;
 
     }
-    
+
   }  /* for i_Constraint ... */
 
   /* Constraints due to P-refinement */
   if(Current.GeoData->P) {
     Index_GeoElement = Geo_GetGeoElementIndex(Current.Element->GeoElement) ;
     if (Current.GeoData->P[Index_GeoElement+1] >= 0 &&
-	Current.GeoData->P[Index_GeoElement+1] < 
+	Current.GeoData->P[Index_GeoElement+1] <
 	QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].BasisFunction->Order){
       QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].Constraint = ASSIGN ;
       for (k = 0 ; k < Current.NbrHar ; k++)
@@ -217,8 +223,11 @@ void  Get_ValueForConstraint(struct ConstraintInFS * Constraint_P, double Value[
   int  k ;
   struct Value  Val_Modulus, Val_TimeFunction ;
 
-  /* Attention: u,v,w et x,y,z == 0 ! */
-  /* Il faudra changer ca. Pour le moment, on repose sur Current.x|y|z  */
+  // Warning: Current.{u,v,w} are not defined, so we cannot interpolate
+  // expressions in the reference element. We thus set Current.Element=0 and
+  // rely on Current.{x,y,z}.
+  struct Element *old = Current.Element;
+  Current.Element = 0;
 
   Get_ValueOfExpression
     ((struct Expression *)
@@ -233,7 +242,7 @@ void  Get_ValueForConstraint(struct ConstraintInFS * Constraint_P, double Value[
       Get_ValueOfExpression
 	((struct Expression *)
 	 List_Pointer(Problem_S.Expression,
-		      Constraint_P->ConstraintPerRegion->TimeFunctionIndex),	 
+		      Constraint_P->ConstraintPerRegion->TimeFunctionIndex),
 	 NULL, 0., 0., 0., &Val_TimeFunction) ;
 
       Cal_ProductValue(&Val_Modulus, &Val_TimeFunction,  &Val_Modulus) ;
@@ -246,8 +255,10 @@ void  Get_ValueForConstraint(struct ConstraintInFS * Constraint_P, double Value[
     // Set this to zero to avoid having an uninitialized imaginary
     // part if you use a complex arithmetic solver (on a real matrix)
     // -- cf. LinAlg_SetScalar() calls in DofData.cpp
-    Value[1] = 0. ; 
+    Value[1] = 0. ;
   }
+
+  Current.Element = old;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -269,19 +280,19 @@ void  Treatment_ConstraintForRegion(struct GlobalQuantity   * GlobalQuantity_P,
 
   Constraint_L = FunctionSpace_P->Constraint ;
   Nbr_Constraint = List_Nbr(Constraint_L) ;
-  
+
   for (i_Constraint = 0 ;
        i_Constraint < Nbr_Constraint &&
 	 ! QuantityStorage_P->BasisFunction[0].Constraint ; i_Constraint++) {
-    
+
     Constraint_P =
       (struct ConstraintInFS*)List_Pointer(Constraint_L, i_Constraint) ;
     ConstraintPerRegion_P = Constraint_P->ConstraintPerRegion ;
-    
+
     if (Constraint_P->QuantityType == GLOBALQUANTITY) {
 
       switch(ConstraintPerRegion_P->Type) {
-	
+
       case ASSIGN :                case INIT :
       case ASSIGNFROMRESOLUTION :  case INITFROMRESOLUTION :
       case CST_LINK : case CST_LINKCPLX :
@@ -291,12 +302,12 @@ void  Treatment_ConstraintForRegion(struct GlobalQuantity   * GlobalQuantity_P,
 		       Constraint_P->ReferenceIndex) ;
 
 	if (GlobalQuantity_Pr == GlobalQuantity_P) {
-	  
+
 	  GroupEntity_Pr = (struct Group*)
 	    List_Pointer(Problem_S.Group, Constraint_P->EntityIndex) ;
-	  
-	  if (/*(GroupEntity_Pr->FunctionType == 
-		((struct Group *)List_Pointer(Problem_S.Group, 
+
+	  if (/*(GroupEntity_Pr->FunctionType ==
+		((struct Group *)List_Pointer(Problem_S.Group,
 		BasisFunction_P->EntityIndex))
 		->FunctionType)  && */
 	      List_Search
@@ -317,6 +328,7 @@ void  Treatment_ConstraintForRegion(struct GlobalQuantity   * GlobalQuantity_P,
 		(Constraint_P,
 		 QuantityStorage_P->BasisFunction[0].CodeEntity,
 		 &QuantityStorage_P->BasisFunction[0].CodeEntity_Link,
+                 QuantityStorage_P->BasisFunction[0].BasisFunction->Orient,
 		 QuantityStorage_P->BasisFunction[0].Value) ;
 	      if (QuantityStorage_P->BasisFunction[0].CodeEntity ==
 		  QuantityStorage_P->BasisFunction[0].CodeEntity_Link)
@@ -332,7 +344,7 @@ void  Treatment_ConstraintForRegion(struct GlobalQuantity   * GlobalQuantity_P,
 	break ;  /* ASSIGN || INIT || ASSIGNFROMRESOLUTION || INITFROMRESOLUTION */
 
       default :
-	Msg::Error("Unknown type of Constraint");
+	Message::Error("Unknown type of Constraint");
 	break;
       }
     }  /* if (GLOBALQUANTITY) ... */
@@ -354,20 +366,20 @@ void  Get_PreResolutionForConstraint(struct ConstraintInFS * Constraint_P,
   *Index_TimeFunction = Constraint_P->ConstraintPerRegion->TimeFunctionIndex ;
 
   if (Constraint_P->Active.ResolutionIndex < 0)
-    if ((Constraint_P->Active.ResolutionIndex = 
+    if ((Constraint_P->Active.ResolutionIndex =
 	 List_ISearchSeq(Problem_S.Resolution,
 			 Constraint_P->ConstraintPerRegion->
 			 Case.Solve.ResolutionName, fcmp_Resolution_Name)) < 0) {
-      Msg::Error("Unknown ResolutionName '%s' in Constraint",
-		 Constraint_P->ConstraintPerRegion->Case.Solve.ResolutionName) ;
+      Message::Error("Unknown ResolutionName '%s' in Constraint",
+                     Constraint_P->ConstraintPerRegion->Case.Solve.ResolutionName) ;
     }
-  if(List_ISearchSeq(PreResolutionIndex_L, &Constraint_P->Active.ResolutionIndex, 
+  if(List_ISearchSeq(PreResolutionIndex_L, &Constraint_P->Active.ResolutionIndex,
 		     fcmp_int) < 0) {
     PreResolutionInfo_S.Index = Constraint_P->Active.ResolutionIndex ;
     PreResolutionInfo_S.Type  = PR_CONSTRAINT ;
     List_Add(PreResolutionIndex_L, &PreResolutionInfo_S) ;
-    Msg::Info("  Adding Resolution '%s' for Pre-Resolution (Constraint)",
-	      Constraint_P->ConstraintPerRegion->Case.Solve.ResolutionName) ;
+    Message::Info("  Adding Resolution '%s' for Pre-Resolution (Constraint)",
+                  Constraint_P->ConstraintPerRegion->Case.Solve.ResolutionName) ;
   }
 }
 
@@ -381,8 +393,13 @@ struct TwoIntOneDouble { int  Int1, Int2 ; double Double, Double2 ; } ;
 
 void  Get_LinkForConstraint(struct ConstraintInFS * Constraint_P,
 			    int Num_Entity,
-			    int * CodeEntity_Link, double Value[])
+			    int * CodeEntity_Link, int Orient, double Value[])
 {
+  // Warning: Current.{u,v,w} are not defined, so we cannot interpolate
+  // expressions in the reference element. We thus set Current.Element=0 and
+  // rely on Current.{x,y,z}.
+  struct Element *old = Current.Element;
+  Current.Element = 0;
 
   struct TwoIntOneDouble  * TwoIntOneDouble_P ;
   List_T * Couples_L ;
@@ -400,25 +417,23 @@ void  Get_LinkForConstraint(struct ConstraintInFS * Constraint_P,
   TwoIntOneDouble_P = (struct TwoIntOneDouble *)
     ((Couples_L = Constraint_P->Active.Active->Case.Link.Couples)?
      List_PQuery(Couples_L, &Num_Entity, fcmp_absint) : NULL) ;
-  /* old
-    List_PQuery(Constraint_P->Active.Active->Case.Link.Couples,
-		&Num_Entity, fcmp_absint) ;
-  if (!TwoIntOneDouble_P)  Msg::Error("Constraint Link: bad definition") ;
-  */
 
   if (TwoIntOneDouble_P) {
     *CodeEntity_Link = abs(TwoIntOneDouble_P->Int2) ;
     Value[0] = TwoIntOneDouble_P->Double ;
-    if (TwoIntOneDouble_P->Int1 < 0)  Value[0] *= -1. ;
+    if (Orient && TwoIntOneDouble_P->Int1 < 0)  Value[0] *= -1. ;
     Value[1] = TwoIntOneDouble_P->Double2 ;  /* LinkCplx */
-    if (TwoIntOneDouble_P->Int1 < 0)  Value[1] *= -1. ;
+    if (Orient && TwoIntOneDouble_P->Int1 < 0)  Value[1] *= -1. ;
   }
+
+  Current.Element = old;
 }
 
 /* Data... */
 
 struct NodeXYZ { int NumNode ; double x, y, z ; } ;
 struct EdgeNN { int NumEdge ; int Node1, Node2 ; double Coef, Coef2 ; } ;
+struct FacetNNN { int NumFacet ; int Node1, Node2, Node3 ; double Coef, Coef2 ; } ;
 void  Generate_LinkNodes(struct ConstraintInFS * Constraint_P,
 			 List_T * ExtendedList_L, List_T * ExtendedSuppList_L,
 			 struct Group * RegionRef_P, struct Group * SubRegionRef_P,
@@ -428,13 +443,20 @@ void  Generate_LinkEdges(struct ConstraintInFS * Constraint_P,
 			 struct Group * Group_P,
 			 struct Group * RegionRef_P, struct Group * SubRegionRef_P,
 			 List_T * Couples_L) ;
+void  Generate_LinkFacets(struct ConstraintInFS * Constraint_P,
+			  struct Group * Group_P,
+			  struct Group * RegionRef_P, struct Group * SubRegionRef_P,
+			  List_T * Couples_L) ;
 int fcmp_XYZ(const void * a, const void * b) ;
 int fcmp_NN(const void * a, const void * b) ;
+int fcmp_NNN(const void * a, const void * b) ;
 void  Generate_LinkRegions(struct ConstraintInFS * Constraint_P,
 			   List_T * Region_L, List_T * RegionRef_L,
 			   int Index_Coef, List_T * Couples_L) ;
 void  Generate_ElementaryEntities_EdgeNN
   (List_T * InitialList, List_T ** ExtendedList, int Type_Entity) ;
+void  Generate_ElementaryEntities_FacetNNN
+  (List_T * InitialList, List_T ** ExtendedList, int Type_Entity) ;
 
 /* ----- */
 
@@ -445,7 +467,7 @@ void  Generate_Link(struct ConstraintInFS * Constraint_P, int Flag_New)
   struct Group  * Group_P, * RegionRef_P, * SubRegionRef_P ;
   int  Nbr_Entity ;
 
-  Msg::Debug("C o n s t r a i n t   ( L i n k )") ;
+  Message::Debug("C o n s t r a i n t   ( L i n k )") ;
 
   if (Flag_New)
     Constraint_P->Active.Active =
@@ -467,10 +489,12 @@ void  Generate_Link(struct ConstraintInFS * Constraint_P, int Flag_New)
 		 Constraint_P->ConstraintPerRegion->Case.Link.SubRegionRefIndex) :
     NULL ;
 
-  if (Group_P->FunctionType == REGION)
+  if (Group_P->FunctionType == REGION){
     Nbr_Entity = List_Nbr(Group_P->InitialList) ;
-  else
+  }
+  else{
     Nbr_Entity = List_Nbr(Group_P->ExtendedList) ;
+  }
 
   if (Nbr_Entity) {
     if (Flag_New)
@@ -500,7 +524,10 @@ void  Generate_Link(struct ConstraintInFS * Constraint_P, int Flag_New)
 		       Active->Case.Link.Couples) ;
     break ;
   case FACETSOF :
-    Msg::Error("Link not yet implemented for FACETSOF") ;
+    Generate_LinkFacets(Constraint_P, Group_P,
+		        RegionRef_P, SubRegionRef_P,
+		        Active->Case.Link.Couples) ;
+    /*Message::Error("Link not yet implemented for FACETSOF") ;*/
     break ;
   case REGION :
     Generate_LinkRegions(Constraint_P,
@@ -509,7 +536,7 @@ void  Generate_Link(struct ConstraintInFS * Constraint_P, int Flag_New)
 			 Active->Case.Link.Couples) ;
     break ;
   default :
-    Msg::Error("Bad function type for Constraint Link: %d", Group_P->FunctionType) ;
+    Message::Error("Bad function type for Constraint Link: %d", Group_P->FunctionType) ;
     break ;
   }
 }
@@ -523,7 +550,7 @@ void  Generate_LinkNodes(struct ConstraintInFS * Constraint_P,
 			 List_T * Couples_L)
 {
   int  Nbr_Entity, i, Nbr_EntityRef, Flag_Filter ;
-  double TOL=Current.GeoData->CharacteristicLength * 1.e-8;
+  double TOL = Current.GeoData->CharacteristicLength * 1.e-8;
   struct TwoIntOneDouble  TwoIntOneDouble ;
   struct NodeXYZ  NodeXYZ, NodeXYZRef ;
   List_T  * NodeXYZ_L, * NodeXYZRef_L ;
@@ -576,12 +603,6 @@ void  Generate_LinkNodes(struct ConstraintInFS * Constraint_P,
 	  List_Search(ExtendedSuppListRef_L, &NodeXYZRef.NumNode, fcmp_int))) {
       Geo_GetNodesCoordinates( 1, &NodeXYZRef.NumNode,
 			       &Current.x, &Current.y, &Current.z) ;
-      /*
-      Get_ValueOfExpressionByIndex(Index_Function, NULL, 0., 0., 0., &Value) ;
-
-      Current.x = Value.Val[0] ; Current.y = Value.Val[1] ;
-      Current.z = Value.Val[2] ;
-      */
       if (Index_Filter < 0)  Flag_Filter = 1 ;
       else {
 	Get_ValueOfExpressionByIndex(Index_Filter, NULL, 0., 0., 0., &Value) ;
@@ -597,48 +618,14 @@ void  Generate_LinkNodes(struct ConstraintInFS * Constraint_P,
   }
   Nbr_EntityRef = List_Nbr(NodeXYZRef_L) ;
 
-  /*
-  Msg::Debug("Before sorting") ;
-  Msg::Debug("- Other") ;
-  for (i = 0 ; i < Nbr_Entity ; i++) {
-    List_Read(NodeXYZ_L, i, &NodeXYZ) ;
-    Msg::Debug("%d -> %d: %e %e :: %e",
-        i, NodeXYZ.NumNode, NodeXYZ.x, NodeXYZ.y,
-	atan2(NodeXYZ.y,NodeXYZ.x)/3.1415926535897*180.) ;
-  }
-  Msg::Debug("- Reference (after rotation)") ;
-  for (i = 0 ; i < Nbr_EntityRef ; i++) {
-    List_Read(NodeXYZRef_L, i, &NodeXYZ) ;
-    Msg::Debug("%d -> %d: %e %e :: %e",
-	i, NodeXYZ.NumNode, NodeXYZ.x, NodeXYZ.y,
-	atan2(NodeXYZ.y,NodeXYZ.x)/3.1415926535897*180.) ;
-  }
-  */
-
   List_Sort(NodeXYZ_L   , fcmp_XYZ) ;
   List_Sort(NodeXYZRef_L, fcmp_XYZ) ;
 
-  Msg::Debug("After sorting") ;
-  Msg::Debug("- Other (after rotation)") ;
-  for (i = 0 ; i < Nbr_Entity ; i++) {
-    List_Read(NodeXYZ_L, i, &NodeXYZ) ;
-    Msg::Debug("%d -> %d: %e %e %e :: %e",
-	       i, NodeXYZ.NumNode, NodeXYZ.x, NodeXYZ.y, NodeXYZ.z,
-	       atan2(NodeXYZ.y,NodeXYZ.x)/3.1415926535897*180.) ;
-  }
-  Msg::Debug("- Reference") ;
-  for (i = 0 ; i < Nbr_EntityRef ; i++) {
-    List_Read(NodeXYZRef_L, i, &NodeXYZ) ;
-    Msg::Debug("%d -> %d: %e %e %e :: %e",
-	       i, NodeXYZ.NumNode, NodeXYZ.x, NodeXYZ.y, NodeXYZ.z,
-	       atan2(NodeXYZ.y,NodeXYZ.x)/3.1415926535897*180.) ;
-  }
-
   if (Nbr_EntityRef != Nbr_Entity)
-    Msg::Error("Constraint Link: bad correspondance of number of Nodes (%d, %d)",
-	       Nbr_Entity, Nbr_EntityRef) ;
+    Message::Error("Constraint Link: bad correspondance of number of Nodes (%d, %d)",
+                   Nbr_Entity, Nbr_EntityRef) ;
 
-  Msg::Debug("==> List of link for nodes") ;
+  Message::Debug("==> List of link for nodes") ;
 
   for (i = 0 ; i < Nbr_Entity ; i++) {
     List_Read(NodeXYZ_L, i, &NodeXYZ) ;
@@ -648,21 +635,18 @@ void  Generate_LinkNodes(struct ConstraintInFS * Constraint_P,
     if ((fabs(NodeXYZ.x-NodeXYZRef.x) > TOL) ||
 	(fabs(NodeXYZ.y-NodeXYZRef.y) > TOL) ||
 	(fabs(NodeXYZ.z-NodeXYZRef.z) > TOL))
-      Msg::Error("Constraint Link: bad correspondance of Nodes (%d, %d)"
-		 " (%e %e %e)",
-		 NodeXYZ.NumNode, NodeXYZRef.NumNode,
-		 fabs(NodeXYZ.x-NodeXYZRef.x), fabs(NodeXYZ.y-NodeXYZRef.y),
-		 fabs(NodeXYZ.z-NodeXYZRef.z)) ;
+      Message::Error("Constraint Link: bad correspondance of Nodes (%d, %d)"
+                       " (%e %e %e), TOL=%g",
+                       NodeXYZ.NumNode, NodeXYZRef.NumNode,
+                       fabs(NodeXYZ.x-NodeXYZRef.x), fabs(NodeXYZ.y-NodeXYZRef.y),
+                       fabs(NodeXYZ.z-NodeXYZRef.z), TOL) ;
 
     TwoIntOneDouble.Int1 = NodeXYZ.NumNode ;
     TwoIntOneDouble.Int2 = NodeXYZRef.NumNode ;
-    /*
-    Current.x = NodeXYZ.x ; Current.y = NodeXYZ.y ; Current.z = NodeXYZ.z ;
-    */
-    /* Calcul du coefficient base sur les coordonnees du noeud de ref ... */
 
-    Geo_GetNodesCoordinates( 1, &NodeXYZRef.NumNode,
-			     &Current.x, &Current.y, &Current.z) ;
+    /* Calcul du coefficient base sur les coordonnees du noeud de ref ... */
+    Geo_GetNodesCoordinates(1, &NodeXYZRef.NumNode,
+                            &Current.x, &Current.y, &Current.z) ;
     Get_ValueOfExpressionByIndex(Index_Coef, NULL, 0., 0., 0., &Value) ;
     TwoIntOneDouble.Double = Value.Val[0] ;
     if (Current.NbrHar == 1)
@@ -670,17 +654,17 @@ void  Generate_LinkNodes(struct ConstraintInFS * Constraint_P,
     else
       TwoIntOneDouble.Double2 = Value.Val[MAX_DIM] ; /* LinkCplx */
 
-
     List_Add(Couples_L, &TwoIntOneDouble) ;
 
-    Msg::Debug("%d %d : coef %e %e", NodeXYZ.NumNode, NodeXYZRef.NumNode, 
-	       TwoIntOneDouble.Double, TwoIntOneDouble.Double2) ;
+    Message::Debug("%d %d : coef %e %e", NodeXYZ.NumNode, NodeXYZRef.NumNode,
+                   TwoIntOneDouble.Double, TwoIntOneDouble.Double2) ;
   }
 
   List_Delete(NodeXYZ_L) ;  List_Delete(NodeXYZRef_L) ;
 }
 
-int fcmp_XYZ(const void * a, const void * b) {
+int fcmp_XYZ(const void * a, const void * b)
+{
   double Result, TOL=Current.GeoData->CharacteristicLength * 1.e-8 ;
 
   if (fabs(Result = ((struct NodeXYZ *)a)->x - ((struct NodeXYZ *)b)->x) > TOL)
@@ -750,10 +734,9 @@ void  Generate_LinkEdges(struct ConstraintInFS * Constraint_P,
 		       CouplesOfNodes2_L) ;
   }
 
-
   /* Couples of edges */
 
-  Msg::Info("== Couples of edges ==") ;
+  Message::Info("== Couples of edges ==") ;
 
   /* Edges with Constraint */
 
@@ -767,11 +750,9 @@ void  Generate_LinkEdges(struct ConstraintInFS * Constraint_P,
   else
     ExtendedSuppListRef_L = NULL ;
 
-  /*  EdgeNN_L = ExtendedList_L ; */
   EdgeNN_L = List_Create(Nbr_Entity, 1, sizeof(struct EdgeNN)) ;
 
-  /*  if (Nbr_Entity != List_Nbr(EdgeNN_L))  Msg::Error("Constraint Link: strange...") ; */
-  if (Nbr_Entity != List_Nbr(ExtendedList_L))  Msg::Error("Constraint Link: strange...") ;
+  if (Nbr_Entity != List_Nbr(ExtendedList_L))  Message::Error("Constraint Link: strange...") ;
 
   for (i = 0 ; i < Nbr_Entity ; i++) {
     List_Read(ExtendedList_L, i, &EdgeNN) ;
@@ -781,13 +762,10 @@ void  Generate_LinkEdges(struct ConstraintInFS * Constraint_P,
       if (EdgeNN.Node2 < EdgeNN.Node1) {
 	Save_Num = EdgeNN.Node2 ;
 	EdgeNN.Node2 = EdgeNN.Node1 ;  EdgeNN.Node1 = Save_Num ;
-	/*	List_Write(EdgeNN_L, i, &EdgeNN) ; */
       }
-      List_Add(EdgeNN_L, &EdgeNN) ;
 
-    /* -- */
-      Msg::Debug("Other %d: a%d, n%d - n%d",
-		 i, EdgeNN.NumEdge, EdgeNN.Node1, EdgeNN.Node2) ;
+      Message::Debug("Image %d: a%d, n%d - n%d",
+                     i, EdgeNN.NumEdge, EdgeNN.Node1, EdgeNN.Node2) ;
 
       TwoIntOneDouble_P = (struct TwoIntOneDouble *)
 	List_PQuery(CouplesOfNodes_L, &EdgeNN.Node1, fcmp_int) ;
@@ -801,18 +779,18 @@ void  Generate_LinkEdges(struct ConstraintInFS * Constraint_P,
 	  TwoIntOneDouble2_P = (struct TwoIntOneDouble *)
 	    List_PQuery(CouplesOfNodes2_L, &EdgeNN.Node2, fcmp_int) ;
 	  if (!TwoIntOneDouble_P)
-	    Msg::Error("Constraint Link: unknown node (%d)", EdgeNN.Node1) ;
+	    Message::Error("Constraint Link: unknown node (%d)", EdgeNN.Node1) ;
 	  if (!TwoIntOneDouble2_P)
-	    Msg::Error("Constraint Link: unknown node (%d)", EdgeNN.Node2) ;
+	    Message::Error("Constraint Link: unknown node (%d)", EdgeNN.Node2) ;
 	}
-	else  Msg::Error("Constraint Link: bad correspondance for edges") ;
+	else  Message::Error("Constraint Link: bad correspondance for edges") ;
       }
 
       EdgeNN.Node1 = TwoIntOneDouble_P->Int2 ;
       EdgeNN.Node2 = TwoIntOneDouble2_P->Int2 ;
 
       if (fabs(TwoIntOneDouble_P->Double - TwoIntOneDouble2_P->Double) > 1.e-18)
-	Msg::Error("Constraint Link: Bad Coefficient for Edges") ;
+	Message::Error("Constraint Link: Bad Coefficient for Edges") ;
 
       EdgeNN.Coef = TwoIntOneDouble_P->Double ;
       EdgeNN.Coef2 = TwoIntOneDouble_P->Double2 ; /* LinkCplx */
@@ -822,11 +800,11 @@ void  Generate_LinkEdges(struct ConstraintInFS * Constraint_P,
 	EdgeNN.Node2 = EdgeNN.Node1 ;  EdgeNN.Node1 = Save_Num ;
 	EdgeNN.NumEdge *= -1 ;
       }
-      /*      List_Write(EdgeNN_L, i, &EdgeNN) ; */
-      List_Write(EdgeNN_L, List_Nbr(EdgeNN_L)-1, &EdgeNN) ;
-      /* -- */
-      Msg::Debug("                         -->  a%d, n%d - n%d",
-		 EdgeNN.NumEdge, EdgeNN.Node1, EdgeNN.Node2) ;
+
+      List_Add(EdgeNN_L, &EdgeNN) ;
+
+      Message::Debug("                         --- (whose source is) --->  a%d, n%d - n%d",
+                     EdgeNN.NumEdge, EdgeNN.Node1, EdgeNN.Node2) ;
 
     }
   }
@@ -844,7 +822,6 @@ void  Generate_LinkEdges(struct ConstraintInFS * Constraint_P,
 
   Nbr_EntityRef = List_Nbr(ExtendedListRef_L) ;
 
-  /*  EdgeNNRef_L = ExtendedListRef_L ; */
   EdgeNNRef_L = List_Create(Nbr_EntityRef, 1, sizeof(struct EdgeNN)) ;
 
   for (i = 0 ; i < Nbr_EntityRef ; i++) {
@@ -854,20 +831,18 @@ void  Generate_LinkEdges(struct ConstraintInFS * Constraint_P,
       if (EdgeNNRef.Node2 < EdgeNNRef.Node1) {
 	Save_Num = EdgeNNRef.Node2 ;
 	EdgeNNRef.Node2 = EdgeNNRef.Node1 ;  EdgeNNRef.Node1 = Save_Num ;
-	/*	List_Write(EdgeNNRef_L, i, &EdgeNNRef) ; */
       }
       List_Add(EdgeNNRef_L, &EdgeNNRef) ;
 
-      /* -- */
-      Msg::Debug("Ref   %d: a%d, n%d - n%d",
-		 i, EdgeNNRef.NumEdge, EdgeNNRef.Node1, EdgeNNRef.Node2) ;
+      Message::Debug("Ref   %d: a%d, n%d - n%d",
+                     i, EdgeNNRef.NumEdge, EdgeNNRef.Node1, EdgeNNRef.Node2) ;
     }
   }
   Nbr_EntityRef = List_Nbr(EdgeNNRef_L) ;
 
   if (Nbr_EntityRef != Nbr_Entity)
-    Msg::Error("Constraint Link: bad correspondance of number of Edges (%d, %d)",
-	       Nbr_Entity, Nbr_EntityRef) ;
+    Message::Error("Constraint Link: bad correspondance of number of Edges (%d, %d)",
+                   Nbr_Entity, Nbr_EntityRef) ;
 
   List_Sort(EdgeNN_L   , fcmp_NN) ;
   List_Sort(EdgeNNRef_L, fcmp_NN) ;
@@ -876,16 +851,15 @@ void  Generate_LinkEdges(struct ConstraintInFS * Constraint_P,
     List_Read(EdgeNN_L, i, &EdgeNN) ;
     List_Read(EdgeNNRef_L, i, &EdgeNNRef) ;
 
-    Msg::Debug("%d: a%d, n%d - n%d (%.16g) / a%d, n%d - n%d",
-	       i, 
-	       EdgeNN.NumEdge, EdgeNN.Node1, EdgeNN.Node2, EdgeNN.Coef,
-	       EdgeNNRef.NumEdge, EdgeNNRef.Node1, EdgeNNRef.Node2) ;
-
+    Message::Debug("Final : %d: a%d, n%d - n%d (%.16g + %.16g i) / a%d, n%d - n%d",
+                   i,
+                   EdgeNN.NumEdge, EdgeNN.Node1, EdgeNN.Node2, EdgeNN.Coef, EdgeNN.Coef2,
+                   EdgeNNRef.NumEdge, EdgeNNRef.Node1, EdgeNNRef.Node2) ;
 
     if (EdgeNN.Node1 != EdgeNNRef.Node1 ||
 	EdgeNN.Node2 != EdgeNNRef.Node2)
-      Msg::Error("Constraint Link: bad correspondance of Edges (%d, %d)",
-		 EdgeNN.NumEdge, EdgeNNRef.NumEdge) ;
+      Message::Error("Constraint Link: bad correspondance of Edges (%d, %d)",
+                     EdgeNN.NumEdge, EdgeNNRef.NumEdge) ;
 
     TwoIntOneDouble.Int1 = EdgeNN.NumEdge ;
     TwoIntOneDouble.Int2 = EdgeNNRef.NumEdge ;
@@ -898,7 +872,7 @@ void  Generate_LinkEdges(struct ConstraintInFS * Constraint_P,
   List_Delete(EdgeNN_L) ;  List_Delete(EdgeNNRef_L) ;
   List_Delete(CouplesOfNodes_L) ;  List_Delete(CouplesOfNodes2_L) ;
 
-  Msg::Debug("====> End Link Edge") ;
+  Message::Info("====> End Link Edge") ;
 }
 
 int fcmp_NN(const void * a, const void * b)
@@ -953,6 +927,310 @@ void  Generate_ElementaryEntities_EdgeNN
   else  *ExtendedList = NULL ;
 }
 
+/*-----------------------------------------*/
+/*| G e n e r a t e _ L i n k F a c e t s |*/
+/*-----------------------------------------*/
+
+void  Generate_LinkFacets(struct ConstraintInFS * Constraint_P,
+			  struct Group * Group_P,
+			  struct Group * RegionRef_P, struct Group * SubRegionRef_P,
+			 List_T * Couples_L)
+{
+  int  Nbr_Entity, Nbr_EntityRef ;
+
+  List_T  * ExtendedListNodes_L ;
+  List_T  * CouplesOfNodes_L, * CouplesOfNodes2_L ;
+  struct FacetNNN  FacetNNN, FacetNNNRef ;
+  List_T  * FacetNNN_L, * FacetNNNRef_L ;
+  List_T  * ExtendedListRef_L, * ExtendedSuppListRef_L ;
+
+  int  i ;
+  struct TwoIntOneDouble *TwoIntOneDouble_P, *TwoIntOneDouble2_P;
+  struct TwoIntOneDouble *TwoIntOneDouble3_P, TwoIntOneDouble ;
+
+  List_T  * ExtendedList_L ;
+  // int  Save_Num1, Save_Num2, Save_Num3;
+  int Flag_Filter ;
+
+  /* Couples of nodes */
+
+  Generate_ElementaryEntities
+    (Group_P->InitialList, &ExtendedListNodes_L, NODESOF) ;
+
+  if ((Nbr_Entity = List_Nbr(ExtendedListNodes_L)))
+    CouplesOfNodes_L = List_Create(Nbr_Entity, 1, sizeof(struct TwoIntOneDouble)) ;
+  else {
+    return ;  /* situation impossible... */
+  }
+
+  if (Constraint_P->ConstraintPerRegion->Case.Link.FilterIndex2 < 0) {
+    Flag_Filter = 0 ;
+    CouplesOfNodes2_L = NULL ;
+    Generate_LinkNodes(Constraint_P, ExtendedListNodes_L, NULL, RegionRef_P, NULL,
+		       Constraint_P->ConstraintPerRegion->Case.Link.FilterIndex,
+		       Constraint_P->ConstraintPerRegion->Case.Link.FunctionIndex,
+		       Constraint_P->ConstraintPerRegion->Case.Link.CoefIndex,
+		       CouplesOfNodes_L) ;
+  }
+  else {
+    Flag_Filter = 1 ;
+    CouplesOfNodes2_L = List_Create(Nbr_Entity, 1, sizeof(struct TwoIntOneDouble)) ;
+
+    Generate_LinkNodes(Constraint_P, ExtendedListNodes_L, NULL, RegionRef_P, NULL,
+		       Constraint_P->ConstraintPerRegion->Case.Link.FilterIndex,
+		       Constraint_P->ConstraintPerRegion->Case.Link.FunctionIndex,
+		       Constraint_P->ConstraintPerRegion->Case.Link.CoefIndex,
+		       CouplesOfNodes_L) ;
+    Generate_LinkNodes(Constraint_P, ExtendedListNodes_L, NULL, RegionRef_P, NULL,
+		       Constraint_P->ConstraintPerRegion->Case.Link.FilterIndex2,
+		       Constraint_P->ConstraintPerRegion->Case.Link.FunctionIndex2,
+		       Constraint_P->ConstraintPerRegion->Case.Link.CoefIndex2,
+		       CouplesOfNodes2_L) ;
+  }
+
+  /* Couples of facets */
+
+  Message::Info("== Couples of facets ==") ;
+
+  /* Facets with Constraint */
+
+  Nbr_Entity = List_Nbr(Group_P->ExtendedList) ;
+
+  Generate_ElementaryEntities_FacetNNN
+    (Group_P->InitialList, &ExtendedList_L, FACETSOF) ;
+  if (Group_P->InitialSuppList)
+    Generate_ElementaryEntities_FacetNNN
+      (Group_P->InitialSuppList, &ExtendedSuppListRef_L, FACETSOF) ;
+  else
+    ExtendedSuppListRef_L = NULL ;
+
+  FacetNNN_L = List_Create(Nbr_Entity, 1, sizeof(struct FacetNNN)) ;
+
+  if (Nbr_Entity != List_Nbr(ExtendedList_L))  Message::Error("Constraint Link: strange...") ;
+
+  Message::Debug("(ajout) Image %d: f%d, n%d - n%d - n%d", i, FacetNNN.NumFacet,
+                 FacetNNN.Node1, FacetNNN.Node2, FacetNNN.Node3) ;
+
+
+  for (i = 0 ; i < Nbr_Entity ; i++) {
+    List_Read(ExtendedList_L, i, &FacetNNN) ;
+    if (!(ExtendedSuppListRef_L &&
+	  List_Search(ExtendedSuppListRef_L, &FacetNNN.NumFacet, fcmp_int))) {
+
+      // FIXME TODO
+      /*if (FacetNNN.Node3 < FacetNNN.Node2) {
+         Save_Num1 = FacetNNN.Node3 ;
+	 FacetNNN.Node3 = FacetNNN.Node2 ;  FacetNNN.Node2 = Save_Num1 ;
+      }
+      if (FacetNNN.Node3 < FacetNNN.Node1) {
+         Save_Num3 = FacetNNN.Node3 ;
+	 FacetNNN.Node3 = FacetNNN.Node1 ;  FacetNNN.Node1 = Save_Num2 ;
+      }
+      if (FacetNNN.Node2 < FacetNNN.Node1) {
+         Save_Num3 = FacetNNN.Node2 ;
+	 FacetNNN.Node2 = FacetNNN.Node1 ;  FacetNNN.Node1 = Save_Num3 ;
+      }*/
+
+      Message::Debug("Image %d: f%d, n%d - n%d - n%d",
+                     i, FacetNNN.NumFacet, FacetNNN.Node1, FacetNNN.Node2, FacetNNN.Node3) ;
+
+      TwoIntOneDouble_P = (struct TwoIntOneDouble *)
+	List_PQuery(CouplesOfNodes_L, &FacetNNN.Node1, fcmp_int) ;
+      TwoIntOneDouble2_P = (struct TwoIntOneDouble *)
+	List_PQuery(CouplesOfNodes_L, &FacetNNN.Node2, fcmp_int) ;
+      TwoIntOneDouble3_P = (struct TwoIntOneDouble *)
+	List_PQuery(CouplesOfNodes_L, &FacetNNN.Node3, fcmp_int) ;
+
+      if (!(TwoIntOneDouble_P && TwoIntOneDouble2_P && TwoIntOneDouble3_P)) {
+	if (Flag_Filter) {
+	  TwoIntOneDouble_P = (struct TwoIntOneDouble *)
+	    List_PQuery(CouplesOfNodes2_L, &FacetNNN.Node1, fcmp_int) ;
+	  TwoIntOneDouble2_P = (struct TwoIntOneDouble *)
+	    List_PQuery(CouplesOfNodes2_L, &FacetNNN.Node2, fcmp_int) ;
+	  TwoIntOneDouble3_P = (struct TwoIntOneDouble *)
+	    List_PQuery(CouplesOfNodes2_L, &FacetNNN.Node3, fcmp_int) ;
+	  if (!TwoIntOneDouble_P)
+	    Message::Error("1-Constraint Link: unknown node (%d)", FacetNNN.Node1) ;
+	  if (!TwoIntOneDouble2_P)
+	    Message::Error("2-Constraint Link: unknown node (%d)", FacetNNN.Node2) ;
+	  if (!TwoIntOneDouble3_P)
+	    Message::Error("3-Constraint Link: unknown node (%d)", FacetNNN.Node3) ;
+	}
+	else  Message::Error("4-Constraint Link: bad correspondance for facets") ;
+      }
+
+      FacetNNN.Node1 = TwoIntOneDouble_P->Int2 ;
+      FacetNNN.Node2 = TwoIntOneDouble2_P->Int2 ;
+      FacetNNN.Node3 = TwoIntOneDouble3_P->Int2 ;
+
+      if ( (fabs(TwoIntOneDouble_P->Double  - TwoIntOneDouble2_P->Double) > 1.e-18) ||
+	   (fabs(TwoIntOneDouble2_P->Double - TwoIntOneDouble3_P->Double) > 1.e-18) ||
+           (fabs(TwoIntOneDouble3_P->Double - TwoIntOneDouble_P->Double ) > 1.e-18)  )
+	Message::Error("5-Constraint Link: Bad Coefficient for Facets") ;
+
+      FacetNNN.Coef  = TwoIntOneDouble_P->Double ;
+      FacetNNN.Coef2 = TwoIntOneDouble_P->Double2 ; /* LinkCplx */
+
+      // FIXME TODO
+      /*if (FacetNNN.Node3 < FacetNNN.Node2) {
+         Save_Num1 = FacetNNN.Node3 ;
+	 FacetNNN.Node3 = FacetNNN.Node2 ;  FacetNNN.Node2 = Save_Num1 ;
+	 FacetNNN.NumFacet *= -1 ;
+      }
+      if (FacetNNN.Node3 < FacetNNN.Node1) {
+         Save_Num2 = FacetNNN.Node3 ;
+	 FacetNNN.Node3 = FacetNNN.Node1 ;  FacetNNN.Node1 = Save_Num2 ;
+	 FacetNNN.NumFacet *= -1 ;
+      }
+      if (FacetNNN.Node2 < FacetNNN.Node1) {
+         Save_Num3 = FacetNNN.Node2 ;
+	 FacetNNN.Node2 = FacetNNN.Node1 ;  FacetNNN.Node1 = Save_Num3 ;
+	 FacetNNN.NumFacet *= -1 ;
+      } */
+
+      List_Add(FacetNNN_L, &FacetNNN) ;
+
+      Message::Debug("                         --- (whose source is) --->  f%d, n%d - n%d - n%d",
+                     FacetNNN.NumFacet, FacetNNN.Node1, FacetNNN.Node2, FacetNNN.Node3) ;
+
+    }
+  }
+  Nbr_Entity = List_Nbr(FacetNNN_L) ;
+
+  /* Facets of reference (Link) */
+
+  Generate_ElementaryEntities_FacetNNN
+    (RegionRef_P->InitialList, &ExtendedListRef_L, FACETSOF) ;
+  if (SubRegionRef_P)
+    Generate_ElementaryEntities_FacetNNN
+      (SubRegionRef_P->InitialList, &ExtendedSuppListRef_L, FACETSOF) ;
+  else
+    ExtendedSuppListRef_L = NULL ;
+
+  Nbr_EntityRef = List_Nbr(ExtendedListRef_L) ;
+
+  FacetNNNRef_L = List_Create(Nbr_EntityRef, 1, sizeof(struct FacetNNN)) ;
+
+  for (i = 0 ; i < Nbr_EntityRef ; i++) {
+    List_Read(ExtendedListRef_L, i, &FacetNNNRef.NumFacet) ;
+    if (!(ExtendedSuppListRef_L &&
+	  List_Search(ExtendedSuppListRef_L, &FacetNNNRef.NumFacet, fcmp_int))) {
+      // FIXME TODO
+      /*if (FacetNNNRef.Node3 < FacetNNNRef.Node2) {
+         Save_Num1 = FacetNNNRef.Node3 ;
+	 FacetNNNRef.Node3 = FacetNNNRef.Node2 ;  FacetNNNRef.Node2 = Save_Num1 ;
+      }
+      if (FacetNNNRef.Node3 < FacetNNNRef.Node1) {
+         Save_Num2 = FacetNNNRef.Node3 ;
+	 FacetNNNRef.Node3 = FacetNNNRef.Node1 ;  FacetNNNRef.Node1 = Save_Num2 ;
+      }
+      if (FacetNNNRef.Node2 < FacetNNNRef.Node1) {
+         Save_Num3 = FacetNNNRef.Node2 ;
+	 FacetNNNRef.Node2 = FacetNNNRef.Node1 ;  FacetNNNRef.Node1 = Save_Num3 ;
+      } */
+      List_Add(FacetNNNRef_L, &FacetNNNRef) ;
+
+      /* -- */
+      Message::Debug("Ref   %d: f%d, n%d - n%d  - n%d ",
+                     i, FacetNNNRef.NumFacet, FacetNNNRef.Node1, FacetNNNRef.Node2,
+                     FacetNNNRef.Node3) ;
+    }
+  }
+  Nbr_EntityRef = List_Nbr(FacetNNNRef_L) ;
+
+  if (Nbr_EntityRef != Nbr_Entity)
+    Message::Error("6-Constraint Link: bad correspondance of number of facets (%d, %d)",
+                   Nbr_Entity, Nbr_EntityRef) ;
+
+  List_Sort(FacetNNN_L   , fcmp_NNN) ;
+  List_Sort(FacetNNNRef_L, fcmp_NNN) ;
+
+  for (i = 0 ; i < Nbr_Entity ; i++) {
+    List_Read(FacetNNN_L, i, &FacetNNN) ;
+    List_Read(FacetNNNRef_L, i, &FacetNNNRef) ;
+
+    Message::Debug("Final : %d: a%d, n%d - n%d - n%d (%.16g + %.16g i) / a%d, n%d - n%d - n%d",
+                   i,
+                   FacetNNN.NumFacet, FacetNNN.Node1, FacetNNN.Node2, FacetNNN.Node3,
+                   FacetNNN.Coef, FacetNNN.Coef2, FacetNNNRef.NumFacet,
+                   FacetNNNRef.Node1, FacetNNNRef.Node2, FacetNNNRef.Node3) ;
+
+    if (FacetNNN.Node1 != FacetNNNRef.Node1 ||
+	FacetNNN.Node2 != FacetNNNRef.Node2 ||
+	FacetNNN.Node3 != FacetNNNRef.Node3 )
+      Message::Error("7-Constraint Link: bad correspondance of facets (%d, %d)",
+                     FacetNNN.NumFacet, FacetNNNRef.NumFacet) ;
+
+    TwoIntOneDouble.Int1 = FacetNNN.NumFacet ;
+    TwoIntOneDouble.Int2 = FacetNNNRef.NumFacet ;
+    TwoIntOneDouble.Double = FacetNNN.Coef ;
+    TwoIntOneDouble.Double2 = FacetNNN.Coef2 ; /* LinkCplx */
+
+    List_Add(Couples_L, &TwoIntOneDouble) ;
+  }
+
+  List_Delete(FacetNNN_L) ;  List_Delete(FacetNNNRef_L) ;
+  List_Delete(CouplesOfNodes_L) ;  List_Delete(CouplesOfNodes2_L) ;
+
+  Message::Info("====> End Link Facet") ;
+}
+
+int fcmp_NNN(const void * a, const void * b)
+{
+  int Result ;
+
+  if ((Result = ((struct FacetNNN *)a)->Node1 - ((struct FacetNNN *)b)->Node1) != 0)
+    return Result ;
+  if ((Result = ((struct FacetNNN *)a)->Node2 - ((struct FacetNNN *)b)->Node2) != 0)
+    return Result ;
+  return        ((struct FacetNNN *)a)->Node3 - ((struct FacetNNN *)b)->Node3 ;
+}
+
+
+void  Generate_ElementaryEntities_FacetNNN
+  (List_T * InitialList, List_T ** ExtendedList, int Type_Entity)
+{
+  Tree_T  * Entity_Tr ;
+  struct Geo_Element  * GeoElement ;
+  int     Nbr_Element, i_Element ;
+  int     Nbr_Entity = 0, i_Entity, * Num_Entities = NULL;
+
+  struct FacetNNN FacetNNN ;
+  int * Num_Nodes ;
+
+  if (InitialList != NULL) {
+
+    Entity_Tr = Tree_Create(sizeof (struct FacetNNN), fcmp_int) ;
+
+    Nbr_Element = Geo_GetNbrGeoElements() ;
+    for (i_Element = 0 ; i_Element < Nbr_Element ; i_Element++) {
+      GeoElement = Geo_GetGeoElement(i_Element) ;
+
+      if (List_Search(InitialList, &GeoElement->Region, fcmp_int) ) {
+	switch (Type_Entity) {
+	case FACETSOF :
+	  if (GeoElement->NbrFacets == 0)  Geo_CreateFacetsOfElement(GeoElement) ;
+	  Nbr_Entity = GeoElement->NbrFacets  ; Num_Entities = GeoElement->NumFacets ;
+	  break ;
+	}
+	for (i_Entity = 0; i_Entity < Nbr_Entity ; i_Entity++) {
+	  FacetNNN.NumFacet = abs(Num_Entities[i_Entity]) ;
+	  Num_Nodes = Geo_GetNodesOfFacetInElement(GeoElement, i_Entity) ;
+	  FacetNNN.Node1 = GeoElement->NumNodes[abs(Num_Nodes[0])-1] ;
+	  FacetNNN.Node2 = GeoElement->NumNodes[abs(Num_Nodes[1])-1] ;
+	  FacetNNN.Node3 = GeoElement->NumNodes[abs(Num_Nodes[2])-1] ;
+	  if ( ! Tree_Search(Entity_Tr, &FacetNNN) )
+	    Tree_Add(Entity_Tr, &FacetNNN) ;
+	}
+      }
+    }
+
+    *ExtendedList = Tree2List(Entity_Tr) ;
+    Tree_Delete(Entity_Tr) ;
+  }
+  else  *ExtendedList = NULL ;
+}
+
 /*  G e n e r a t e _ L i n k R e g i o n s  */
 
 void  Generate_LinkRegions(struct ConstraintInFS * Constraint_P,
@@ -963,7 +1241,7 @@ void  Generate_LinkRegions(struct ConstraintInFS * Constraint_P,
   struct Value  Value ;
 
   if (List_Nbr(Region_L) > 1 || List_Nbr(RegionRef_L) > 1)
-    Msg::Error("More than one region for link type constraint") ;
+    Message::Error("More than one region for link type constraint") ;
 
   List_Read(Region_L, 0, &TwoIntOneDouble.Int1) ;
   List_Read(RegionRef_L, 0, &TwoIntOneDouble.Int2) ;
