@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -22,6 +22,7 @@ void  Get_ValueForConstraint(struct ConstraintInFS * Constraint_P, double Value[
 void  Get_PreResolutionForConstraint(struct ConstraintInFS * Constraint_P,
 				     int * Index_TimeFunction) ;
 void  Get_LinkForConstraint(struct ConstraintInFS * Constraint_P,
-			    int Num_Entity, int * CodeEntity_Link, double Value[]) ;
+			    int Num_Entity, int * CodeEntity_Link,
+                            int Orient, double Value[]) ;
 
 #endif
