@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -58,7 +58,7 @@ void  Get_DofOfElement(struct Element          * Element,
   /* Get the SubSpace */
 
   Nbr_BasisFunctionAll = List_Nbr(FunctionSpace_P->BasisFunction) ;
-  BasisFunction_P0 = (Nbr_BasisFunctionAll) ? 
+  BasisFunction_P0 = (Nbr_BasisFunctionAll) ?
     (struct BasisFunction*)List_Pointer(FunctionSpace_P->BasisFunction, 0) : NULL ;
 
   if (!BasisFunctionIndex_L) {
@@ -68,12 +68,12 @@ void  Get_DofOfElement(struct Element          * Element,
   else {
     Flag_SubSpace = 1 ;
     Nbr_BasisFunction = List_Nbr(BasisFunctionIndex_L) ;
-    BasisFunctionIndex_P0 = (Nbr_BasisFunction) ? 
+    BasisFunctionIndex_P0 = (Nbr_BasisFunction) ?
       (int*)List_Pointer(BasisFunctionIndex_L, 0) : NULL ;
   }
 
   /* Set the DofData if explicitely specified */
-  
+
   switch (TreatmentStatus) {
   case _CAL :  case _POS :
     if(QuantityStorage_P->DefineQuantity->DofData)
@@ -97,7 +97,7 @@ void  Get_DofOfElement(struct Element          * Element,
 	   the interpolation order is lower or equal to the maximum order allowed
 	   the element is in the support of the BasisFunction */
 
-    if ( ( BasisFunction_P->ElementType & Current.Element->Type ) 
+    if ( ( BasisFunction_P->ElementType & Current.Element->Type )
 	 &&
 	 ( Flag_ORDER < 0. || BasisFunction_P->Order <= Flag_ORDER )
 	 &&
@@ -179,6 +179,19 @@ void  Get_DofOfElement(struct Element          * Element,
 	   StartingIndex,  i_BFunction, GROUPSOFEDGESOF, NULL) ;
 	break ;
 
+      case GROUPSOFFACETSOF :
+	if (Element->GeoElement->NbrFacets == 0)
+	  Geo_CreateFacetsOfElement(Element->GeoElement) ;
+	Get_GroupsOfElementaryEntitiesOfElement
+	  (Element, &StartingIndex,
+	   Element->GeoElement->NbrFacets, Element->GeoElement->NumFacets,
+	   BasisFunction_P) ;
+	Get_CodesOfElement
+	  (FunctionSpace_P, QuantityStorage_P,
+	   Element->NbrGroupsOfEntities, Element->NumGroupsOfEntities,
+	   StartingIndex,  i_BFunction, GROUPSOFFACETSOF, NULL) ;
+	break ;
+
       case REGION :
 	Get_RegionForElement(Element, &StartingIndex, BasisFunction_P) ;
 	Get_CodesOfElement
@@ -217,7 +230,6 @@ void  Get_GroupsOfElementaryEntitiesOfElement
 
   int            i, j, Num_Entity, Nbr_SubFunction, i_SF ;
   struct TwoInt  * Key_P, Key ;
-  int iFirst;
 
   if (Element->NumLastElementForGroupsOfEntities != Element->Num) {
     Element->NumLastElementForGroupsOfEntities = Element->Num ;
@@ -231,15 +243,23 @@ void  Get_GroupsOfElementaryEntitiesOfElement
   for (i = 0 ; i < Nbr_ElementaryEntities ; i++) {
     Num_Entity = abs(Num_ElementaryEntities[i]) ;
 
-    /*
-    if ((Key_P = (struct TwoInt*)List_PQuery(GroupEntity_P->ExtendedList, 
-					     &Num_Entity, fcmp_absint ))) {
-    */
-    iFirst = 1;
-    Key.Int1 = Num_Entity;
-    Key_P = &Key;
-    while (List_LQuery(GroupEntity_P->ExtendedList, &Key, fcmp_absint, iFirst)) {
-      iFirst = 0;
+    int iFirst = 1;
+    while(1){
+      if(!GroupEntity_P->IsExtendedListMultiValued){ // fast search
+        Key.Int1 = Num_Entity;
+        Key_P = (struct TwoInt*)List_PQuery(GroupEntity_P->ExtendedList,
+                                            &Key, fcmp_absint);
+        if(!Key_P) break;
+      }
+      else{
+        // general case (e.g. allowing multiple cuts to be considered
+        // for one element)
+        Key.Int1 = Num_Entity;
+        Key_P = &Key;
+        if(!List_LQuery(GroupEntity_P->ExtendedList, &Key, fcmp_absint, iFirst))
+          break;
+        iFirst = 0;
+      }
 
       j = *StartingIndex ;
       while ((j < Element->NbrGroupsOfEntities) &&
@@ -275,7 +295,8 @@ void  Get_GroupsOfElementaryEntitiesOfElement
 	    (Key_P->Int1 > 0)?  (i+1) : -(i+1) ;
       }
 
-    } /* if Key_P */
+      if(!GroupEntity_P->IsExtendedListMultiValued) break;
+    }
   }
 }
 
@@ -419,7 +440,7 @@ void  Get_CodesOfElement(struct FunctionSpace    * FunctionSpace_P,
   struct Dof  * Dof_P = NULL;
 
   /*  1.  F o r   e a c h   e n t i t y   t o   w h i c h   a   b a s i s
-          f u n c t i o n   c o u l d   b e   a s s o c i a t e d : 
+          f u n c t i o n   c o u l d   b e   a s s o c i a t e d :
           (Node, Edge, Facet, Volume, GroupOfNodes, Region, ...)  */
 
   for (i_Entity = StartingIndex ; i_Entity < Nbr_Entity ; i_Entity++) {
@@ -432,7 +453,7 @@ void  Get_CodesOfElement(struct FunctionSpace    * FunctionSpace_P,
     case _POS :
     case _CST :
       if(!FunctionSpace_P->DofData)
-	Msg::Error("Empty DofData in FunctionSpace '%s' (no unknowns?)",
+	Message::Error("Empty DofData in FunctionSpace '%s' (no unknowns?)",
 		   FunctionSpace_P->Name);
 
       CodeExist =
@@ -452,15 +473,15 @@ void  Get_CodesOfElement(struct FunctionSpace    * FunctionSpace_P,
 	Check_IsEntityInExtendedGroup(GroupEntity_P, abs(Num_Entity[i_Entity]), 0) ;
       break ;
 
-    default : 
-      Msg::Error("Unknown TreatmentStatus (%d)", TreatmentStatus);
+    default :
+      Message::Error("Unknown TreatmentStatus (%d)", TreatmentStatus);
     }
 
     /*  2.  O n e   a s s o c i a t e s   a   b a s i s   f u n c t i o n :  */
 
     if (CodeExist) {
 
-      QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].Dof 
+      QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].Dof
 	= Dof_P ;
       QuantityStorage_P->BasisFunction[Nbr_ElementaryBF].NumEntityInElement
 	= i_Entity ;
@@ -478,9 +499,9 @@ void  Get_CodesOfElement(struct FunctionSpace    * FunctionSpace_P,
 				       i_BFunction, TypeConstraint) ;
 
       Nbr_ElementaryBF++ ;
-      
+
     }  /* if CodeExist ... */
-    
+
   }  /* for i_Entity ... */
 }
 
@@ -521,7 +542,7 @@ void  Get_DofOfRegion(int  Num_Region,
     case _POS :
     case _CST :
       if(!FunctionSpace_P->DofData)
-	Msg::Error("Empty DofData in FunctionSpace '%s' (no unknowns?)",
+	Message::Error("Empty DofData in FunctionSpace '%s' (no unknowns?)",
 		   FunctionSpace_P->Name);
 
       CodeExist =
@@ -574,7 +595,7 @@ void  Get_PreResolutionForGlobalBasisFunction(int Nbr_Global, int StartingIndex,
 	Element->GlobalBasisFunction[StartingIndex + i]->ResolutionIndex ;
       PreResolutionInfo_S.Type  = PR_GLOBALBASISFUNCTION ;
       List_Add(PreResolutionIndex_L, &PreResolutionInfo_S) ;
-      Msg::Info("  Adding Resolution '%s' for Pre-Resolution (Global BF)", 
+      Message::Info("  Adding Resolution '%s' for Pre-Resolution (Global BF)",
 		((struct Resolution*)List_Pointer(Problem_S.Resolution,
 						  PreResolutionInfo_S.Index))->Name) ;
     }
