@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -16,14 +16,6 @@
 #include "BF.h"
 #include "Message.h"
 
-#if defined(HAVE_GSL)
-#include <gsl/gsl_vector.h>
-#include <gsl/gsl_linalg.h>
-#else
-#include "nrutil.h"
-void dsvdcmp(double **a, int m, int n, double w[], double **v);
-#endif
-
 #define THESIGN(a) ((a)>=0 ? 1 : -1)
 #define SQU(a)     ((a)*(a)) 
 #define HYPOT(a,b) (sqrt((a)*(a)+(b)*(b)))
@@ -86,8 +78,8 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _3D ; return((void *)JacobianVol3D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian Vol for Element Type (%s)", 
-	  Get_StringForDefine(Element_Type, Type_Element));
+      Message::Error("Unknown Jacobian Vol for Element Type (%s)", 
+                     Get_StringForDefine(Element_Type, Type_Element));
     }
 
   case JACOBIAN_VOL_SPH_SHELL :
@@ -105,8 +97,8 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _3D ; return((void *)JacobianVolSphShell3D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian VolSphShell for Element Type (%s)", 
-	  Get_StringForDefine(Element_Type, Type_Element));
+      Message::Error("Unknown Jacobian VolSphShell for Element Type (%s)", 
+                     Get_StringForDefine(Element_Type, Type_Element));
     }
 
   case JACOBIAN_VOL_RECT_SHELL :
@@ -124,8 +116,8 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _3D ; return((void *)JacobianVolRectShell3D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian VolRectShell for Element Type (%s)", 
-	  Get_StringForDefine(Element_Type, Type_Element));
+      Message::Error("Unknown Jacobian VolRectShell for Element Type (%s)", 
+                     Get_StringForDefine(Element_Type, Type_Element));
     }
 
   case JACOBIAN_VOL_PLPD_X :
@@ -137,8 +129,8 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _2D ; return((void *)JacobianVolPlpdX2D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian VolPlpdX for Element Type (%s)",
-	  Get_StringForDefine(Element_Type, Type_Element));
+      Message::Error("Unknown Jacobian VolPlpdX for Element Type (%s)",
+                     Get_StringForDefine(Element_Type, Type_Element));
     }
 
   case JACOBIAN_VOL_AXI :
@@ -150,8 +142,8 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _2D ; return((void *)JacobianVolAxi2D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian VolAxi for Element Type (%s)",
-	  Get_StringForDefine(Element_Type, Type_Element)); 
+      Message::Error("Unknown Jacobian VolAxi for Element Type (%s)",
+                     Get_StringForDefine(Element_Type, Type_Element)); 
     }
     
   case JACOBIAN_VOL_AXI_SPH_SHELL :
@@ -163,8 +155,8 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _2D ; return((void *)JacobianVolAxiSphShell2D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian VolAxiSphShell for Element Type (%s)",
-	  Get_StringForDefine(Element_Type, Type_Element));
+      Message::Error("Unknown Jacobian VolAxiSphShell for Element Type (%s)",
+                     Get_StringForDefine(Element_Type, Type_Element));
     }
 
   case JACOBIAN_VOL_AXI_RECT_SHELL :
@@ -176,8 +168,8 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _2D ; return((void *)JacobianVolAxiRectShell2D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian VolAxiRectShell for Element Type (%s)",
-	  Get_StringForDefine(Element_Type, Type_Element));
+      Message::Error("Unknown Jacobian VolAxiRectShell for Element Type (%s)",
+                     Get_StringForDefine(Element_Type, Type_Element));
     }
 
   case JACOBIAN_VOL_AXI_PLPD_X :
@@ -189,8 +181,8 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _2D ; return((void *)JacobianVolAxiPlpdX2D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian VolAxiPlpdX for Element Type (%s)",
-	  Get_StringForDefine(Element_Type, Type_Element));
+      Message::Error("Unknown Jacobian VolAxiPlpdX for Element Type (%s)",
+                     Get_StringForDefine(Element_Type, Type_Element));
     }
 
 
@@ -203,8 +195,8 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _2D ; return((void *)JacobianVolAxiSqu2D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian VolAxiSqu for Element Type (%s)",
-	  Get_StringForDefine(Element_Type, Type_Element));
+      Message::Error("Unknown Jacobian VolAxiSqu for Element Type (%s)",
+                     Get_StringForDefine(Element_Type, Type_Element));
     }
 
   case JACOBIAN_VOL_AXI_SQU_SPH_SHELL :
@@ -216,8 +208,8 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _2D ; return((void *)JacobianVolAxiSquSphShell2D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian VolAxiSquSphShell for Element Type (%s)",
-	  Get_StringForDefine(Element_Type, Type_Element));
+      Message::Error("Unknown Jacobian VolAxiSquSphShell for Element Type (%s)",
+                     Get_StringForDefine(Element_Type, Type_Element));
     }
 
   case JACOBIAN_VOL_AXI_SQU_RECT_SHELL :
@@ -229,8 +221,8 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _2D ; return((void *)JacobianVolAxiSquRectShell2D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian VolAxiSquRectShell for Element Type (%s)",
-	  Get_StringForDefine(Element_Type, Type_Element));
+      Message::Error("Unknown Jacobian VolAxiSquRectShell for Element Type (%s)",
+                     Get_StringForDefine(Element_Type, Type_Element));
     }
 
   case JACOBIAN_SUR :
@@ -248,8 +240,8 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _3D ; return((void *)JacobianSur3D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian Sur for element Type (%s)",
-	  Get_StringForDefine(Element_Type, Type_Element));
+      Message::Error("Unknown Jacobian Sur for element Type (%s)",
+                     Get_StringForDefine(Element_Type, Type_Element));
     }
 
   case JACOBIAN_SUR_SPH_SHELL :
@@ -260,8 +252,8 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _2D ; return((void *)JacobianSurSphShell2D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian SurSphShell for element Type (%s)",
-	  Get_StringForDefine(Element_Type, Type_Element));
+      Message::Error("Unknown Jacobian SurSphShell for element Type (%s)",
+                     Get_StringForDefine(Element_Type, Type_Element));
     }
 
   case JACOBIAN_SUR_AXI :
@@ -272,8 +264,8 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _2D ; return((void *)JacobianSurAxi2D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian SurAxi for Element Type (%s)",
-	  Get_StringForDefine(Element_Type, Type_Element));
+      Message::Error("Unknown Jacobian SurAxi for Element Type (%s)",
+                     Get_StringForDefine(Element_Type, Type_Element));
     }
 
   case JACOBIAN_LIN :
@@ -287,12 +279,12 @@ void  * Get_JacobianFunction (int Type_Jacobian, int Type_Element,
       *Type_Dimension = _3D ; return((void *)JacobianLin3D) ;
 
     default : 
-      Msg::Error("Unknown Jacobian Lin for Element Type (%s)",
-	  Get_StringForDefine(Element_Type, Type_Element));
+      Message::Error("Unknown Jacobian Lin for Element Type (%s)",
+                     Get_StringForDefine(Element_Type, Type_Element));
     }
 
   default :
-    Msg::Error("Unknown Jacobian"); return(NULL) ;
+    Message::Error("Unknown Jacobian"); return(NULL) ;
   }
 
 }
@@ -318,8 +310,8 @@ double  PlpdX2D (struct Element * Element, MATRIX3x3 * Jac)
   R = CoorX ;
 
   if ( (R > B+1.e-12*B) || (R < A-1.e-12*A) )
-    Msg::Error("Bad parameters for unidirectional transformation Jacobian: "
-	       "Rint=%g, Rext=%g, R=%g", A, B, R) ;
+    Message::Error("Bad parameters for unidirectional transformation Jacobian: "
+                   "Rint=%g, Rext=%g, R=%g", A, B, R) ;
 
   if (B == R) {
     Jac->c11  = 1. ;  Jac->c12  = 0. ;
@@ -375,7 +367,7 @@ double  Transformation (int Dim, int Type, struct Element * Element, MATRIX3x3 *
     B = Element->JacobianCase->Para[1] ;
   }
   else
-    Msg::Error("Missing interior and/or exterior radius for transformation Jacobian");
+    Message::Error("Missing interior and/or exterior radius for transformation Jacobian");
 
   if(Type == JACOBIAN_RECT){
     if(Element->JacobianCase->NbrParameters >= 3)
@@ -390,8 +382,8 @@ double  Transformation (int Dim, int Type, struct Element * Element, MATRIX3x3 *
     if(Element->JacobianCase->NbrParameters >= 8)
       L = Element->JacobianCase->Para[7] ;
     if(Element->JacobianCase->NbrParameters >= 9){
-      Msg::Error("Too many parameters for rectangular transformation Jacobian");
-      Msg::Error("Valid parameters: Radius1 Radius2 Axis CenterX CenterY CenterZ Power 1/Infinity");
+      Message::Error("Too many parameters for rectangular transformation Jacobian. "
+                     "Vvalid parameters: Radius1 Radius2 Axis CenterX CenterY CenterZ Power 1/Infinity");
     }
   }
   else if(Type == JACOBIAN_SPH){
@@ -405,12 +397,12 @@ double  Transformation (int Dim, int Type, struct Element * Element, MATRIX3x3 *
     if(Element->JacobianCase->NbrParameters >= 7)
       L = Element->JacobianCase->Para[6] ;
     if(Element->JacobianCase->NbrParameters >= 8){
-      Msg::Error("Too many parameters for spherical transformation Jacobian");
-      Msg::Error("Valid parameters: Radius1 Radius2 CenterX CenterY CenterZ Power 1/Infinity");
+      Message::Error("Too many parameters for spherical transformation Jacobian. "
+                     "Valid parameters: Radius1 Radius2 CenterX CenterY CenterZ Power 1/Infinity");
     }
   }
   else
-    Msg::Error("Unknown type of transformation Jacobian");
+    Message::Error("Unknown type of transformation Jacobian");
 
   if(L) B = (B*B-A*A*L)/(B-A*L);
 
@@ -425,13 +417,13 @@ double  Transformation (int Dim, int Type, struct Element * Element, MATRIX3x3 *
     case 1: R = fabs(X-Cx) ; dRdx =THESIGN(X-Cx) ; dRdy =0.0 ;           dRdz =0.0 ;           break;
     case 2: R = fabs(Y-Cy) ; dRdx =0.0 ;           dRdy =THESIGN(Y-Cy) ; dRdz =0.0 ;           break;
     case 3: R = fabs(Z-Cz) ; dRdx =0.0 ;           dRdy =0.0 ;           dRdz =THESIGN(Z-Cz) ; break;
-    default: Msg::Error("Bad axis specification: 1 for X, 2 for Y and 3 for Z");
+    default: Message::Error("Bad axis specification: 1 for X, 2 for Y and 3 for Z");
     }
   }
 
   if ( (fabs(R) > fabs(B) + 1.e-2*fabs(B)) || 
        (fabs(R) < fabs(A) - 1.e-2*fabs(A)) )
-    Msg::Error("Bad parameters for transformation Jacobian: %g not in [%g,%g]", R, A, B) ;
+    Message::Error("Bad parameters for transformation Jacobian: %g not in [%g,%g]", R, A, B) ;
 
   if (B == R) {
     Jac->c11 = 1. ; Jac->c12 = 0. ; Jac->c13 = 0. ;
@@ -798,13 +790,39 @@ double  JacobianVolRectShell3D (struct Element * Element, MATRIX3x3 * Jac)
 /*  J a c o b i a n S u r                                                   */
 /* ------------------------------------------------------------------------ */
 
+void prodve(double a[3], double b[3], double c[3])
+{
+  c[2] = a[0] * b[1] - a[1] * b[0];
+  c[1] = -a[0] * b[2] + a[2] * b[0];
+  c[0] = a[1] * b[2] - a[2] * b[1];
+}
+void prosca(double a[3], double b[3], double *c)
+{
+  *c = a[0] * b[0] + a[1] * b[1] + a[2] * b[2];
+}
+double norm3(double a[3])
+{
+  return sqrt(a[0] * a[0] + a[1] * a[1] + a[2] * a[2]);
+}
+double norme(double a[3])
+{
+  const double mod = norm3(a);
+  if(mod != 0.0){
+    const double one_over_mod = 1./mod;
+    a[0] *= one_over_mod;
+    a[1] *= one_over_mod;
+    a[2] *= one_over_mod;
+  }
+  return mod;
+}
+
 double  JacobianSur2D (struct Element * Element, MATRIX3x3 * Jac)
 {
   int  i ;
   double DetJac ;
 
   Jac->c11 = 0. ;  Jac->c12 = 0. ;  Jac->c13 = 0. ;
-  Jac->c21 = 0. ;  Jac->c22 = 0. ;  Jac->c23 = 0. ; /*modif: c22=1 !!! faux*/
+  Jac->c21 = 0. ;  Jac->c22 = 0. ;  Jac->c23 = 0. ;
   Jac->c31 = 0. ;  Jac->c32 = 0. ;  Jac->c33 = 1. ;
 
   for ( i = 0 ; i < Element->GeoElement->NbrNodes ; i++ ) {
@@ -814,6 +832,18 @@ double  JacobianSur2D (struct Element * Element, MATRIX3x3 * Jac)
 
   DetJac = HYPOT(Jac->c11, Jac->c12) ;
 
+  // regularize matrix
+  double b[3] = {Jac->c12, -Jac->c11, 0.};
+  norme(b);
+  Jac->c21 = b[0]; Jac->c22 = b[1];
+
+  // make sure DetJac > 0: this is not necessary in theory, but it is
+  // required here because we use DetJac when we invert the matrix
+  double realDetJac = Jac->c11 * Jac->c22 - Jac->c12 * Jac->c21;
+  if(realDetJac < 0.){
+    Jac->c21 = - Jac->c21; Jac->c22 = - Jac->c22;
+  }
+
   return(DetJac) ;
 }
 
@@ -822,10 +852,10 @@ double  JacobianSurSphShell2D (struct Element * Element, MATRIX3x3 * Jac)
   MATRIX3x3  Jac1, Jac2 ;
   double     DetJac1, DetJac2 ;
 
-  DetJac1 = JacobianSur2D (Element, &Jac1) ;
+  DetJac1 = JacobianSur2D(Element, &Jac1) ;
   DetJac2 = Transformation(_2D, JACOBIAN_SPH, Element, &Jac2) ;
 
-  Get_ProductMatrix( _3D, &Jac1, &Jac2, Jac) ;
+  Get_ProductMatrix(_3D, &Jac1, &Jac2, Jac) ;
 
   return(DetJac1 * DetJac2) ;
 }
@@ -835,7 +865,7 @@ double  JacobianSurRectShell2D (struct Element * Element, MATRIX3x3 * Jac)
   MATRIX3x3  Jac1, Jac2 ;
   double     DetJac1, DetJac2 ;
 
-  DetJac1 = JacobianSur2D (Element, &Jac1) ;
+  DetJac1 = JacobianSur2D(Element, &Jac1) ;
   DetJac2 = Transformation(_2D, JACOBIAN_RECT, Element, &Jac2) ;
 
   Get_ProductMatrix( _3D, &Jac1, &Jac2, Jac) ;
@@ -853,7 +883,6 @@ double  JacobianSurAxi2D (struct Element * Element, MATRIX3x3 * Jac)
   Jac->c33 = 0. ;
   for (i = 0 ; i < Element->GeoElement->NbrNodes ; i++)
     Jac->c33 += Element->x[i] * Element->n[i] ;
-
   return(DetJac * Jac->c33) ;
 }
 
@@ -881,6 +910,24 @@ double  JacobianSur3D (struct Element * Element, MATRIX3x3 * Jac)
 		 + SQU(Jac->c13 * Jac->c21 - Jac->c11 * Jac->c23)
 		 + SQU(Jac->c12 * Jac->c23 - Jac->c13 * Jac->c22) ) ;
 
+  // regularize matrix
+  double a[3] = {Jac->c11, Jac->c12, Jac->c13};
+  double b[3] = {Jac->c21, Jac->c22, Jac->c23};
+  double c[3];
+  prodve(a, b, c);
+  norme(c);
+  Jac->c31 = c[0]; Jac->c32 = c[1]; Jac->c33 = c[2];
+
+  // make sure DetJac > 0: this is not necessary in theory, but it is
+  // required here because we use DetJac when we invert the matrix
+  double realDetJac = 
+    Jac->c11 * Jac->c22 * Jac->c33 + Jac->c13 * Jac->c21 * Jac->c32 +
+    Jac->c12 * Jac->c23 * Jac->c31 - Jac->c13 * Jac->c22 * Jac->c31 -
+    Jac->c11 * Jac->c23 * Jac->c32 - Jac->c12 * Jac->c21 * Jac->c33;
+  if(realDetJac < 0.){
+    Jac->c31 = - Jac->c31; Jac->c32 = - Jac->c32; Jac->c33 = - Jac->c33;    
+  }
+
   return(DetJac) ;
 }
 
@@ -905,6 +952,33 @@ double  JacobianLin3D (struct Element * Element, MATRIX3x3 * Jac)
 
   DetJac = sqrt(SQU(Jac->c11)+SQU(Jac->c12)+SQU(Jac->c13)) ;
 
+  // regularize matrix
+  double a[3] = {Jac->c11, Jac->c12, Jac->c13};
+  double b[3];
+  if((fabs(a[0]) >= fabs(a[1]) && fabs(a[0]) >= fabs(a[2])) ||
+     (fabs(a[1]) >= fabs(a[0]) && fabs(a[1]) >= fabs(a[2]))) {
+    b[0] = a[1]; b[1] = -a[0]; b[2] = 0.;
+  }
+  else {
+    b[0] = 0.; b[1] = a[2]; b[2] = -a[1];
+  }
+  norme(b);
+  double c[3];
+  prodve(a, b, c);
+  norme(c);
+  Jac->c21 = b[0]; Jac->c22 = b[1]; Jac->c23 = b[2];
+  Jac->c31 = c[0]; Jac->c32 = c[1]; Jac->c33 = c[2];
+
+  // make sure DetJac > 0: this is not necessary in theory, but it is
+  // required here because we use DetJac when we invert the matrix
+  double realDetJac = 
+    Jac->c11 * Jac->c22 * Jac->c33 + Jac->c13 * Jac->c21 * Jac->c32 +
+    Jac->c12 * Jac->c23 * Jac->c31 - Jac->c13 * Jac->c22 * Jac->c31 -
+    Jac->c11 * Jac->c23 * Jac->c32 - Jac->c12 * Jac->c21 * Jac->c33;
+  if(realDetJac < 0.){
+    Jac->c31 = - Jac->c31; Jac->c32 = - Jac->c32; Jac->c33 = - Jac->c33;    
+  }
+
   return(DetJac) ;
 }
 
@@ -912,98 +986,6 @@ double  JacobianLin3D (struct Element * Element, MATRIX3x3 * Jac)
 /*  G e t _ I n v e r s e M a t r i x                                       */
 /* ------------------------------------------------------------------------ */
 
-void  Get_InverseSingularMatrix(MATRIX3x3 * Mat, MATRIX3x3 * InvMat)
-{
-  double T[3][3] = {{0.,0.,0.},{0.,0.,0.},{0.,0.,0.}};
-  int i, j, k;
-#if defined(HAVE_GSL)
-  gsl_matrix *M, *V;
-  gsl_vector *W, *TMPVEC;
-  double ww;
-#else
-  double **M, **V, *W;
-#endif
-
-  InvMat->c11 = InvMat->c12 = InvMat->c13 = 0.0;
-  InvMat->c21 = InvMat->c22 = InvMat->c23 = 0.0;
-  InvMat->c31 = InvMat->c32 = InvMat->c33 = 0.0;
-
-#if defined(HAVE_GSL)
-  M = gsl_matrix_alloc(3, 3);
-  V = gsl_matrix_alloc(3, 3);
-  W = gsl_vector_alloc(3);
-  TMPVEC = gsl_vector_alloc(3);
-
-  gsl_matrix_set(M, 0, 0, Mat->c11);
-  gsl_matrix_set(M, 0, 1, Mat->c12);
-  gsl_matrix_set(M, 0, 2, Mat->c13);
-  gsl_matrix_set(M, 1, 0, Mat->c21);
-  gsl_matrix_set(M, 1, 1, Mat->c22);
-  gsl_matrix_set(M, 1, 2, Mat->c23);
-  gsl_matrix_set(M, 2, 0, Mat->c31);
-  gsl_matrix_set(M, 2, 1, Mat->c32);
-  gsl_matrix_set(M, 2, 2, Mat->c33);
-
-  gsl_linalg_SV_decomp(M, V, W, TMPVEC);
-  for(i = 0; i < 3; i++) {
-    for(j = 0; j < 3; j++) {
-      ww = gsl_vector_get(W, i);
-      if(fabs(ww) > 1.e-16) {   /* singular value precision */
-        T[i][j] += gsl_matrix_get(M, j, i) / ww;
-      }
-    }
-  }
-  for(k=0 ; k<3 ; k++){
-    InvMat->c11 += gsl_matrix_get(V, 0, k) * T[k][0] ;
-    InvMat->c12 += gsl_matrix_get(V, 0, k) * T[k][1] ;
-    InvMat->c13 += gsl_matrix_get(V, 0, k) * T[k][2] ;
-    InvMat->c21 += gsl_matrix_get(V, 1, k) * T[k][0] ;
-    InvMat->c22 += gsl_matrix_get(V, 1, k) * T[k][1] ;
-    InvMat->c23 += gsl_matrix_get(V, 1, k) * T[k][2] ;
-    InvMat->c31 += gsl_matrix_get(V, 2, k) * T[k][0] ;
-    InvMat->c32 += gsl_matrix_get(V, 2, k) * T[k][1] ;
-    InvMat->c33 += gsl_matrix_get(V, 2, k) * T[k][2] ;
-  }
-  gsl_matrix_free(M);
-  gsl_matrix_free(V);
-  gsl_vector_free(W);
-  gsl_vector_free(TMPVEC);
-#else
-  M = dmatrix(1,3,1,3);
-  V = dmatrix(1,3,1,3);
-  W = dvector(1,3);
-
-  M[1][1] = Mat->c11 ; M[1][2] = Mat->c12 ; M[1][3] = Mat->c13 ;
-  M[2][1] = Mat->c21 ; M[2][2] = Mat->c22 ; M[2][3] = Mat->c23 ;
-  M[3][1] = Mat->c31 ; M[3][2] = Mat->c32 ; M[3][3] = Mat->c33 ;
-
-  dsvdcmp(M, 3, 3, W, V);
-
-  /* cf. Numerical Recipes in C, p. 62 */
-
-  for(i=1 ; i<=3 ; i++)
-    for(j=1 ; j<=3 ; j++)
-      if(fabs(W[i]) > 1.e-16) /* precision */
-	T[i-1][j-1] += M[j][i] / W[i] ;
-  
-  for(k=1 ; k<=3 ; k++){
-    InvMat->c11 += V[1][k] * T[k-1][0] ;
-    InvMat->c12 += V[1][k] * T[k-1][1] ;
-    InvMat->c13 += V[1][k] * T[k-1][2] ;
-    InvMat->c21 += V[2][k] * T[k-1][0] ;
-    InvMat->c22 += V[2][k] * T[k-1][1] ;
-    InvMat->c23 += V[2][k] * T[k-1][2] ;
-    InvMat->c31 += V[3][k] * T[k-1][0] ;
-    InvMat->c32 += V[3][k] * T[k-1][1] ;
-    InvMat->c33 += V[3][k] * T[k-1][2] ;
-  }
-
-  free_dmatrix(M,1,3,1,3);
-  free_dmatrix(V,1,3,1,3);
-  free_dvector(W,1,3);
-#endif
-}
-
 void  Get_InverseMatrix(int Type_Dimension, int Type_Element, double DetMat,
 			MATRIX3x3 * Mat, MATRIX3x3 * InvMat)
 {
@@ -1026,56 +1008,30 @@ void  Get_InverseMatrix(int Type_Dimension, int Type_Element, double DetMat,
     break ;
 
   case _2D :
-    
-    switch(Type_Element){
-
-    case TRIANGLE   : case TRIANGLE_2   : 
-    case QUADRANGLE : case QUADRANGLE_2 : case QUADRANGLE_2_8N :
-      if(!DetMat) Msg::Error("Null determinant in 'Get_InverseMatrix'");
-      InvMat->c11 =   Mat->c22 * Mat->c33 / DetMat ;
-      InvMat->c21 = - Mat->c21 * Mat->c33 / DetMat ;
-      InvMat->c12 = - Mat->c12 * Mat->c33 / DetMat ;
-      InvMat->c22 =   Mat->c11 * Mat->c33 / DetMat ;
-      InvMat->c13 = InvMat->c23 = InvMat->c31 = InvMat->c32 = 0. ;
-      InvMat->c33 =   1. / Mat->c33 ;
-      break ;
-
-    default : 
-      Get_InverseSingularMatrix(Mat, InvMat);
-      break;
-
-    }
+    if(!DetMat) Message::Error("Null determinant in 'Get_InverseMatrix'");
+    InvMat->c11 =   Mat->c22 * Mat->c33 / DetMat ;
+    InvMat->c21 = - Mat->c21 * Mat->c33 / DetMat ;
+    InvMat->c12 = - Mat->c12 * Mat->c33 / DetMat ;
+    InvMat->c22 =   Mat->c11 * Mat->c33 / DetMat ;
+    InvMat->c13 = InvMat->c23 = InvMat->c31 = InvMat->c32 = 0. ;
+    InvMat->c33 =   1. / Mat->c33 ;
     break;
 
   case _3D :
-
-    switch(Type_Element){
-
-    case TETRAHEDRON : case TETRAHEDRON_2 : 
-    case HEXAHEDRON  : case HEXAHEDRON_2  : 
-    case PRISM       : case PRISM_2       :
-    case PYRAMID     : case PYRAMID_2     :
-      if(!DetMat) Msg::Error("Null determinant in 'Get_InverseMatrix'");
-      InvMat->c11 =  ( Mat->c22 * Mat->c33 - Mat->c23 * Mat->c32 ) / DetMat ;
-      InvMat->c21 = -( Mat->c21 * Mat->c33 - Mat->c23 * Mat->c31 ) / DetMat ;
-      InvMat->c31 =  ( Mat->c21 * Mat->c32 - Mat->c22 * Mat->c31 ) / DetMat ;
-      InvMat->c12 = -( Mat->c12 * Mat->c33 - Mat->c13 * Mat->c32 ) / DetMat ;
-      InvMat->c22 =  ( Mat->c11 * Mat->c33 - Mat->c13 * Mat->c31 ) / DetMat ;
-      InvMat->c32 = -( Mat->c11 * Mat->c32 - Mat->c12 * Mat->c31 ) / DetMat ;
-      InvMat->c13 =  ( Mat->c12 * Mat->c23 - Mat->c13 * Mat->c22 ) / DetMat ;
-      InvMat->c23 = -( Mat->c11 * Mat->c23 - Mat->c13 * Mat->c21 ) / DetMat ;
-      InvMat->c33 =  ( Mat->c11 * Mat->c22 - Mat->c12 * Mat->c21 ) / DetMat ;
-      break;
-
-    default :
-      Get_InverseSingularMatrix(Mat, InvMat);
-      break;
-
-    }
+    if(!DetMat) Message::Error("Null determinant in 'Get_InverseMatrix'");
+    InvMat->c11 =  ( Mat->c22 * Mat->c33 - Mat->c23 * Mat->c32 ) / DetMat ;
+    InvMat->c21 = -( Mat->c21 * Mat->c33 - Mat->c23 * Mat->c31 ) / DetMat ;
+    InvMat->c31 =  ( Mat->c21 * Mat->c32 - Mat->c22 * Mat->c31 ) / DetMat ;
+    InvMat->c12 = -( Mat->c12 * Mat->c33 - Mat->c13 * Mat->c32 ) / DetMat ;
+    InvMat->c22 =  ( Mat->c11 * Mat->c33 - Mat->c13 * Mat->c31 ) / DetMat ;
+    InvMat->c32 = -( Mat->c11 * Mat->c32 - Mat->c12 * Mat->c31 ) / DetMat ;
+    InvMat->c13 =  ( Mat->c12 * Mat->c23 - Mat->c13 * Mat->c22 ) / DetMat ;
+    InvMat->c23 = -( Mat->c11 * Mat->c23 - Mat->c13 * Mat->c21 ) / DetMat ;
+    InvMat->c33 =  ( Mat->c11 * Mat->c22 - Mat->c12 * Mat->c21 ) / DetMat ;
     break;
 
   default :
-    Msg::Error("Wrong dimension in 'Get_InverseMatrix'");
+    Message::Error("Wrong dimension in 'Get_InverseMatrix'");
     break ;
 
   }
@@ -1152,8 +1108,8 @@ void *Get_ChangeOfCoordinates(int Flag_ChangeCoord, int Type_Form)
 		 (void *)ChangeOfCoord_No123) ;
 
   default :
-    Msg::Error("Unknown type of field (%s)",
-	Get_StringForDefine(Field_Type, Type_Form)) ; 
+    Message::Error("Unknown type of field (%s)",
+                   Get_StringForDefine(Field_Type, Type_Form)) ; 
     return(NULL) ;
   }
 }
@@ -1188,7 +1144,7 @@ void  ChangeOfCoord_Form1(struct Element * Element,
 void  ChangeOfCoord_Form2(struct Element * Element,
 			  double vBFu[], double vBFx[])
 {
-  if(!Element->DetJac) Msg::Error("Null determinant in 'ChangeOfCoord_Form2'");
+  if(!Element->DetJac) Message::Error("Null determinant in 'ChangeOfCoord_Form2'");
 
   vBFx[0] = (vBFu[0] * Element->Jac.c11 + vBFu[1] * Element->Jac.c21
 	     + vBFu[2] * Element->Jac.c31) / Element->DetJac ;
@@ -1201,7 +1157,7 @@ void  ChangeOfCoord_Form2(struct Element * Element,
 void  ChangeOfCoord_Form3(struct Element * Element,
 			  double vBFu[], double vBFx[])
 {
-  if(!Element->DetJac) Msg::Error("Null determinant in 'ChangeOfCoord_Form3'");
+  if(!Element->DetJac) Message::Error("Null determinant in 'ChangeOfCoord_Form3'");
 
   vBFx[0] = vBFu[0] / Element->DetJac ;
 }
@@ -1219,7 +1175,7 @@ void  ChangeOfCoord_Form1P(struct Element * Element,
 void  ChangeOfCoord_Form2P(struct Element * Element,
 			   double vBFu[], double vBFx[])
 {
-  if(!Element->DetJac) Msg::Error("Null determinant in 'ChangeOfCoord_Form2P'");
+  if(!Element->DetJac) Message::Error("Null determinant in 'ChangeOfCoord_Form2P'");
 
   vBFx[0] = (vBFu[0] * Element->Jac.c11 + vBFu[1] * Element->Jac.c21)
     / Element->DetJac ;
@@ -1231,7 +1187,7 @@ void  ChangeOfCoord_Form2P(struct Element * Element,
 void  ChangeOfCoord_Form1S(struct Element * Element,
 			   double vBFu[], double vBFx[])
 {
-  if(!Element->DetJac) Msg::Error("Null determinant in 'ChangeOfCoord_Form1S'");
+  if(!Element->DetJac) Message::Error("Null determinant in 'ChangeOfCoord_Form1S'");
 
   vBFx[0] = 0. ;  
   vBFx[1] = 0. ;
