@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -11,10 +11,12 @@
 
 #if !defined(HAVE_PETSC) && !defined(HAVE_SPARSKIT)
 
-#define err Msg::Error("No solver is compiled in this version of GetDP")
+#define err Message::Error("No solver is compiled in this version of GetDP")
 
 void LinAlg_InitializeSolver(int* argc, char*** argv){}
 void LinAlg_FinalizeSolver(){}
+void LinAlg_SetCommSelf(){}
+void LinAlg_SetCommWorld(){}
 void LinAlg_CreateSolver(gSolver *Solver, const char * SolverDataFileName){ err; }
 void LinAlg_CreateVector(gVector *V, gSolver *Solver, int n){ err; }
 void LinAlg_CreateMatrix(gMatrix *M, gSolver *Solver, int n, int m){ err; }
@@ -34,8 +36,10 @@ void LinAlg_ReadScalar(FILE *file, gScalar *S){ err; }
 void LinAlg_ReadVector(FILE *file, gVector *V){ err; }
 void LinAlg_ReadMatrix(FILE *file, gMatrix *M){ err; }
 void LinAlg_PrintScalar(FILE *file, gScalar *S){ err; }
-void LinAlg_PrintVector(FILE *file, gVector *V, bool matlab){ err; }
-void LinAlg_PrintMatrix(FILE *file, gMatrix *M, bool matlab){ err; }
+void LinAlg_PrintVector(FILE *file, gVector *V, bool matlab,
+                        const char* fileName, const char* varName){ err; }
+void LinAlg_PrintMatrix(FILE *file, gMatrix *M, bool matlab,
+                        const char* fileName, const char*varName){ err; }
 void LinAlg_WriteScalar(FILE *file, gScalar *S){ err; }
 void LinAlg_WriteVector(FILE *file, gVector *V){ err; }
 void LinAlg_WriteMatrix(FILE *file, gMatrix *M){ err; }
@@ -96,5 +100,7 @@ void LinAlg_AssembleMatrix(gMatrix *M){ err; }
 void LinAlg_AssembleVector(gVector *V){ err; }
 void LinAlg_Solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X, int solverIndex){ err; }
 void LinAlg_SolveAgain(gMatrix *A, gVector *B, gSolver *Solver, gVector *X, int solverIndex){ err; }
+void LinAlg_SolveNL(gMatrix *A, gVector *B, gMatrix *J, gVector *R, gSolver *Solver,
+                    gVector *X, int solverIndex){ err; }
 
 #endif
