@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -33,6 +33,7 @@ typedef struct { Solver_Params Params ; } gSolver ;
   #error "GetDP requires PETSc version 2.3 or higher"
 #else
   #include "petscksp.h"
+  #include "petscsnes.h"
 #endif
 #if defined(PETSC_USE_COMPLEX)
   #define gSCALAR_SIZE 2
@@ -44,7 +45,7 @@ typedef struct { Solver_Params Params ; } gSolver ;
 typedef struct { PetscScalar s ; } gScalar ;
 typedef struct { Mat M ; }         gMatrix ;
 typedef struct { Vec V, Vseq ; }   gVector ;
-typedef struct { KSP ksp[10] ; }   gSolver ;
+typedef struct { KSP ksp[10] ;  SNES snes[10] ; }   gSolver ;
 
 #else
 
@@ -59,6 +60,8 @@ typedef struct { int dummy; }   gSolver ;
 
 void LinAlg_InitializeSolver(int* argc, char*** argv);
 void LinAlg_FinalizeSolver(void);
+void LinAlg_SetCommSelf();
+void LinAlg_SetCommWorld();
 
 void LinAlg_CreateSolver(gSolver *Solver, const char * SolverDataFileName);
 void LinAlg_CreateVector(gVector *V, gSolver *Solver, int n);
@@ -85,8 +88,10 @@ void LinAlg_ReadVector(FILE *file, gVector *V);
 void LinAlg_ReadMatrix(FILE *file, gMatrix *M);
 
 void LinAlg_PrintScalar(FILE *file, gScalar *S);
-void LinAlg_PrintVector(FILE *file, gVector *V, bool matlab=false);
-void LinAlg_PrintMatrix(FILE *file, gMatrix *M, bool matlab=false);
+void LinAlg_PrintVector(FILE *file, gVector *V, bool matlab=false,
+                        const char* fileName="vector.m", const char* varName="Vec_0");
+void LinAlg_PrintMatrix(FILE *file, gMatrix *M, bool matlab=false,
+                        const char* fileName="matrix.m", const char* varName="Mat_0");
 
 void LinAlg_WriteScalar(FILE *file, gScalar *S);
 void LinAlg_WriteVector(FILE *file, gVector *V);
@@ -157,4 +162,7 @@ void LinAlg_AssembleVector(gVector *V);
 void LinAlg_Solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X, int solverIndex=0);
 void LinAlg_SolveAgain(gMatrix *A, gVector *B, gSolver *Solver, gVector *X, int solverIndex=0);
 
+void LinAlg_SolveNL(gMatrix *A, gVector *B, gMatrix *Jac, gVector *R, gSolver *Solver,
+                    gVector *X, int solverIndex=0);
+
 #endif
