@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -33,7 +33,7 @@ extern struct CurrentData Current ;
 #if defined(HAVE_PETSC)
 
 // Options for PETSc can be provided on the command line, or in the file
-// ~/.petscrc. 
+// ~/.petscrc.
 //
 // By default we try to use the MUMPS or UMFPACK direct solver (if
 // available, with PETSc 3). Otherwise we use a GMRES iterative solver
@@ -59,26 +59,36 @@ extern struct CurrentData Current ;
 // e.g if the matrix changed a bit, but you want to keep the same
 // factorization using "SolveAgain").
 
-static void _try(int ierr){ CHKERRABORT(PETSC_COMM_WORLD, ierr); }
+static MPI_Comm MyComm = MPI_COMM_SELF;
+static PetscViewer MyPetscViewer;
+
+static void _try(int ierr){ CHKERRABORT(MyComm, ierr); }
 static int SolverInitialized = 0;
 
+#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)))
+#define PetscTruth PetscBool
+#define PetscOptionsGetTruth PetscOptionsGetBool
+#endif
+
 void LinAlg_InitializeSolver(int* argc, char*** argv)
 {
   // This function detects if MPI is initialized
   PetscInitialize(argc, argv, PETSC_NULL, PETSC_NULL);
+  MyPetscViewer = PETSC_VIEWER_STDOUT_SELF;
+  MyComm = PETSC_COMM_WORLD;
 #if defined(HAVE_SLEPC)
   SlepcInitialize(argc, argv, PETSC_NULL, PETSC_NULL);
 #endif
   SolverInitialized = 1;
 
   // get additional petsc options from specified file (useful e.g. on
-  // Windows where we don't know where to search for ~/.petsrc)
+  // Windows where we don't know where to search for ~/.petscrc)
   for(int i = 0; i < *argc - 1; i++){
     if (!strcmp((*argv)[i], "-solver")){
 #if (PETSC_VERSION_MAJOR == 2)
       PetscOptionsInsertFile((*argv)[i+1]);
 #else
-      PetscOptionsInsertFile(PETSC_COMM_WORLD, (*argv)[i+1], PETSC_FALSE);
+      PetscOptionsInsertFile(MyComm, (*argv)[i+1], PETSC_FALSE);
 #endif
     }
   }
@@ -94,40 +104,66 @@ void LinAlg_FinalizeSolver()
   }
 }
 
+void LinAlg_SetCommSelf()
+{
+  MyComm = PETSC_COMM_SELF;
+  Message::SetIsCommWorld(0);
+}
+
+void LinAlg_SetCommWorld()
+{
+  MyComm = PETSC_COMM_WORLD;
+  Message::SetIsCommWorld(1);
+}
+
 void LinAlg_CreateSolver(gSolver *Solver, const char *SolverDataFileName)
 {
-  for(int i = 0; i < 10; i++) Solver->ksp[i] = NULL;
+  for(int i = 0; i < 10; i++){
+    Solver->ksp[i] = NULL;
+    Solver->snes[i] = NULL;
+  }
 }
 
 void LinAlg_CreateVector(gVector *V, gSolver *Solver, int n)
 {
-  _try(VecCreate(PETSC_COMM_WORLD, &V->V));
+  _try(VecCreate(MyComm, &V->V));
   _try(VecSetSizes(V->V, PETSC_DECIDE, n));
+
   // override the default options with the ones from the option
   // database (if any)
   _try(VecSetFromOptions(V->V));
 
   // create sequential vector that will contain all the values on all
   // the procs
-  if(Msg::GetCommSize() > 1)
+  if(Message::GetCommSize() > 1)
     _try(VecCreateSeq(PETSC_COMM_SELF, n, &V->Vseq));
 }
 
+void _fillseq(Vec &V, Vec &Vseq)
+{
+  // collect all the values from the parallel petsc vector into a sequential
+  // vector on each processor
+  VecScatter ctx;
+  VecScatterCreateToAll(V, &ctx, &Vseq);
+#if (PETSC_VERSION_MAJOR == 2) && (PETSC_VERSION_MINOR == 3) && (PETSC_VERSION_SUBMINOR < 3)
+  VecScatterBegin(V, Vseq, INSERT_VALUES, SCATTER_FORWARD, ctx);
+  VecScatterEnd(V, Vseq, INSERT_VALUES, SCATTER_FORWARD, ctx);
+#else
+  VecScatterBegin(ctx, V, Vseq, INSERT_VALUES, SCATTER_FORWARD);
+  VecScatterEnd(ctx, V, Vseq, INSERT_VALUES, SCATTER_FORWARD);
+#endif
+
+#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)) )
+  VecScatterDestroy(&ctx);
+#else
+  VecScatterDestroy(ctx);
+#endif
+}
+
 static void _fillseq(gVector *V)
 {
-  if(Msg::GetCommSize() == 1) return;
-  // collect all the values from the parallel petsc vector into a
-  // sequential vector on each processor
-  VecScatter ctx;
-  VecScatterCreateToAll(V->V, &ctx, &V->Vseq);
-#if (PETSC_VERSION_MAJOR == 2) && (PETSC_VERSION_MINOR == 3) && (PETSC_VERSION_SUBMINOR < 3) 
-  VecScatterBegin(V->V, V->Vseq, INSERT_VALUES, SCATTER_FORWARD, ctx);
-  VecScatterEnd(V->V, V->Vseq, INSERT_VALUES, SCATTER_FORWARD, ctx);
-#else
-  VecScatterBegin(ctx, V->V, V->Vseq, INSERT_VALUES, SCATTER_FORWARD);
-  VecScatterEnd(ctx, V->V, V->Vseq, INSERT_VALUES, SCATTER_FORWARD);
-#endif
-  VecScatterDestroy(ctx);
+  if(Message::GetCommSize() == 1) return;
+  _fillseq(V->V, V->Vseq);
 }
 
 void LinAlg_CreateMatrix(gMatrix *M, gSolver *Solver, int n, int m)
@@ -137,7 +173,7 @@ void LinAlg_CreateMatrix(gMatrix *M, gSolver *Solver, int n, int m)
   PetscOptionsGetInt(PETSC_NULL, "-petsc_prealloc", &prealloc, &set);
 
   // prealloc cannot be bigger than the number of rows!
-  prealloc = (n < prealloc) ? n : prealloc; 
+  prealloc = (n < prealloc) ? n : prealloc;
   std::vector<PetscInt> nnz(n, prealloc);
 
   // preallocate non local equations as full lines (this is not
@@ -147,11 +183,16 @@ void LinAlg_CreateMatrix(gMatrix *M, gSolver *Solver, int n, int m)
   for(unsigned int i = 0; i < Current.DofData->NonLocalEquations.size(); i++)
     nnz[Current.DofData->NonLocalEquations[i] - 1] = n;
 
-  if(Msg::GetCommSize() > 1) // FIXME: alloc full lines...
-    _try(MatCreateMPIAIJ(PETSC_COMM_WORLD, PETSC_DECIDE, PETSC_DECIDE, n, m, 
+  if(Message::GetCommSize() > 1) // FIXME: alloc full lines...
+#if ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR == 3))
+    _try(MatCreateAIJ(MyComm, PETSC_DECIDE, PETSC_DECIDE, n, m,
+                      prealloc, PETSC_NULL, prealloc, PETSC_NULL, &M->M));
+#else
+ _try(MatCreateMPIAIJ(MyComm, PETSC_DECIDE, PETSC_DECIDE, n, m,
                          prealloc, PETSC_NULL, prealloc, PETSC_NULL, &M->M));
+#endif
   else
-    _try(MatCreateSeqAIJ(PETSC_COMM_WORLD, n, m, 0, &nnz[0], &M->M));
+    _try(MatCreateSeqAIJ(PETSC_COMM_SELF, n, m, 0, &nnz[0], &M->M));
 
   // override the default options with the ones from the option
   // database (if any)
@@ -160,20 +201,37 @@ void LinAlg_CreateMatrix(gMatrix *M, gSolver *Solver, int n, int m)
 
 void LinAlg_DestroySolver(gSolver *Solver)
 {
-  for(int i = 0; i < 10; i++)
+  for(int i = 0; i < 10; i++){
+#if (PETSC_VERSION_RELEASE == 0  || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)) )
+    if(Solver->ksp[i]) _try(KSPDestroy(&Solver->ksp[i]));
+    if(Solver->snes[i]) _try(SNESDestroy(&Solver->snes[i]));
+#else
     if(Solver->ksp[i]) _try(KSPDestroy(Solver->ksp[i]));
+    if(Solver->snes[i]) _try(SNESDestroy(Solver->snes[i]));
+#endif
+  }
 }
 
 void LinAlg_DestroyVector(gVector *V)
 {
+#if (PETSC_VERSION_RELEASE == 0  || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)))
+  _try(VecDestroy(&V->V));
+  if(Message::GetCommSize() > 1)
+    _try(VecDestroy(&V->Vseq));
+#else
   _try(VecDestroy(V->V));
-  if(Msg::GetCommSize() > 1)
+  if(Message::GetCommSize() > 1)
     _try(VecDestroy(V->Vseq));
+#endif
 }
 
 void LinAlg_DestroyMatrix(gMatrix *M)
 {
+#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)) )
+  _try(MatDestroy(&M->M));
+#else
   _try(MatDestroy(M->M));
+#endif
 }
 
 void LinAlg_CopyScalar(gScalar *S1, gScalar *S2)
@@ -184,7 +242,7 @@ void LinAlg_CopyScalar(gScalar *S1, gScalar *S2)
 void LinAlg_CopyVector(gVector *V1, gVector *V2)
 {
   _try(VecCopy(V1->V, V2->V));
-  if(Msg::GetCommSize() > 1)
+  if(Message::GetCommSize() > 1)
     _try(VecCopy(V1->Vseq, V2->Vseq));
 }
 
@@ -203,7 +261,7 @@ void LinAlg_ZeroVector(gVector *V)
   PetscScalar zero = 0.0;
 
   _try(VecSet(V->V, zero));
-  if(Msg::GetCommSize() > 1)
+  if(Message::GetCommSize() > 1)
     _try(VecSet(V->Vseq, zero));
 }
 
@@ -225,9 +283,9 @@ void LinAlg_ScanScalar(FILE *file, gScalar *S)
 
 void LinAlg_ScanVector(FILE *file, gVector *V)
 {
-  if(Msg::GetCommSize() > 1)
-    Msg::Error("ScanVector not implemented in parallel");
-  PetscInt n;  
+  if(Message::GetCommSize() > 1)
+    Message::Error("ScanVector not implemented in parallel");
+  PetscInt n;
   _try(VecGetSize(V->V, &n));
   for(PetscInt i = 0; i < n; i++){
     double a, b;
@@ -245,18 +303,18 @@ void LinAlg_ScanVector(FILE *file, gVector *V)
 
 void LinAlg_ScanMatrix(FILE *file, gMatrix *M)
 {
-  Msg::Error("ScanMatrix not yet implemented");  
+  Message::Error("ScanMatrix not yet implemented");
 }
 
 void LinAlg_ReadScalar(FILE *file, gScalar *S)
 {
-  Msg::Error("ReadScalar not yet implemented");
+  Message::Error("ReadScalar not yet implemented");
 }
 
 void LinAlg_ReadVector(FILE *file, gVector *V)
 {
-  if(Msg::GetCommSize() > 1)
-    Msg::Error("ReadVector not implemented in parallel");
+  if(Message::GetCommSize() > 1)
+    Message::Error("ReadVector not implemented in parallel");
   PetscInt n;
   _try(VecGetSize(V->V, &n));
   PetscScalar *tmp = (PetscScalar*)Malloc(n*sizeof(PetscScalar));
@@ -268,7 +326,7 @@ void LinAlg_ReadVector(FILE *file, gVector *V)
 
 void LinAlg_ReadMatrix(FILE *file, gMatrix *M)
 {
-  Msg::Error("ReadMatrix not yet implemented");  
+  Message::Error("ReadMatrix not yet implemented");
 }
 
 void LinAlg_PrintScalar(FILE *file, gScalar *S)
@@ -280,12 +338,13 @@ void LinAlg_PrintScalar(FILE *file, gScalar *S)
 #endif
 }
 
-void LinAlg_PrintVector(FILE *file, gVector *V, bool matlab)
+void LinAlg_PrintVector(FILE *file, gVector *V, bool matlab,
+                        const char* fileName, const char* varName)
 {
   if(!matlab){
     PetscInt n;
     _try(VecGetSize(V->V, &n));
-    Vec VV = Msg::GetCommSize() > 1 ? V->Vseq : V->V;
+    Vec VV = Message::GetCommSize() > 1 ? V->Vseq : V->V;
     PetscScalar *tmp;
     _try(VecGetArray(VV, &tmp));
     for (int i = 0; i < n; i++){
@@ -300,33 +359,74 @@ void LinAlg_PrintVector(FILE *file, gVector *V, bool matlab)
   }
   else{
     PetscViewer fd;
-    _try(PetscViewerASCIIOpen(PETSC_COMM_WORLD, "vector.m", &fd));
+    _try(PetscViewerASCIIOpen(MyComm, fileName, &fd));
     _try(PetscViewerSetFormat(fd, PETSC_VIEWER_ASCII_MATLAB));
+    _try(PetscObjectSetName((PetscObject)V->V, varName));
     _try(VecView(V->V, fd));
+#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)))
+    _try(PetscViewerDestroy(&fd));
+#else
     _try(PetscViewerDestroy(fd));
+#endif
   }
-} 
+}
 
-void LinAlg_PrintMatrix(FILE *file, gMatrix *M, bool matlab)
+void LinAlg_PrintMatrix(FILE *file, gMatrix *M, bool matlab,
+                        const char* fileName, const char* varName)
 {
-  if(!matlab) Msg::Error("Non-matlab output not available for this matrix");
-  PetscViewer fd;
-  _try(PetscViewerASCIIOpen(PETSC_COMM_WORLD, "matrix.m", &fd));
-  _try(PetscViewerSetFormat(fd, PETSC_VIEWER_ASCII_MATLAB));
-  _try(MatView(M->M, fd));
-  _try(PetscViewerDestroy(fd));
+  if(!matlab){
+    PetscInt n, m;
+    _try(MatGetSize(M->M, &n, &m));
+    for(int i = 0; i < n; i++){
+      PetscInt ncols;
+      const PetscInt *cols;
+      const PetscScalar *vals;
+      _try(MatGetRow(M->M, i, &ncols, &cols, &vals));
+      for(int j = 0; j < m; j++){
+#if defined(PETSC_USE_COMPLEX)
+        fprintf(file, "[%d, %d] %.16g %.16g\n", i, j, real(vals[j]), imag(vals[j]) );
+#else
+        fprintf(file, "[%d, %d] %.16g\n", i, j, vals[j]);
+#endif
+      }
+      _try(MatRestoreRow(M->M, i, &ncols, &cols, &vals));
+    }
+  }
+  else{
+    // ASCII
+    PetscViewer fd;
+    _try(PetscViewerASCIIOpen(MyComm, fileName, &fd));
+    _try(PetscViewerSetFormat(fd, PETSC_VIEWER_ASCII_MATLAB));
+    _try(PetscObjectSetName((PetscObject)M->M, varName));
+    _try(MatView(M->M, fd));
+
+    // Binary
+    PetscViewer fd2;
+    std::string tmp(fileName);
+    _try(PetscViewerBinaryOpen(MyComm, (tmp + ".bin").c_str(), FILE_MODE_WRITE, &fd2));
+    _try(PetscViewerSetFormat(fd2, PETSC_VIEWER_DEFAULT));
+    _try(MatView(M->M, fd2));
+
+#if (PETSC_VERSION_RELEASE == 0 || ((PETSC_VERSION_MAJOR == 3) && (PETSC_VERSION_MINOR >= 2)))
+    _try(PetscViewerDestroy(&fd));
+    _try(PetscViewerDestroy(&fd2));
+#else
+    _try(PetscViewerDestroy(fd));
+    _try(PetscViewerDestroy(fd2));
+#endif
+  }
 }
 
 void LinAlg_WriteScalar(FILE *file, gScalar *S)
 {
-  Msg::Error("WriteScalar not yet implemented");
+  Message::Error("WriteScalar not yet implemented");
 }
 
 void LinAlg_WriteVector(FILE *file, gVector *V)
 {
   PetscInt n;
   _try(VecGetSize(V->V, &n));
-  Vec VV = Msg::GetCommSize() > 1 ? V->Vseq : V->V;
+  Vec VV = Message::GetCommSize() > 1 ? V->Vseq : V->V;
   PetscScalar *tmp;
   _try(VecGetArray(VV, &tmp));
   fwrite(tmp, sizeof(PetscScalar), n, file);
@@ -336,14 +436,14 @@ void LinAlg_WriteVector(FILE *file, gVector *V)
 
 void LinAlg_WriteMatrix(FILE *file, gMatrix *M)
 {
-  Msg::Error("WriteMatrix not yet implemented");  
+  Message::Error("WriteMatrix not yet implemented");
 }
 
 void LinAlg_GetVectorSize(gVector *V, int *i)
 {
   PetscInt t;
   _try(VecGetSize(V->V, &t));
-  if(t > INT_MAX) Msg::Error("Problem too big");
+  if(t > INT_MAX) Message::Error("Problem too big");
   *i = t;
 }
 
@@ -351,14 +451,14 @@ void LinAlg_GetLocalVectorRange(gVector *V, int *low, int *high)
 {
   PetscInt tlow, thigh;
   _try(VecGetOwnershipRange(V->V, &tlow, &thigh));
-  if(tlow > INT_MAX || thigh > INT_MAX) Msg::Error("Problem too big");
+  if(tlow > INT_MAX || thigh > INT_MAX) Message::Error("Problem too big");
   *low = tlow;
   *high = thigh;
 }
 
 static bool _isInLocalRange(gVector *V, int i)
 {
-  if(Msg::GetCommSize() == 1) return true;
+  if(Message::GetCommSize() == 1) return true;
   int imin, imax;
   LinAlg_GetLocalVectorRange(V, &imin, &imax);
   return (i >= imin && i < imax);
@@ -368,7 +468,7 @@ void LinAlg_GetMatrixSize(gMatrix *M, int *i, int *j)
 {
   PetscInt ti, tj;
   _try(MatGetSize(M->M, &ti, &tj));
-  if(ti > INT_MAX || tj > INT_MAX) Msg::Error("Problem too big");
+  if(ti > INT_MAX || tj > INT_MAX) Message::Error("Problem too big");
   *i = ti;
   *j = tj;
 }
@@ -377,14 +477,14 @@ void LinAlg_GetLocalMatrixRange(gMatrix *M, int *low, int *high)
 {
   PetscInt tlow, thigh;
   _try(MatGetOwnershipRange(M->M, &tlow, &thigh));
-  if(tlow > INT_MAX || thigh > INT_MAX) Msg::Error("Problem too big");
+  if(tlow > INT_MAX || thigh > INT_MAX) Message::Error("Problem too big");
   *low = tlow;
   *high = thigh;
 }
 
 static bool _isInLocalRange(gMatrix *M, int i)
 {
-  if(Msg::GetCommSize() == 1) return true;
+  if(Message::GetCommSize() == 1) return true;
   int imin, imax;
   LinAlg_GetLocalMatrixRange(M, &imin, &imax);
   return (i >= imin && i < imax);
@@ -405,13 +505,13 @@ void LinAlg_GetComplexInScalar(double *d1, double *d2, gScalar *S)
   *d1 = real(S->s);
   *d2 = imag(S->s);
 #else
-  Msg::Error("'LinAlg_GetComplexInScalar' not available with this Solver");  
+  Message::Error("'LinAlg_GetComplexInScalar' not available with this Solver");
 #endif
 }
 
 void LinAlg_GetScalarInVector(gScalar *S, gVector *V, int i)
 {
-  Vec VV = Msg::GetCommSize() > 1 ? V->Vseq : V->V;
+  Vec VV = Message::GetCommSize() > 1 ? V->Vseq : V->V;
   PetscScalar *tmp;
   _try(VecGetArray(VV, &tmp));
   S->s = tmp[i];
@@ -420,7 +520,7 @@ void LinAlg_GetScalarInVector(gScalar *S, gVector *V, int i)
 
 void LinAlg_GetDoubleInVector(double *d, gVector *V, int i)
 {
-  Vec VV = Msg::GetCommSize() > 1 ? V->Vseq : V->V;
+  Vec VV = Message::GetCommSize() > 1 ? V->Vseq : V->V;
   PetscScalar *tmp;
   _try(VecGetArray(VV, &tmp));
 #if defined(PETSC_USE_COMPLEX)
@@ -433,7 +533,7 @@ void LinAlg_GetDoubleInVector(double *d, gVector *V, int i)
 
 void LinAlg_GetAbsDoubleInVector(double *d, gVector *V, int i)
 {
-  Vec VV = Msg::GetCommSize() > 1 ? V->Vseq : V->V;
+  Vec VV = Message::GetCommSize() > 1 ? V->Vseq : V->V;
   PetscScalar *tmp;
   _try(VecGetArray(VV, &tmp));
 #if defined(PETSC_USE_COMPLEX)
@@ -446,7 +546,7 @@ void LinAlg_GetAbsDoubleInVector(double *d, gVector *V, int i)
 
 void LinAlg_GetComplexInVector(double *d1, double *d2, gVector *V, int i, int j)
 {
-  Vec VV = Msg::GetCommSize() > 1 ? V->Vseq : V->V;
+  Vec VV = Message::GetCommSize() > 1 ? V->Vseq : V->V;
   PetscScalar *tmp;
   _try(VecGetArray(VV, &tmp));
 #if defined(PETSC_USE_COMPLEX)
@@ -461,22 +561,40 @@ void LinAlg_GetComplexInVector(double *d1, double *d2, gVector *V, int i, int j)
 
 void LinAlg_GetScalarInMatrix(gScalar *S, gMatrix *M, int i, int j)
 {
-  Msg::Error("GetScalarInMatrix not yet implemented");  
+  if(!_isInLocalRange(M, i)) return;
+  PetscInt ti = i, tj = j;
+  _try(MatGetValues(M->M, 1, &ti, 1, &tj, &S->s));
 }
 
 void LinAlg_GetDoubleInMatrix(double *d, gMatrix *M, int i, int j)
 {
-  Msg::Error("GetDoubleInMatrix not yet implemented");  
+  if(!_isInLocalRange(M, i)) return;
+  PetscInt ti = i, tj = j;
+  _try(MatGetValues(M->M, 1, &ti, 1, &tj, (PetscScalar*)d));
 }
 
 void LinAlg_GetComplexInMatrix(double *d1, double *d2, gMatrix *M, int i, int j, int k, int l)
 {
-  Msg::Error("GetComplexInMatrix not yet implemented");  
+  PetscScalar tmp;
+#if defined(PETSC_USE_COMPLEX)
+  PetscInt ti = i, tj = j;
+  if(_isInLocalRange(M, i)){
+    _try(MatGetValues(M->M, 1, &ti, 1, &tj, &tmp));
+    *d1 = real(tmp) ;
+    *d2 = imag(tmp) ;
+  }
+#else
+  PetscInt ti = i, tj = j, tk = k, tl = l;
+  if(_isInLocalRange(M, i))
+    _try(MatGetValues(M->M, 1, &ti, 1, &tj, (PetscScalar*)d1));
+  if(_isInLocalRange(M, k))
+    _try(MatGetValues(M->M, 1, &tk, 1, &tj, (PetscScalar*)d2));
+#endif
 }
 
 void LinAlg_GetColumnInMatrix(gMatrix *M, int col, gVector *V1)
 {
-  Msg::Error("GetColumnInMatrix not yet implemented");  
+  Message::Error("GetColumnInMatrix not yet implemented");
 }
 
 void LinAlg_SetScalar(gScalar *S, double *d)
@@ -492,7 +610,7 @@ void LinAlg_SetVector(gVector *V, double *v)
 {
   PetscScalar tmp = *v;
   _try(VecSet(V->V, tmp));
-  if(Msg::GetCommSize() > 1)
+  if(Message::GetCommSize() > 1)
     _try(VecSet(V->Vseq, tmp));
 }
 
@@ -542,12 +660,40 @@ void LinAlg_SetScalarInMatrix(gScalar *S, gMatrix *M, int i, int j)
 
 void LinAlg_SetDoubleInMatrix(double d, gMatrix *M, int i, int j)
 {
-  Msg::Error("SetDoubleInMatrix not yet implemented");  
+  if(!_isInLocalRange(M, i)) return;
+  PetscInt ti = i, tj = j;
+  _try(MatSetValues(M->M, 1, &ti, 1, &tj, (PetscScalar*)&d, INSERT_VALUES));
 }
 
 void LinAlg_SetComplexInMatrix(double d1, double d2, gMatrix *M, int i, int j, int k, int l)
 {
-  Msg::Error("SetComplexInMatrix not yet implemented");  
+  PetscScalar tmp;
+#if defined(PETSC_USE_COMPLEX)
+  PetscInt ti = i, tj = j;
+  if(_isInLocalRange(M, i)){
+    tmp = d1 + PETSC_i * d2;
+    _try(MatSetValues(M->M, 1, &ti, 1, &tj, &tmp, INSERT_VALUES));
+  }
+#else
+  PetscInt ti = i, tj = j, tk = k, tl = l;
+  if(d1){
+    tmp = d1;
+    if(_isInLocalRange(M, i))
+      _try(MatSetValues(M->M, 1, &ti, 1, &tj, &tmp, INSERT_VALUES));
+    if(_isInLocalRange(M, k))
+      _try(MatSetValues(M->M, 1, &tk, 1, &tl, &tmp, INSERT_VALUES));
+  }
+  if(d2){
+    if(_isInLocalRange(M, i)){
+      tmp = -d2;
+      _try(MatSetValues(M->M, 1, &ti, 1, &tl, &tmp, INSERT_VALUES));
+    }
+    if(_isInLocalRange(M, k)){
+      tmp = d2;
+      _try(MatSetValues(M->M, 1, &tk, 1, &tj, &tmp, INSERT_VALUES));
+    }
+  }
+#endif
 }
 
 void LinAlg_AddScalarScalar(gScalar *S1, gScalar *S2, gScalar *S3)
@@ -558,7 +704,7 @@ void LinAlg_AddScalarScalar(gScalar *S1, gScalar *S2, gScalar *S3)
 void LinAlg_DummyVector(gVector *V)
 {
   if(Current.DofData->DummyDof)
-    Msg::Error("DummyVector not yet implemented");
+    Message::Error("DummyVector not yet implemented");
   return;
 }
 
@@ -657,7 +803,7 @@ void LinAlg_AddVectorVector(gVector *V1, gVector *V2, gVector *V3)
     _fillseq(V2);
   }
   else
-    Msg::Error("Wrong arguments in 'LinAlg_AddVectorVector'");  
+    Message::Error("Wrong arguments in 'LinAlg_AddVectorVector'");
 }
 
 void LinAlg_AddVectorProdVectorDouble(gVector *V1, gVector *V2, double d, gVector *V3)
@@ -672,7 +818,7 @@ void LinAlg_AddVectorProdVectorDouble(gVector *V1, gVector *V2, double d, gVecto
     _fillseq(V2);
   }
   else
-    Msg::Error("Wrong arguments in 'LinAlg_AddVectorProdVectorDouble'");  
+    Message::Error("Wrong arguments in 'LinAlg_AddVectorProdVectorDouble'");
 }
 
 void LinAlg_AddMatrixMatrix(gMatrix *M1, gMatrix *M2, gMatrix *M3)
@@ -683,7 +829,7 @@ void LinAlg_AddMatrixMatrix(gMatrix *M1, gMatrix *M2, gMatrix *M3)
   else if(M3 == M2)
     _try(MatAXPY(M2->M, tmp, M1->M, DIFFERENT_NONZERO_PATTERN));
   else
-    Msg::Error("Wrong arguments in 'LinAlg_AddMatrixMatrix'");  
+    Message::Error("Wrong arguments in 'LinAlg_AddMatrixMatrix'");
 }
 
 void LinAlg_AddMatrixProdMatrixDouble(gMatrix *M1, gMatrix *M2, double d, gMatrix *M3)
@@ -698,7 +844,7 @@ void LinAlg_AddMatrixProdMatrixDouble(gMatrix *M1, gMatrix *M2, double d, gMatri
     _try(MatAYPX(M2->M, tmp, M1->M, DIFFERENT_NONZERO_PATTERN));
 #endif
   else
-    Msg::Error("Wrong arguments in 'LinAlg_AddMatrixProdMatrixDouble'");
+    Message::Error("Wrong arguments in 'LinAlg_AddMatrixProdMatrixDouble'");
 }
 
 void LinAlg_SubScalarScalar(gScalar *S1, gScalar *S2, gScalar *S3)
@@ -710,20 +856,27 @@ void LinAlg_SubVectorVector(gVector *V1, gVector *V2, gVector *V3)
 {
   PetscScalar tmp = -1.0;
   if(V3 == V1){
-    _try(VecAXPY(V1->V, tmp, V2->V));
+    _try(VecAXPY(V1->V, tmp, V2->V)); // V1->V = V1->V - V2->V
     _fillseq(V1);
   }
   else if(V3 == V2){
-    _try(VecAYPX(V2->V, tmp, V1->V));
+    _try(VecAYPX(V2->V, tmp, V1->V)); // V2->V = V1->V - V2->V
     _fillseq(V2);
   }
   else
-    Msg::Error("Wrong arguments in 'LinAlg_SubVectorVector'");  
+    Message::Error("Wrong arguments in 'LinAlg_SubVectorVector'");
 }
 
 void LinAlg_SubMatrixMatrix(gMatrix *M1, gMatrix *M2, gMatrix *M3)
 {
-  Msg::Error("SubMatrixMatrix not yet implemented");    
+
+  PetscScalar tmp = -1.0;
+  if(M3 == M1)
+    _try(MatAXPY(M1->M, tmp, M2->M, DIFFERENT_NONZERO_PATTERN)); // M1->M = M1->M - M2->M
+  else if(M3 == M2)
+    _try(MatAYPX(M2->M, tmp, M1->M, DIFFERENT_NONZERO_PATTERN)); // M2->M = M1->M - M2->M
+  else
+    Message::Error("Wrong arguments in 'LinAlg_SubMatrixMatrix'");
 }
 
 void LinAlg_ProdScalarScalar(gScalar *S1, gScalar *S2, gScalar *S3)
@@ -753,13 +906,13 @@ void LinAlg_ProdScalarComplex(gScalar *S, double d1, double d2, double *d3, doub
 }
 
 void LinAlg_ProdVectorScalar(gVector *V1, gScalar *S, gVector *V2)
-{ 
+{
   if(V2 == V1){
     _try(VecScale(V1->V, S->s));
     _fillseq(V1);
   }
   else
-    Msg::Error("Wrong arguments in 'LinAlg_ProdVectorScalar'");
+    Message::Error("Wrong arguments in 'LinAlg_ProdVectorScalar'");
 }
 
 void LinAlg_ProdVectorDouble(gVector *V1, double d, gVector *V2)
@@ -770,12 +923,12 @@ void LinAlg_ProdVectorDouble(gVector *V1, double d, gVector *V2)
     _fillseq(V1);
   }
   else
-    Msg::Error("Wrong arguments in 'LinAlg_ProdVectorDouble'");
+    Message::Error("Wrong arguments in 'LinAlg_ProdVectorDouble'");
 }
 
 void LinAlg_ProdVectorComplex(gVector *V1, double d1, double d2, gVector *V2)
 {
-  Msg::Error("ProdVectorComplex not yet implemented");
+  Message::Error("ProdVectorComplex not yet implemented");
 }
 
 void LinAlg_ProdVectorVector(gVector *V1, gVector *V2, double *d)
@@ -792,7 +945,7 @@ void LinAlg_ProdVectorVector(gVector *V1, gVector *V2, double *d)
 void LinAlg_ProdMatrixVector(gMatrix *M, gVector *V1, gVector *V2)
 {
   if(V2 == V1)
-    Msg::Error("Wrong arguments in 'LinAlg_ProdMatrixVector'");
+    Message::Error("Wrong arguments in 'LinAlg_ProdMatrixVector'");
   else{
     _try(MatMult(M->M, V1->V, V2->V));
     _fillseq(V2);
@@ -804,7 +957,7 @@ void LinAlg_ProdMatrixScalar(gMatrix *M1, gScalar *S, gMatrix *M2)
   if(M2 == M1)
     _try(MatScale(M1->M, S->s));
   else
-    Msg::Error("Wrong arguments in 'LinAlg_ProdMatrixScalar'");
+    Message::Error("Wrong arguments in 'LinAlg_ProdMatrixScalar'");
 }
 
 void LinAlg_ProdMatrixDouble(gMatrix *M1, double d, gMatrix *M2)
@@ -813,7 +966,7 @@ void LinAlg_ProdMatrixDouble(gMatrix *M1, double d, gMatrix *M2)
   if(M2 == M1)
     _try(MatScale(M1->M, tmp));
   else
-    Msg::Error("Wrong arguments in 'LinAlg_ProdMatrixDouble'");
+    Message::Error("Wrong arguments in 'LinAlg_ProdMatrixDouble'");
 }
 
 void LinAlg_ProdMatrixComplex(gMatrix *M1, double d1, double d2, gMatrix *M2)
@@ -824,9 +977,9 @@ void LinAlg_ProdMatrixComplex(gMatrix *M1, double d1, double d2, gMatrix *M2)
     _try(MatScale(M1->M, tmp));
   }
   else
-    Msg::Error("Wrong arguments in 'LinAlg_ProdMatrixDouble'");
+    Message::Error("Wrong arguments in 'LinAlg_ProdMatrixDouble'");
 #else
-  Msg::Error("ProdMatrixComplex not yet implemented");
+  Message::Error("ProdMatrixComplex not yet implemented");
 #endif
 }
 
@@ -856,14 +1009,14 @@ void LinAlg_VectorNormInf(gVector *V1, double *norm)
 
 void LinAlg_AssembleMatrix(gMatrix *M)
 {
-  Msg::Barrier();
+  Message::Barrier();
   _try(MatAssemblyBegin(M->M, MAT_FINAL_ASSEMBLY));
   _try(MatAssemblyEnd(M->M, MAT_FINAL_ASSEMBLY));
 }
 
 void LinAlg_AssembleVector(gVector *V)
 {
-  Msg::Barrier();
+  Message::Barrier();
   _try(VecAssemblyBegin(V->V));
   _try(VecAssemblyEnd(V->V));
   _fillseq(V);
@@ -872,7 +1025,7 @@ void LinAlg_AssembleVector(gVector *V)
 #if defined(HAVE_ZITSOL)
 
 extern "C" {
-  int getdp_zitsol(int n, int nnz, int *row, int *col, double *valr, double *vali, 
+  int getdp_zitsol(int n, int nnz, int *row, int *col, double *valr, double *vali,
 		   double *rhsr, double *rhsi, double *solr, double *soli,
 		   int precond, int lfil, double tol0, double tol, int im, int maxits);
 }
@@ -913,27 +1066,27 @@ static void _zitsol(gMatrix *A, gVector *B, gVector *X)
     _try(MatGetRow(A->M, i, &ncols, &cols, &vals));
     for(int j = 0; j < ncols; j++){
       if(k >= nnz){
-	Msg::Error("Something wrong in nnz: %d >= %d", k, nnz);
+	Message::Error("Something wrong in nnz: %d >= %d", k, nnz);
 	return;
       }
       row[k] = i;
       col[k] = cols[j];
-      Msg::Debug("A[%d][%d] = ", row[k], col[k]);
+      Message::Debug("A[%d][%d] = ", row[k], col[k]);
 #if defined(PETSC_USE_COMPLEX)
       valr[k] = real(vals[j]);
       vali[k] = imag(vals[j]);
-      Msg::Debug("%g+i*%g", valr[k], vali[k]);
+      Message::Debug("%g+i*%g", valr[k], vali[k]);
 #else
       valr[k] = vals[j];
       vali[k] = 0.;
-      Msg::Debug("%g", valr[k]);
+      Message::Debug("%g", valr[k]);
 #endif
       k++;
     }
     _try(MatRestoreRow(A->M, i, &ncols, &cols, &vals));
   }
 
-  Msg::Info("n = %d, nnz = %d (check k = %d)", n, nnz, k);
+  Message::Info("n = %d, nnz = %d (check k = %d)", n, nnz, k);
 
   PetscScalar *b, *x;
   _try(VecGetArray(B->V, &b));
@@ -957,10 +1110,10 @@ static void _zitsol(gMatrix *A, gVector *B, gVector *X)
   int its = getdp_zitsol(n, nnz, row, col, valr, vali, rhsr, rhsi, solr, soli,
                          precond, lfil, tol0, tol, im, maxits);
   if(its >= maxits)
-    Msg::Error("Did not converge in %d iterations", maxits);
+    Message::Error("Did not converge in %d iterations", maxits);
   else
-    Msg::Info("Converged in %d iterations", its);
-  
+    Message::Info("Converged in %d iterations", its);
+
   for(PetscInt i = 0; i < n; i++){
     PetscScalar d;
 #if defined(PETSC_USE_COMPLEX)
@@ -970,7 +1123,7 @@ static void _zitsol(gMatrix *A, gVector *B, gVector *X)
 #endif
     _try(VecSetValues(X->V, 1, &i, &d, INSERT_VALUES));
   }
-  
+
   Free(row); Free(col);
   Free(valr); Free(vali);
   Free(rhsr); Free(rhsi);
@@ -981,11 +1134,11 @@ static void _zitsol(gMatrix *A, gVector *B, gVector *X)
 
 static PetscErrorCode _myKspMonitor(KSP ksp, PetscInt it, PetscReal rnorm, void *mctx)
 {
-  Msg::Info("%3ld KSP Residual norm %14.12e", (long)it, rnorm);
+  Message::Info("%3ld KSP Residual norm %14.12e", (long)it, rnorm);
   return 0;
 }
 
-static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X, 
+static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
                    int precond, int kspIndex)
 {
 #if defined(HAVE_ZITSOL)
@@ -996,35 +1149,34 @@ static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
 #endif
 
   if(kspIndex < 0 || kspIndex > 9){
-    Msg::Error("Linear Solver index out of range (%d)", kspIndex);
+    Message::Error("Linear Solver index out of range (%d)", kspIndex);
     return;
   }
 
   PetscInt i, j;
   _try(MatGetSize(A->M, &i, &j));
   if(!i){
-    Msg::Warning("Zero-size system: skipping solve!");
+    Message::Warning("Zero-size system: skipping solve!");
     return;
   }
 
-  bool view = (!Solver->ksp[kspIndex] && Msg::GetVerbosity() > 2);
+  bool view = (!Solver->ksp[kspIndex] && Message::GetVerbosity() > 2);
 
-  if(view && !Msg::GetCommRank())
-    Msg::Info("N: %ld", (long)i);
+  if(view && (!Message::GetCommRank() || !Message::GetIsCommWorld()))
+    Message::Info("N: %ld", (long)i);
 
   if(kspIndex != 0)
-    Msg::Info("Using solver index %d", kspIndex);
+    Message::Info("Using solver index %d", kspIndex);
 
   if(!Solver->ksp[kspIndex]) {
-    _try(KSPCreate(PETSC_COMM_WORLD, &Solver->ksp[kspIndex]));
+    _try(KSPCreate(MyComm, &Solver->ksp[kspIndex]));
     _try(KSPSetOperators(Solver->ksp[kspIndex], A->M, A->M, DIFFERENT_NONZERO_PATTERN));
-    if(Msg::UseSocket())
-      _try(KSPMonitorSet(Solver->ksp[kspIndex], _myKspMonitor, PETSC_NULL, PETSC_NULL));
+    _try(KSPMonitorSet(Solver->ksp[kspIndex], _myKspMonitor, PETSC_NULL, PETSC_NULL));
     PC pc;
     _try(KSPGetPC(Solver->ksp[kspIndex], &pc));
 
     // set some default options
-    _try(KSPSetTolerances(Solver->ksp[kspIndex], 1.e-12, PETSC_DEFAULT, PETSC_DEFAULT, 
+    _try(KSPSetTolerances(Solver->ksp[kspIndex], 1.e-12, PETSC_DEFAULT, PETSC_DEFAULT,
                           PETSC_DEFAULT));
 #if (PETSC_VERSION_MAJOR > 2) && defined(PETSC_HAVE_MUMPS) // use MUMPS by default if available
     _try(PCSetType(pc, PCLU));
@@ -1041,10 +1193,10 @@ static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
     _try(PCILUSetLevels(pc, 6));
 #elif (PETSC_VERSION_MAJOR == 2) && (PETSC_VERSION_MINOR == 3) && (PETSC_VERSION_SUBMINOR < 3)
     _try(PCFactorSetMatOrdering(pc, MATORDERING_RCM));
-    _try(PCFactorSetLevels(pc, 6));
+    _try(PCFactorSetLevels(pc, 1));
 #else
     _try(PCFactorSetMatOrderingType(pc, MATORDERING_RCM));
-    _try(PCFactorSetLevels(pc, 6));
+    _try(PCFactorSetLevels(pc, 1));
 #endif
 #endif
 
@@ -1062,12 +1214,12 @@ static void _solve(gMatrix *A, gVector *B, gSolver *Solver, gVector *X,
   _fillseq(X);
 
   if(view)
-    _try(KSPView(Solver->ksp[kspIndex], PETSC_VIEWER_STDOUT_WORLD));
-  
-  if(!Msg::GetCommRank()){
+    _try(KSPView(Solver->ksp[kspIndex], MyPetscViewer));
+
+  if(!Message::GetCommRank() || !Message::GetIsCommWorld()){
     PetscInt its;
     _try(KSPGetIterationNumber(Solver->ksp[kspIndex], &its));
-    Msg::Info("%d iterations", its);
+    Message::Info("%d iterations", its);
   }
 }
 
@@ -1081,4 +1233,158 @@ void LinAlg_SolveAgain(gMatrix *A, gVector *B, gSolver *Solver, gVector *X, int
   _solve(A, B, Solver, X, 0, solverIndex);
 }
 
+extern void Generate_Residual (gVector *x, gVector *f) ;
+extern void Generate_FullJacobian (gVector *x, gMatrix *Jac) ;
+
+static PetscErrorCode _NLFormFunction(SNES snes, Vec x, Vec f, void *mctx)
+{
+  /*
+    snes - the SNES context
+    x 	 - input vector (solution at each NL iteration)
+    f    - vector to store function value (residual)
+    mctx - [optional] user-defined Jacobian context
+  */
+
+  gVector gx, gf ;
+  gx.V = x ;
+  gf.V = f ;
+
+  Generate_Residual(&gx, &gf) ;
+
+  PetscScalar *ff ;
+  _try(VecGetArray(gf.V, &ff)) ;
+
+  PetscInt n;
+  _try(VecGetSize(f, &n)) ;
+  for(PetscInt i = 0; i < n; i++)
+    _try(VecSetValues(f, 1, &i, &ff[i], INSERT_VALUES));
+
+  _try(VecGetArray(f, &ff));
+
+  return 0;
+}
+
+static PetscErrorCode _NLFormJacobian(SNES snes, Vec x, Mat *J, Mat *PC,
+                                      MatStructure *flag, void *mctx)
+{
+  /*
+    snes - the SNES context
+    x 	 - input vector
+    J    - Jacobian matrix
+    PC 	 - preconditioner matrix, usually the same as Jac
+    flag - flag indicating information about the preconditioner matrix structure
+           (same as flag in KSPSetOperators()), one of
+           SAME_NONZERO_PATTERN,DIFFERENT_NONZERO_PATTERN,SAME_PRECONDITIONER
+    mctx - [optional] user-defined Jacobian context
+  */
+
+  gVector gx ;
+  gx.V = x ;
+
+  gMatrix gJ ;
+  gJ.M = *J ;
+
+  Generate_FullJacobian(&gx, &gJ);
+
+  *J = gJ.M ;
+  *flag = DIFFERENT_NONZERO_PATTERN ;
+
+  Message::Barrier();
+  _try(MatAssemblyBegin(*J, MAT_FINAL_ASSEMBLY));
+  _try(MatAssemblyEnd(*J, MAT_FINAL_ASSEMBLY));
+
+  if (*PC != *J){
+    _try(MatAssemblyBegin(*PC, MAT_FINAL_ASSEMBLY));
+    _try(MatAssemblyEnd(*PC, MAT_FINAL_ASSEMBLY));
+  }
+
+  return 0;
+}
+
+static PetscErrorCode _NLFormInitialGuess(Vec x, PetscScalar val)
+{
+  // FormInitialGuess - Computes initial guess
+
+  VecSet(x,val);
+
+  return 0;
+}
+
+static PetscErrorCode _mySnesMonitor(SNES snes, PetscInt it, PetscReal rnorm, void *mctx)
+{
+  Message::Info("%3ld SNES Residual norm %14.12e", (long)it, rnorm);
+  return 0;
+}
+
+// SNES - PETSC nonlinear solvers
+static void _solveNL(gMatrix *A, gVector *B, gMatrix *J, gVector *R, gSolver *Solver,
+                     gVector *X, int precond, int solverIndex)
+{
+  if(solverIndex < 0 || solverIndex > 9){
+    Message::Error("NonLinear Solver index out of range (%d)", solverIndex);
+    return;
+  }
+
+  PetscInt n, m;
+  _try(MatGetSize(J->M, &n, &m));
+  if(!n){
+    Message::Warning("Zero-size jacobian: skipping solve!");
+    return;
+  }
+
+  bool view = (!Solver->snes[solverIndex] && Message::GetVerbosity() > 2);
+
+  if(view && (!Message::GetCommRank() || !Message::GetIsCommWorld()))
+    Message::Info("N: %ld", (long)n);
+
+  if(solverIndex != 0)
+    Message::Info("Using nonlinear solver index %d", solverIndex);
+
+  PetscTruth fd_jacobian = PETSC_FALSE, snes_fd = PETSC_FALSE ;
+
+  // Setting nonlinear solver defaults
+  if(!Solver->snes[solverIndex]) {
+    _try(SNESCreate(MyComm, &Solver->snes[solverIndex]));
+    if(Message::UseSocket())
+      _try(SNESMonitorSet(Solver->snes[solverIndex], _mySnesMonitor, PETSC_NULL, PETSC_NULL));
+    _try(SNESSetTolerances(Solver->snes[solverIndex], 1.e-12, PETSC_DEFAULT, PETSC_DEFAULT, PETSC_DEFAULT,
+                           PETSC_DEFAULT));
+
+    // override default options with those from database (if any)
+    _try(SNESSetFromOptions(Solver->snes[solverIndex]));
+
+    PetscOptionsGetTruth(PETSC_NULL,"-fd_jacobian",&fd_jacobian,0);
+    PetscOptionsGetTruth(PETSC_NULL,"-snes_fd",&snes_fd,0);
+    if (fd_jacobian || snes_fd) {
+      Message::Error("Finite Difference Jacobian not yet implemented");
+      _try(SNESSetJacobian(Solver->snes[solverIndex], J->M, J->M, SNESDefaultComputeJacobian,PETSC_NULL));
+    }
+    else {
+      Message::Info("Jacobian computed by GetDP");
+      _try(SNESSetJacobian(Solver->snes[solverIndex], J->M, J->M, _NLFormJacobian, PETSC_NULL));
+    }
+    _try(SNESSetFunction(Solver->snes[solverIndex], R->V, _NLFormFunction, PETSC_NULL)); // R(x) = A(x)*x-b
+  }
+
+  _try(SNESSolve(Solver->snes[solverIndex], PETSC_NULL, X->V));
+
+  // copy result on all procs
+  _fillseq(X);
+
+  if(view)
+    _try(SNESView(Solver->snes[solverIndex], MyPetscViewer));
+
+  if(!Message::GetCommRank() || !Message::GetIsCommWorld()){
+    PetscInt its;
+    _try(SNESGetIterationNumber(Solver->snes[solverIndex], &its));
+    Message::Info("Number of Newton iterations %d", its);
+  }
+}
+
+void LinAlg_SolveNL(gMatrix *A, gVector *B, gMatrix *J, gVector *R,
+                    gSolver *Solver, gVector *X, int solverIndex)
+{
+  _solveNL(A, B, J, R, Solver, X, 1, solverIndex);
+}
+
 #endif
