@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -10,12 +10,17 @@
 #include "GetDPVersion.h"
 #include "ProData.h"
 #include "SolvingAnalyse.h"
-#include "Numeric.h"
 #include "LinAlg.h"
 #include "OS.h"
 #include "MallocUtils.h"
 #include "Message.h"
 
+#if defined(HAVE_GMSH)
+#include <gmsh/Gmsh.h>
+#include <gmsh/GmshVersion.h>
+#include <gmsh/GmshConfig.h>
+#endif
+
 int     Flag_PRE = 0, Flag_CAL = 0, Flag_POS = 0, Flag_RESTART = 0;
 int     Flag_XDATA = 0, Flag_BIN = 0, Flag_SPLIT = 0, Flag_GMSH_VERSION = 1;
 double  Flag_ORDER = -1.;
@@ -28,9 +33,9 @@ void Info(int level, char *arg0)
 {
   switch(level){
   case 0 :
-    fprintf(stderr, 
+    fprintf(stderr,
 	    "GetDP, a General environment for the treatment of Discrete Problems\n"
-	    "Copyright (C) 1997-2010 P. Dular, C. Geuzaine\n"
+	    "Copyright (C) 1997-2012 P. Dular, C. Geuzaine\n"
 	    "Usage: %s [file] [options]\n"
 	    "Processing options:\n"
 	    "  -pre 'Resolution'         pre-processing\n"
@@ -56,11 +61,11 @@ void Info(int level, char *arg0)
 	    "Output options:\n"
 	    "  -bin                      create binary output files\n"
 	    "  -v2                       create mesh-based Gmsh output files when possible\n"
-	    "  -socket string            communicate through socket\n"
 	    "Other options:\n"
 	    "  -check                    interactive check of problem structure\n"
 	    "  -v num                    set verbosity level (default: 4)\n"
 	    "  -p num                    set progress indicator update (default: 10)\n"
+	    "  -onelab name address      communicate with OneLab server through socket\n"
 	    "  -version                  show version number\n"
 	    "  -info                     show detailed version information\n"
 	    "  -help                     show this message\n", arg0);
@@ -70,22 +75,22 @@ void Info(int level, char *arg0)
     break;
   case 2:
     fprintf(stderr, "Version      : %s\n", GETDP_VERSION);
+#if defined(HAVE_PETSC) && defined(PETSC_USE_COMPLEX)
+    fprintf(stderr, "Arithmetic   : Complex\n");
+#else
+    fprintf(stderr, "Arithmetic   : Real\n");
+#endif
     fprintf(stderr, "License      : %s\n", GETDP_SHORT_LICENSE);
     fprintf(stderr, "Build OS     : %s\n", GETDP_OS);
+    fprintf(stderr, "Build options:%s\n", GETDP_CONFIG_OPTIONS);
     fprintf(stderr, "Build date   : %s\n", GETDP_DATE);
     fprintf(stderr, "Build host   : %s\n", GETDP_HOST);
-    fprintf(stderr, "Packager     : %s\n", GETDP_PACKAGER);
-#if defined(HAVE_SPARSKIT) && defined(HAVE_ILU_FLOAT)
-    fprintf(stderr, "Solver       : Sparskit (real arithmetic, single precision ILU)\n");
-#elif defined(HAVE_SPARSKIT)
-    fprintf(stderr, "Solver       : Sparskit (real arithmetic)\n");
-#elif defined(HAVE_PETSC) && defined(PETSC_USE_COMPLEX)
-    fprintf(stderr, "Solver       : PETSc (complex arithmetic)\n");
-#elif defined(HAVE_PETSC)
-    fprintf(stderr, "Solver       : PETSc (real arithmetic)\n");
-#else
-    fprintf(stderr, "Solver       : none\n");
+#if defined(HAVE_GMSH)
+    fprintf(stderr, "Gmsh version : %s%s (%s)\n",
+            GMSH_VERSION, GMSH_EXTRA_VERSION, GMSH_DATE);
+    fprintf(stderr, "Gmsh options :%s\n", GMSH_CONFIG_OPTIONS);
 #endif
+    fprintf(stderr, "Packager     : %s\n", GETDP_PACKAGER);
     fprintf(stderr, "Web site     : http://www.geuz.org/getdp/\n");
     fprintf(stderr, "Mailing list : getdp@geuz.org\n");
     break;
@@ -97,63 +102,78 @@ void Info(int level, char *arg0)
 /*  G e t _ O p t i o n s                                                   */
 /* ------------------------------------------------------------------------ */
 
-void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *pro, 
+void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *pro,
 		 int *lres, int *lpos, int *check)
 {
   int  i, j, Flag_NameProblem = 0;
 
-  strcpy(pro, "");  
+  strcpy(pro, "");
 
   i = *sargc = 1;
 
   while (i < argc) {
-    
+
     if (argv[i][0] == '-') {
-      
-      if      (!strcmp(argv[i]+1, "cal"))    { Flag_CAL     = 1; i++; } 
-      else if (!strcmp(argv[i]+1, "check"))  { *check       = 1; i++; } 
-      else if (!strcmp(argv[i]+1, "xdata"))  { Flag_XDATA   = 1; i++; } 
-      else if (!strcmp(argv[i]+1, "bin"))    { Flag_BIN     = 1; i++; } 
-      else if (!strcmp(argv[i]+1, "v2"))     { Flag_GMSH_VERSION = 2; i++; } 
-      else if (!strcmp(argv[i]+1, "ascii"))  { Flag_BIN     = 0; i++; } 
-      else if (!strcmp(argv[i]+1, "split"))  { Flag_SPLIT   = 1; i++; } 
+
+      if      (!strcmp(argv[i]+1, "cal"))    { Flag_CAL     = 1; i++; }
+      else if (!strcmp(argv[i]+1, "check"))  { *check       = 1; i++; }
+      else if (!strcmp(argv[i]+1, "xdata"))  { Flag_XDATA   = 1; i++; }
+      else if (!strcmp(argv[i]+1, "bin"))    { Flag_BIN     = 1; i++; }
+      else if (!strcmp(argv[i]+1, "v2"))     { Flag_GMSH_VERSION = 2; i++; }
+      else if (!strcmp(argv[i]+1, "ascii"))  { Flag_BIN     = 0; i++; }
+      else if (!strcmp(argv[i]+1, "split"))  { Flag_SPLIT   = 1; i++; }
 
       else if (!strcmp(argv[i]+1, "socket")) {
 	i++;
-	if (i<argc && argv[i][0]!='-') { 
-	  Msg::InitializeSocket(argv[i]); i++; 
+	if (i<argc && argv[i][0]!='-') {
+	  Message::InitializeSocket(argv[i]); i++;
 	}
 	else {
-	  Msg::Error("Missing socket name");
+	  Message::Error("Missing socket name");
 	}
       }
 
-      else if (!strcmp(argv[i]+1, "restart")){ 
+      else if (!strcmp(argv[i]+1, "onelab")) {
+        i++;
+	if (i + 1 < argc && argv[i][0] != '-' && argv[i + 1][0] != '-') {
+	  Message::InitializeOnelab(argv[i], argv[i + 1]);
+          i += 2;
+	}
+	else if (i < argc && argv[i][0] != '-') {
+	  Message::InitializeOnelab(argv[i], "");
+          i += 1;
+	}
+	else {
+	  Message::Error("Missing client name and/or address of OneLab server");
+	}
+      }
+
+      else if (!strcmp(argv[i]+1, "restart")){
 	Flag_CAL = Flag_RESTART = 1; i++;
-      } 
+      }
 
       else if (!strcmp(argv[i]+1, "verbose") ||
 	       !strcmp(argv[i]+1, "v")) {
 	i++;
-	if (i<argc && argv[i][0]!='-') { 
-	  Msg::SetVerbosity(atoi(argv[i])); i++; 
+	if (i<argc && argv[i][0]!='-') {
+	  Message::SetVerbosity(atoi(argv[i])); i++;
 	}
 	else {
-	  Msg::Error("Missing number");
+	  Message::Error("Missing number");
 	}
-      } 
+      }
 
       else if (!strcmp(argv[i]+1, "help")  || !strcmp(argv[i]+1, "h") ||
 	       !strcmp(argv[i]+1, "-help") || !strcmp(argv[i]+1, "-h")) {
 	Info(0, argv[0]);
       }
 
-      else if (!strcmp(argv[i]+1, "version") || 
+      else if (!strcmp(argv[i]+1, "version") ||
 	       !strcmp(argv[i]+1, "-version")) {
 	Info(1, argv[0]);
       }
 
-      else if (!strcmp(argv[i]+1, "info") || 
+      else if (!strcmp(argv[i]+1, "info") ||
                !strcmp(argv[i]+1, "-info")) {
         Info(2, argv[0]);
       }
@@ -161,21 +181,21 @@ void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *pro,
       else if (!strcmp(argv[i]+1, "progress") ||
 	       !strcmp(argv[i]+1, "p")) {
 	i++;
-	if (i<argc && argv[i][0]!='-') { 
-	  Msg::SetProgressMeterStep(atoi(argv[i])); i++;
+	if (i<argc && argv[i][0]!='-') {
+	  Message::SetProgressMeterStep(atoi(argv[i])); i++;
 	}
 	else {
-	  Msg::Error("Missing number");
+	  Message::Error("Missing number");
 	}
-      } 
+      }
 
       else if (!strcmp(argv[i]+1, "pre")) {
 	i++;
 	if (i<argc && argv[i][0]=='#') {
 	  Flag_PRE = 1; *lres = -atoi(argv[i]+1); i++;
 	}
-	else if (i<argc && argv[i][0]!='-') { 
-	  Flag_PRE = 1; Name_Resolution = argv[i]; i++; 
+	else if (i<argc && argv[i][0]!='-') {
+	  Flag_PRE = 1; Name_Resolution = argv[i]; i++;
 	}
 	else {
 	  Flag_PRE = *lres = 1;
@@ -185,11 +205,11 @@ void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *pro,
       else if (!strcmp(argv[i]+1, "order") ||
 	       !strcmp(argv[i]+1, "ord")) {
 	i++;
-	if (i<argc && argv[i][0]!='-') { 
+	if (i<argc && argv[i][0]!='-') {
 	  Flag_ORDER = atof(argv[i]); i++;
 	}
 	else {
-	  Msg::Error("Missing interpolation order");
+	  Message::Error("Missing interpolation order");
 	}
       }
 
@@ -197,11 +217,11 @@ void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *pro,
         // fix when calling getdp from gmsh (since the GUI forces us
         // to put the -solver option before the .pro file!)
 	sargv[(*sargc)++] = argv[i++];
-	if (i<argc && argv[i][0]!='-') { 
+	if (i<argc && argv[i][0]!='-') {
           sargv[(*sargc)++] = argv[i++];
 	}
 	else {
-	  Msg::Error("Missing solver option file name");
+	  Message::Error("Missing solver option file name");
 	}
       }
 
@@ -212,9 +232,9 @@ void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *pro,
 	  Flag_PRE = Flag_CAL = 1; *lres = -atoi(argv[i]+1); i++;
 	}
 	else if (i<argc && argv[i][0]!='-') {
-	  Flag_PRE = Flag_CAL = 1; Name_Resolution = argv[i]; i++; 
+	  Flag_PRE = Flag_CAL = 1; Name_Resolution = argv[i]; i++;
 	}
-	else { 
+	else {
 	  Flag_PRE = Flag_CAL = *lres = 1;
 	}
       }
@@ -226,10 +246,10 @@ void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *pro,
 	  Flag_POS = 1; *lpos = -atoi(argv[i]+1); i++;
 	} /* Only one numbered (#) PostOperation allowed */
 	else {
-	  while (i<argc && argv[i][0]!='-') { 
+	  while (i<argc && argv[i][0]!='-') {
 	    Name_PostOperation[j] = argv[i]; i++; j++;
 	    if(j == NBR_MAX_POS)
-	      Msg::Error("Too many PostOperations");
+	      Message::Error("Too many PostOperations");
 	  }
 	  if(!j){
 	    Flag_POS = *lpos = 1;
@@ -245,11 +265,11 @@ void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *pro,
 	       !strcmp(argv[i]+1, "msh") ||
 	       !strcmp(argv[i]+1, "m")) {
 	i++;
-	if (i<argc && argv[i][0]!='-') { 
-	  Name_MshFile = argv[i]; i++; 
+	if (i<argc && argv[i][0]!='-') {
+	  Name_MshFile = argv[i]; i++;
 	}
 	else {
-	  Msg::Error("Missing file name");
+	  Message::Error("Missing file name");
 	}
       }
 
@@ -257,23 +277,23 @@ void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *pro,
 	       !strcmp(argv[i]+1, "adap") ||
 	       !strcmp(argv[i]+1, "ada")) {
 	i++;
-	if (i<argc && argv[i][0]!='-') { 
-	  Name_AdaptFile = argv[i]; i++; 
+	if (i<argc && argv[i][0]!='-') {
+	  Name_AdaptFile = argv[i]; i++;
 	}
 	else {
-	  Msg::Error("Missing file name");
+	  Message::Error("Missing file name");
 	}
       }
 
       else if (!strcmp(argv[i]+1, "res")) {
 	i++; j = 0;
-	while (i<argc && argv[i][0]!='-') { 
+	while (i<argc && argv[i][0]!='-') {
 	  Name_ResFile[j] = argv[i]; i++; j++;
 	  if(j == NBR_MAX_RES)
-	    Msg::Error("Too many '.res' files");
+	    Message::Error("Too many '.res' files");
 	}
 	if(!j)
-	  Msg::Error("Missing file name");
+	  Message::Error("Missing file name");
 	else{
 	  Name_ResFile[j] = NULL;
 	}
@@ -281,28 +301,28 @@ void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *pro,
 
       else if (!strcmp(argv[i]+1, "name")) {
 	i++;
-	if (i<argc && argv[i][0]!='-') { 
+	if (i<argc && argv[i][0]!='-') {
 	  Name_Generic = (char*)Malloc((strlen(argv[i]) + 1) * sizeof(char));
-	  strcpy(Name_Generic, argv[i]); i++; 
+	  strcpy(Name_Generic, argv[i]); i++;
 	}
 	else {
-	  Msg::Error("Missing string");
+	  Message::Error("Missing string");
 	}
       }
 
-      else if (!strcmp(argv[i]+1, "petscinfo") || 
+      else if (!strcmp(argv[i]+1, "petscinfo") ||
 	       !strcmp(argv[i]+1, "-petscinfo")) {
 	sargv[(*sargc)++] = (char*)"-info";
 	i++;
       }
 
       else {
-	sargv[(*sargc)++] = argv[i++]; 
+	sargv[(*sargc)++] = argv[i++];
       }
-      
+
     }
     else{
-      if (!Flag_NameProblem) { 
+      if (!Flag_NameProblem) {
 	Flag_NameProblem = 1;
 	sargv[0] = argv[i];
 	strcpy(pro, argv[i++]);
@@ -311,11 +331,11 @@ void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *pro,
 	sargv[(*sargc)++] = argv[i++];
       }
     }
-    
+
   }
-  
+
   if(!strlen(pro))
-    Msg::Error("Missing input file name");
+    Message::Error("Missing input file name");
   else{
     if(!Name_Generic){
       Name_Generic = (char*)Malloc((strlen(pro) + 1) * sizeof(char));
@@ -338,6 +358,21 @@ void Get_Options(int argc, char *argv[], int *sargc, char **sargv, char *pro,
   }
 }
 
+#if defined(HAVE_GMSH)
+class GmshMsg : public GmshMessage{
+public:
+  void operator()(std::string level, std::string msg)
+  {
+    if(level == "Fatal" || level == "Error")
+      Message::Error("%s", msg.c_str());
+    else if(level == "Warning")
+      Message::Warning("%s", msg.c_str());
+    else
+      Message::Info("%s", msg.c_str());
+  }
+};
+#endif
+
 int MainLegacy(int argc, char *argv[])
 {
   if(argc < 2) Info(0, argv[0]);
@@ -347,29 +382,29 @@ int MainLegacy(int argc, char *argv[])
   std::string currtime(ctime(&now));
   currtime.resize(currtime.size() - 1);
 
-  std::string cmdline;
+  std::string cmdline("");
   for(int i = 0; i < argc; i++){
     if(i) cmdline += " ";
     cmdline += argv[i];
   }
 
-  Msg::Init(argc, argv);
+  Message::Init(argc, argv);
 
   char pro[256];
   char **sargv = (char**)Malloc(256 * sizeof(char*));
   int sargc, lres = 0, lpos = 0, check = 0;
   Get_Options(argc, argv, &sargc, sargv, pro, &lres, &lpos, &check);
 
-  Msg::Info("Running '%s' [%d node(s)]", cmdline.c_str(), Msg::GetCommSize());
-  Msg::Info("Started on %s", currtime.c_str());
+  Message::Info("Running '%s' [%d node(s)]", cmdline.c_str(), Message::GetCommSize());
+  Message::Info("Started on %s", currtime.c_str());
 
   if(sargc > 1){
-    std::string solveropt;
+    std::string solveropt("");
     for(int i = 1; i < sargc; i++){
       if(i > 1) solveropt += " ";
       solveropt += sargv[i];
     }
-    Msg::Debug("Passing unused options to solver: '%s'", solveropt.c_str());
+    Message::Debug("Passing unused options to solver: '%s'", solveropt.c_str());
   }
 
   if(!Name_ResFile[0]){
@@ -379,19 +414,30 @@ int MainLegacy(int argc, char *argv[])
     Name_ResFile[1] = 0;
   }
 
+  if(!Name_MshFile){
+    Message::GetOnelabString("Gmsh/MshFileName", &Name_MshFile);
+    if(Name_MshFile)
+      Message::Info("Got mesh name from Onelab: '%s'", Name_MshFile);
+  }
   if(!Name_MshFile){
     Name_MshFile = (char*)Malloc((strlen(Name_Generic)+5)*sizeof(char));
     strcpy(Name_MshFile, Name_Generic);
     strcat(Name_MshFile, ".msh");
   }
 
-  check_gsl();
-  CheckResources();
+#if defined(HAVE_GMSH)
+  Message::Info("Initializing Gmsh");
+  GmshInitialize();
+  GmshMsg c;
+  GmshSetMessageHandler(&c);
+#endif
 
+  IncreaseStackSize();
   LinAlg_InitializeSolver(&sargc, &sargv);
 
   Init_ProblemStructure();
   Read_ProblemStructure(pro);
+  Finalize_ProblemStructure();
 
   int choose = 1;
   if (!Flag_PRE && !Flag_CAL && !Flag_POS && !check){
@@ -399,16 +445,14 @@ int MainLegacy(int argc, char *argv[])
     choose = 0;
   }
 
-  // Msg::TestSocket();
-
-  if(check) 
+  if(check)
     Print_ProblemStructure();
 
   if(lres)
     Print_ListResolution(choose, lres, &Name_Resolution);
 
-  if(lpos) 
-    Print_ListPostOperation(choose, lpos, &Name_PostOperation[0]);
+  if(lpos)
+    Print_ListPostOperation(choose, lpos, Name_PostOperation);
 
   if(Flag_PRE || Flag_CAL || Flag_POS)
     SolvingAnalyse();
@@ -418,10 +462,14 @@ int MainLegacy(int argc, char *argv[])
   time(&now);
   currtime = ctime(&now);
   currtime[currtime.size() - 1] = '\0';
-  Msg::Info("Stopped on %s", currtime.c_str());
+  Message::Info("Stopped on %s", currtime.c_str());
 
-  Msg::Direct("E n d");
-  Msg::FinalizeSocket();
-  Msg::Exit(0);
+  Message::Direct("E n d");
+
+#if defined(HAVE_GMSH)
+  GmshFinalize();
+#endif
+
+  Message::Exit(0);
   return 0;
 }
