@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -9,16 +9,16 @@
 #include "ProData.h"
 #include "ListUtils.h"
 
-void  Format_PostFormat(int Format) ;
+void  Format_PostFormat(int Format, int NoMesh) ;
 
-void  Format_PostHeader(int Format, int Contour, 
-			int NbTimeStep, int HarmonicToTime,
+void  Format_PostHeader(int Format, int SubType, double Time, int TimeStep,
+			int Contour, int NbTimeStep, int HarmonicToTime,
 			int Type, int Order, char *Name1, char *Name2) ;
 
 void  Format_PostFooter(struct PostSubOperation * PSO_P, int Store) ;
 
 void  Format_PostElement(struct PostSubOperation * PSO_P, int Contour, int Store,
-			 double Time, int TimeStep, int NbTimeStep, 
+			 double Time, int TimeStep, int NbTimeStep,
 			 int NbrHarmonics, int HarmonicToTime,
 			 double *Dummy, struct PostElement *PE);
 
