@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -50,7 +50,71 @@ bool IsEquationNonLocal(BasisFunction *bf)
      bf->Function == (void(*)())BF_Global ||
      bf->Function == (void(*)())BF_dGlobal ||
      bf->Function == (void(*)())BF_Zero ||
-     bf->Function == (void(*)())BF_One)
+     bf->Function == (void(*)())BF_One ||
+     bf->Function == (void(*)())BF_GroupOfNodes ||
+     bf->Function == (void(*)())BF_GroupOfNodes_2E ||
+     bf->Function == (void(*)())BF_GroupOfNodes_2F ||
+     bf->Function == (void(*)())BF_GroupOfNodes_2V ||
+     bf->Function == (void(*)())BF_GroupOfNodes_3E ||
+     bf->Function == (void(*)())BF_GroupOfNodes_3F ||
+     bf->Function == (void(*)())BF_GroupOfNodes_3V ||
+     bf->Function == (void(*)())BF_GradGroupOfNodes ||
+     bf->Function == (void(*)())BF_GradGroupOfNodes_2E ||
+     bf->Function == (void(*)())BF_GradGroupOfNodes_2F ||
+     bf->Function == (void(*)())BF_GradGroupOfNodes_2V ||
+     bf->Function == (void(*)())BF_GradGroupOfNodes_3E ||
+     bf->Function == (void(*)())BF_GradGroupOfNodes_3F ||
+     bf->Function == (void(*)())BF_GradGroupOfNodes_3V ||
+     bf->Function == (void(*)())BF_GroupOfEdges ||
+     bf->Function == (void(*)())BF_GroupOfEdges_2E ||
+     bf->Function == (void(*)())BF_GroupOfEdges_2F ||
+     bf->Function == (void(*)())BF_GroupOfEdges_2V ||
+     bf->Function == (void(*)())BF_GroupOfEdges_3E ||
+     bf->Function == (void(*)())BF_GroupOfEdges_3F_a ||
+     bf->Function == (void(*)())BF_GroupOfEdges_3F_b ||
+     bf->Function == (void(*)())BF_GroupOfEdges_3F_c ||
+     bf->Function == (void(*)())BF_GroupOfEdges_3V ||
+     bf->Function == (void(*)())BF_GroupOfEdges_4E ||
+     bf->Function == (void(*)())BF_GroupOfEdges_4F ||
+     bf->Function == (void(*)())BF_GroupOfEdges_4V ||
+     bf->Function == (void(*)())BF_CurlGroupOfEdges ||
+     bf->Function == (void(*)())BF_CurlGroupOfEdges_2E ||
+     bf->Function == (void(*)())BF_CurlGroupOfEdges_2F ||
+     bf->Function == (void(*)())BF_CurlGroupOfEdges_2V ||
+     bf->Function == (void(*)())BF_CurlGroupOfEdges_3E ||
+     bf->Function == (void(*)())BF_CurlGroupOfEdges_3F_a ||
+     bf->Function == (void(*)())BF_CurlGroupOfEdges_3F_b ||
+     bf->Function == (void(*)())BF_CurlGroupOfEdges_3F_c ||
+     bf->Function == (void(*)())BF_CurlGroupOfEdges_3V ||
+     bf->Function == (void(*)())BF_CurlGroupOfEdges_4E ||
+     bf->Function == (void(*)())BF_CurlGroupOfEdges_4F ||
+     bf->Function == (void(*)())BF_CurlGroupOfEdges_4V ||
+     bf->Function == (void(*)())BF_GroupOfPerpendicularEdges ||
+     bf->Function == (void(*)())BF_GroupOfPerpendicularEdges_2E ||
+     bf->Function == (void(*)())BF_GroupOfPerpendicularEdges_2F ||
+     bf->Function == (void(*)())BF_GroupOfPerpendicularEdges_2V ||
+     bf->Function == (void(*)())BF_GroupOfPerpendicularEdges_3E ||
+     bf->Function == (void(*)())BF_GroupOfPerpendicularEdges_3F ||
+     bf->Function == (void(*)())BF_GroupOfPerpendicularEdges_3V ||
+     bf->Function == (void(*)())BF_CurlGroupOfPerpendicularEdges ||
+     bf->Function == (void(*)())BF_CurlGroupOfPerpendicularEdges_2E ||
+     bf->Function == (void(*)())BF_CurlGroupOfPerpendicularEdges_2F ||
+     bf->Function == (void(*)())BF_CurlGroupOfPerpendicularEdges_2V ||
+     bf->Function == (void(*)())BF_CurlGroupOfPerpendicularEdges_3E ||
+     bf->Function == (void(*)())BF_CurlGroupOfPerpendicularEdges_3F ||
+     bf->Function == (void(*)())BF_CurlGroupOfPerpendicularEdges_3V ||
+     bf->Function == (void(*)())BF_GroupOfNodesX ||
+     bf->Function == (void(*)())BF_GroupOfNodesY ||
+     bf->Function == (void(*)())BF_GroupOfNodesZ ||
+     bf->Function == (void(*)())BF_GroupOfNodesX_D1 ||
+     bf->Function == (void(*)())BF_GroupOfNodesY_D1 ||
+     bf->Function == (void(*)())BF_GroupOfNodesZ_D1 ||
+     bf->Function == (void(*)())BF_GroupOfNodesX_D2 ||
+     bf->Function == (void(*)())BF_GroupOfNodesY_D2 ||
+     bf->Function == (void(*)())BF_GroupOfNodesZ_D2 ||
+     bf->Function == (void(*)())BF_GroupOfNodesX_D12 ||
+     bf->Function == (void(*)())BF_GroupOfNodesY_D12 ||
+     bf->Function == (void(*)())BF_GroupOfNodesZ_D12)
     return true;
   return false;
 }
@@ -494,11 +558,11 @@ void Pre_FemGlobalEquation(struct EquationTerm    * EquationTerm_P,
 	if (!List_Search(RegionIndex_L, &Num_Region, fcmp_int))
 	  List_Add(RegionIndex_L, &Num_Region) ;
 	else
-	  Msg::Error("2 occurences of Elementary Region #%d in Contraint '%s'",
-		     Num_Region, Constraint_P->Name);
+	  Message::Error("2 occurences of Elementary Region #%d in Contraint '%s'",
+                         Num_Region, Constraint_P->Name);
       }
       else
-	Msg::Error("Not 1 Elementary Region in Group '%s'", Group_P->Name);
+	Message::Error("Not 1 Elementary Region in Group '%s'", Group_P->Name);
     }
   }
   Nbr_EquAndDof = List_Nbr(RegionIndex_L) ;
@@ -539,10 +603,10 @@ void Pre_FemGlobalEquation(struct EquationTerm    * EquationTerm_P,
     }
   }
   if (Nbr_GlobalEqu != Nbr_EquAndDof){
-    Msg::Error("Incompatible number of equations with Contraint '%s'",
-	       Constraint_P->Name);
-    Msg::Error("(%d equations obtained while %d branches are defined)",
-	       Nbr_GlobalEqu, Nbr_EquAndDof);
+    Message::Error("Incompatible number of equations with Contraint '%s'",
+                   Constraint_P->Name);
+    Message::Error("(%d equations obtained while %d branches are defined)",
+                   Nbr_GlobalEqu, Nbr_EquAndDof);
   }
   
   List_Delete(RegionIndex_L) ;
