@@ -1,4 +1,4 @@
-// GetDP - Copyright (C) 1997-2010 P. Dular, C. Geuzaine
+// GetDP - Copyright (C) 1997-2012 P. Dular, C. Geuzaine
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to <getdp@geuz.org>.
@@ -76,8 +76,8 @@ void  Init_DofDataInDefineQuantity(struct DefineSystem *DefineSystem_P,
 
     if(DefineQuantity_P->DofDataIndex >= 0){
       if(DefineQuantity_P->DofDataIndex >= List_Nbr(DefineSystem_P->OriginSystemIndex))
-	Msg::Error("Invalid System index (%d) in discrete Quantity (%s)",
-		   DefineQuantity_P->DofDataIndex, DefineQuantity_P->Name);
+	Message::Error("Invalid System index (%d) in discrete Quantity (%s)",
+                       DefineQuantity_P->DofDataIndex, DefineQuantity_P->Name);
 
       List_Read(DefineSystem_P->OriginSystemIndex, DefineQuantity_P->DofDataIndex, &j) ;
       DefineQuantity_P->DofData = DofData_P0 + j ;
@@ -90,7 +90,7 @@ void  Init_DofDataInDefineQuantity(struct DefineSystem *DefineSystem_P,
 /* ------------------------------------------------------------------------ */
 /*  T r e a t m e n t _ P r e p r o c e s s i n g                           */
 /* ------------------------------------------------------------------------ */
-/*! For each DefineSystem: 
+/*! For each DefineSystem:
       For each Formulation: Definition of Dof's in associated DofData */
 
 void Treatment_Preprocessing(int Nbr_DefineSystem,
@@ -108,24 +108,24 @@ void Treatment_Preprocessing(int Nbr_DefineSystem,
     DefineSystem_P = DefineSystem_P0 + i ;
     DofData_P = DofData_P0 + i ;
     Dof_SetCurrentDofData(Current.DofData = DofData_P) ;
-    
-    Geo_SetCurrentGeoData(Current.GeoData = 
+
+    Geo_SetCurrentGeoData(Current.GeoData =
 			  GeoData_P0 + DofData_P->GeoDataIndex) ;
-    
+
     Current.NbrHar = Current.DofData->NbrHar ;
-    
+
     Nbr_Formulation = List_Nbr(DefineSystem_P->FormulationIndex) ;
 
     for (k = 0 ; k < Nbr_Formulation ; k++) {
       List_Read(DefineSystem_P->FormulationIndex, k, &Index_Formulation) ;
       Formulation_P = (struct Formulation*)
         List_Pointer(Problem_S.Formulation, Index_Formulation) ;
-      Msg::Info("Treatment Formulation '%s'", Formulation_P->Name) ;
+      Message::Info("Treatment Formulation '%s'", Formulation_P->Name) ;
 
       Init_DofDataInDefineQuantity(DefineSystem_P, DofData_P0, Formulation_P) ;
       Treatment_Formulation(Formulation_P) ;
     }
-    
+
     Dof_NumberUnknownDof() ;
   }
 }
@@ -149,23 +149,23 @@ void  Treatment_PostOperation(struct Resolution     * Resolution_P,
   int    Nbr_PostSubOperation, i_POP, i ;
 
   if (!List_Nbr(PostProcessing_P->PostQuantity))
-    Msg::Error("No Quantity available for PostProcessing '%s'",
-	       PostProcessing_P->Name) ;
+    Message::Error("No Quantity available for PostProcessing '%s'",
+                   PostProcessing_P->Name) ;
 
   Formulation_P = (struct Formulation *)
     List_Pointer(Problem_S.Formulation, PostProcessing_P->FormulationIndex) ;
 
   if (!List_Nbr(Formulation_P->DefineQuantity))
-    Msg::Error("No discrete Quantity in Formulation '%s'", Formulation_P->Name);
+    Message::Error("No discrete Quantity in Formulation '%s'", Formulation_P->Name);
 
   /* Choice of Current DofData */
   Current.DofData = 0;
   if(PostProcessing_P->NameOfSystem){
-    if ((i = List_ISearchSeq(Resolution_P->DefineSystem, 
+    if ((i = List_ISearchSeq(Resolution_P->DefineSystem,
 			     PostProcessing_P->NameOfSystem,
 			     fcmp_DefineSystem_Name)) < 0){
-      Msg::Error("Unknown System name (%s) in PostProcessing (%s)", 
-		 PostProcessing_P->NameOfSystem, PostProcessing_P->Name) ;
+      Message::Error("Unknown System name (%s) in PostProcessing (%s)",
+                     PostProcessing_P->NameOfSystem, PostProcessing_P->Name) ;
       return;
     }
     Current.DofData = DofData_P0 + i;
@@ -176,7 +176,7 @@ void  Treatment_PostOperation(struct Resolution     * Resolution_P,
   }
   else{
     for(i = 0 ; i < List_Nbr(Formulation_P->DefineQuantity) ; i++){
-      Current.DofData = 
+      Current.DofData =
 	((struct FunctionSpace *)
 	 List_Pointer(Problem_S.FunctionSpace,
 		      ((struct DefineQuantity *)
@@ -185,31 +185,31 @@ void  Treatment_PostOperation(struct Resolution     * Resolution_P,
       if(Current.DofData) break;
     }
     if(Current.DofData)
-      Msg::Info("NameOfSystem not set in PostProcessing: selected '%s'",
-		(DefineSystem_P0 + Current.DofData->Num)->Name) ;
+      Message::Info("NameOfSystem not set in PostProcessing: selected '%s'",
+                    (DefineSystem_P0 + Current.DofData->Num)->Name) ;
   }
 
   if(!Current.DofData){
-    Msg::Error("PostProcessing not compatible with Resolution");
+    Message::Error("PostProcessing not compatible with Resolution");
     return;
   }
 
   DefineSystem_P = DefineSystem_P0 + Current.DofData->Num ;
   Current.NbrHar = Current.DofData->NbrHar ;
 
-  Geo_SetCurrentGeoData(Current.GeoData = 
+  Geo_SetCurrentGeoData(Current.GeoData =
 			GeoData_P0 + Current.DofData->GeoDataIndex) ;
 
-  Msg::Info("Selected PostProcessing '%s'", PostProcessing_P->Name);
-  Msg::Info("Selected Mesh '%s'", Current.GeoData->Name);
+  Message::Info("Selected PostProcessing '%s'", PostProcessing_P->Name);
+  Message::Info("Selected Mesh '%s'", Current.GeoData->Name);
 
   Init_DofDataInDefineQuantity(DefineSystem_P,DofData_P0,Formulation_P);
-  
+
   Nbr_PostSubOperation = List_Nbr(PostOperation_P->PostSubOperation) ;
-  for (i_POP = 0 ; i_POP < Nbr_PostSubOperation ; i_POP++) {      
-    Msg::Info("PostOperation %d/%d ", i_POP+1, Nbr_PostSubOperation) ;      
+  for (i_POP = 0 ; i_POP < Nbr_PostSubOperation ; i_POP++) {
+    Message::Info("PostOperation %d/%d ", i_POP+1, Nbr_PostSubOperation) ;
     PostSubOperation_P = (struct PostSubOperation*)
-      List_Pointer(PostOperation_P->PostSubOperation, i_POP) ;    
+      List_Pointer(PostOperation_P->PostSubOperation, i_POP) ;
     Pos_Formulation(Formulation_P, PostProcessing_P, PostSubOperation_P) ;
   }
 
@@ -244,16 +244,16 @@ void  Init_HarInDofData(struct DefineSystem * DefineSystem_P,
   }
 
   if (DofData_P->NbrHar > NBR_MAX_HARMONIC)
-    Msg::Error("Too many harmonics to generate system (%d > %d)", 
-	       DofData_P->NbrHar/2, NBR_MAX_HARMONIC/2) ; 
+    Message::Error("Too many harmonics to generate system (%d > %d)",
+                   DofData_P->NbrHar/2, NBR_MAX_HARMONIC/2) ;
 
   if (DofData_P->NbrHar > 1) {
     for (j = 0 ; j < DofData_P->NbrHar/2 ; j++)
-      Msg::Info("System '%s' : Complex, Frequency = %.8g Hz",
-		DefineSystem_P->Name, DofData_P->Val_Pulsation[j]/TWO_PI) ;
+      Message::Info("System '%s' : Complex, Frequency = %.8g Hz",
+                    DefineSystem_P->Name, DofData_P->Val_Pulsation[j]/TWO_PI) ;
   }
   else{
-    Msg::Info("System '%s' : Real", DefineSystem_P->Name) ;
+    Message::Info("System '%s' : Real", DefineSystem_P->Name) ;
   }
 }
 
@@ -278,14 +278,14 @@ void  Treatment_Resolution(int ResolutionIndex,
   *Resolution_P = (struct Resolution*)List_Pointer(Problem_S.Resolution,
                                                    ResolutionIndex) ;
 
-  Msg::Info("Selected Resolution '%s'", (*Resolution_P)->Name) ;
-  
+  Message::Info("Selected Resolution '%s'", (*Resolution_P)->Name) ;
+
   *Nbr_DefineSystem = List_Nbr((*Resolution_P)->DefineSystem) ;
   if (!*Nbr_DefineSystem){
-    Msg::Error("No System exists for Resolution '%s'", (*Resolution_P)->Name) ;
+    Message::Error("No System exists for Resolution '%s'", (*Resolution_P)->Name) ;
     return;
   }
-  
+
   if (*Nbr_OtherSystem)  *Nbr_OtherSystem -= *Nbr_DefineSystem ;
 
   *DofData_L = List_Create(*Nbr_DefineSystem + *Nbr_OtherSystem, 6,
@@ -309,7 +309,7 @@ void  Treatment_Resolution(int ResolutionIndex,
     Dof_InitDofData(&DofData_S, i + *Nbr_DefineSystem, -1, -1, NULL) ;
     List_Add(*DofData_L, &DofData_S) ;
   }
-  
+
   *DofData_P0 = (struct DofData*)List_Pointer(*DofData_L, 0) ;
   *GeoData_P0 = (struct GeoData*)List_Pointer(GeoData_L, 0) ;
 }
@@ -333,14 +333,13 @@ double  * Get_TimeFunctionValues(struct DofData * DofData_P)
   for (i = 0 ; i < Nbr_TimeFunction ; i++) {
     List_Read(DofData_P->TimeFunctionIndex, i, &Index) ;
     if ((DofData_P->NbrHar == 1) && (Index > 0)) {
-      Get_ValueOfExpressionByIndex(Index - 1,
-				   NULL, 0., 0., 0., &Val_Expression) ;
+      Get_ValueOfExpressionByIndex(Index - 1, NULL, 0., 0., 0., &Val_Expression) ;
       Values[Index] = Val_Expression.Val[0] ;
     }
     else
       Values[Index] = 1. ;
   }
-  
+
   return(Values) ;
 }
 
@@ -385,7 +384,7 @@ void SolvingAnalyse()
       Num_Resolution = List_ISearchSeq(Problem_S.Resolution, Name_Resolution,
                                        fcmp_Resolution_Name) ;
     else
-      Msg::Error("Missing Resolution");
+      Message::Error("Missing Resolution");
   }
   else if (Flag_CAL || Flag_POS) {
     Dof_OpenFile(DOF_PRE, Name_Generic, "r") ;
@@ -396,7 +395,7 @@ void SolvingAnalyse()
 
   if (Num_Resolution < 0 ||
       Num_Resolution + 1 > List_Nbr(Problem_S.Resolution))
-    Msg::Error("Unknown Resolution (%s)", Name_Resolution);
+    Message::Error("Unknown Resolution (%s)", Name_Resolution);
 
   Treatment_Resolution(Num_Resolution, &Nbr_DefineSystem, &Nbr_OtherSystem,
                        &Resolution_P, &DefineSystem_P0, &DofData_P0,
@@ -408,10 +407,10 @@ void SolvingAnalyse()
 
   TreatmentStatus = _PRE ;
 
-  Msg::Direct("P r e - P r o c e s s i n g . . .") ;
+  Message::Direct("P r e - P r o c e s s i n g . . .") ;
 
   if (Flag_PRE) {
-    
+
     PreResolutionIndex_L = List_Create(10, 10, sizeof(struct PreResolutionInfo)) ;
 
     Treatment_Preprocessing(Nbr_DefineSystem, DofData_P0,
@@ -421,9 +420,9 @@ void SolvingAnalyse()
 
     for (i = 0 ; i < Nbr_PreResolution ; i++) {
 
-      Msg::Direct("P r e - R e s o l u t i o n  (%d/%d) . . .",
-		  i+1, Nbr_PreResolution) ;
-      
+      Message::Direct("P r e - R e s o l u t i o n  (%d/%d) . . .",
+                      i+1, Nbr_PreResolution) ;
+
       List_Read(PreResolutionIndex_L, i, &PreResolutionInfo_S) ;
       Num_Resolution2 = PreResolutionInfo_S.Index ;
 
@@ -441,7 +440,7 @@ void SolvingAnalyse()
 
       Init_DofDataInFunctionSpace(Nbr_DefineSystem2, DofData2_P0) ;
 
-      Current.TypeTime = TIME_STATIC  ; 
+      Current.TypeTime = TIME_STATIC  ;
       Current.Time = 0. ; Current.TimeImag = 0. ; Current.TimeStep = 0. ;
       Current.Iteration = 0 ;
       Current.RelativeDifference = 0. ; Current.RelaxationFactor = 1. ;
@@ -451,9 +450,9 @@ void SolvingAnalyse()
       Current.NbrSystem  = Nbr_DefineSystem2 ;  /* Attention: init for Dt[] */
       Current.DofData_P0 = DofData2_P0 ;
 
-      Treatment_Operation(Resolution2_P, Resolution2_P->Operation, 
+      Treatment_Operation(Resolution2_P, Resolution2_P->Operation,
                           DofData2_P0, GeoData_P0, Resolution_P, DofData_P0) ;
-      
+
       if (PreResolutionInfo_S.Type == PR_GLOBALBASISFUNCTION) {
         for (j = 0 ; j < Nbr_DefineSystem2 ; j++) {
           DofData_P = DofData2_P0 + j ;
@@ -465,10 +464,10 @@ void SolvingAnalyse()
         DofData_P0 = (struct DofData*)List_Pointer(DofData_L, 0) ;  /* New Value ... */
       }
 
-      Msg::Direct("E n d   P r e - R e s o l u t i o n  (%d/%d)",
-		  i+1, Nbr_PreResolution) ;
+      Message::Direct("E n d   P r e - R e s o l u t i o n  (%d/%d)",
+                      i+1, Nbr_PreResolution) ;
     }
-    
+
     Dof_OpenFile(DOF_PRE, Name_Generic, "w+") ;
     Dof_WriteFilePRE0(Num_Resolution, Resolution_P->Name, Nbr_DefineSystem) ;
 
@@ -485,11 +484,11 @@ void SolvingAnalyse()
       for (i = 0 ; i < Nbr_DefineSystem ; i++)
         Dof_TransferDofTreeToList(DofData_P0 + i) ;
 
-  }  
+  }
 
   else if (Flag_CAL || Flag_POS) {
-    
-    Msg::Info("Loding Pre-Processing data '%s.pre'", Name_Generic) ;
+
+    Message::Info("Loding Pre-Processing data '%s.pre'", Name_Generic) ;
 
     for(i = 0 ; i < Nbr_DefineSystem ; i++)
       Dof_ReadFilePRE(DofData_P0 + i) ;
@@ -508,15 +507,17 @@ void SolvingAnalyse()
     Nbr_DefineSystem = List_Nbr(DofData_L) ;  /* New Value ... */
 
     Nbr_GeoData = List_Nbr(GeoData_L) ;
-    
+
     Geo_ReadFilePRE(GeoData_P0, Nbr_GeoData, Problem_S.Group) ;
 
     Dof_CloseFile(DOF_PRE) ;
-    
-  }  
 
-  Msg::Cpu("");
-  Msg::Direct("E n d   P r e - P r o c e s s i n g");
+  }
+
+  Message::Cpu("");
+  Message::Direct("E n d   P r e - P r o c e s s i n g");
+
+  if(Message::GetOnelabAction() == "stop") return;
 
   /* ---------- */
   /* Processing */
@@ -524,42 +525,44 @@ void SolvingAnalyse()
 
   if (Flag_CAL) {
     TreatmentStatus = _CAL ;
-    Msg::Direct("P r o c e s s i n g . . .") ;
+    Message::Direct("P r o c e s s i n g . . .") ;
 
     Init_DofDataInFunctionSpace(Nbr_DefineSystem, DofData_P0) ;
 
     if(Flag_RESTART) {
       i = 0 ;
       while(Name_ResFile[i]){
-	Msg::Info("Loding Processing data '%s'", Name_ResFile[i]) ;
+	Message::Info("Loding Processing data '%s'", Name_ResFile[i]) ;
 	Dof_OpenFile(DOF_RES, Name_ResFile[i], "rb");
-	Dof_ReadFileRES(DofData_L, NULL, -1, &Current.Time, &Current.TimeImag, 
+	Dof_ReadFileRES(DofData_L, NULL, -1, &Current.Time, &Current.TimeImag,
 			&Current.TimeStep) ;
 	Dof_CloseFile(DOF_RES);
 	i++ ;
       }
-      Msg::Info("Restarting computation (time = %g) s (TimeStep %g)", 
+      Message::Info("Restarting computation (time = %g) s (TimeStep %g)",
 		Current.Time, Current.TimeStep) ;
     }
     else{
       Current.Time = Current.TimeImag = Current.TimeStep = 0. ;
     }
 
-    Current.NbrHar = 1 ; /* Bug : peut ne pas etre initialise si -cal sans 
+    Current.NbrHar = 1 ; /* Bug : peut ne pas etre initialise si -cal sans
 			    -pre et evaluation d'expression sans init de systeme avant */
     Current.TypeTime = TIME_STATIC ;
-    Current.RelativeDifference = 0. ; 
+    Current.RelativeDifference = 0. ;
     Current.RelaxationFactor = 1. ;
     Current.NbrSystem  = Nbr_DefineSystem ;  /* Attention: init for Dt[] */
     Current.DofData_P0 = DofData_P0 ;
 
-    Treatment_Operation(Resolution_P, Resolution_P->Operation, 
+    Treatment_Operation(Resolution_P, Resolution_P->Operation,
                         DofData_P0, GeoData_P0, NULL, NULL) ;
 
-    Msg::Cpu("");
-    Msg::Direct("E n d   P r o c e s s i n g");
+    Message::Cpu("");
+    Message::Direct("E n d   P r o c e s s i n g");
   }
 
+  if(Message::GetOnelabAction() == "stop") return;
+
   /* --------------- */
   /* Post-processing */
   /* --------------- */
@@ -567,13 +570,13 @@ void SolvingAnalyse()
   if (Flag_POS) {
     TreatmentStatus = _POS ;
 
-    Msg::Direct("P o s t - P r o c e s s i n g . . .") ;
+    Message::Direct("P o s t - P r o c e s s i n g . . .") ;
 
     i = 0 ;
     while(Name_PostOperation[i]){
       if((Num = List_ISearchSeq(Problem_S.PostOperation, Name_PostOperation[i],
 				fcmp_PostOperation_Name)) < 0)
-	Msg::Error("Unknown PostOperation (%s)", Name_PostOperation[i]) ;
+	Message::Error("Unknown PostOperation (%s)", Name_PostOperation[i]) ;
       PostOperation_P[i] = (struct PostOperation*)
 	List_Pointer(Problem_S.PostOperation, Num) ;
       PostProcessing_P[i] = (struct PostProcessing *)
@@ -585,7 +588,7 @@ void SolvingAnalyse()
     if (!Flag_CAL) {
       i = 0 ;
       while(Name_ResFile[i]){
-	Msg::Info("Loading Processing data '%s'", Name_ResFile[i]) ;
+	Message::Info("Loading Processing data '%s'", Name_ResFile[i]) ;
 	Dof_OpenFile(DOF_RES, Name_ResFile[i], "rb");
 	Dof_ReadFileRES(DofData_L, NULL, -1, &d, &d, &d) ;
 	Dof_CloseFile(DOF_RES) ;
@@ -610,7 +613,7 @@ void SolvingAnalyse()
                                   function de base globale... a reorganiser */
         if (DofData_P->Solutions == NULL)
           DofData_P->Solutions = List_Create( 1, 1, sizeof(struct Solution)) ;
-        Solution_S.Time = 0. ;  
+        Solution_S.Time = 0. ;
 	Solution_S.SolutionExist = 0 ;
         List_Add(DofData_P->Solutions, &Solution_S) ;
         Nbr_Solution = 1 ;
@@ -629,10 +632,10 @@ void SolvingAnalyse()
       /* La solution courante est la 1ere. A mieux gerer ? */
     }
     Init_DofDataInFunctionSpace(Nbr_DefineSystem, DofData_P0) ;
-    
+
     Current.NbrSystem  = Nbr_DefineSystem ;  /* Attention: init for Dt[] */
     Current.DofData_P0 = DofData_P0 ;
-    
+
     i = 0 ;
     while(PostProcessing_P[i]){
       Treatment_PostOperation(Resolution_P, DofData_P0, DefineSystem_P0, GeoData_P0,
@@ -640,8 +643,8 @@ void SolvingAnalyse()
       i++ ;
     }
 
-    Msg::Cpu("");
-    Msg::Direct("E n d   P o s t - P r o c e s s i n g");
+    Message::Cpu("");
+    Message::Direct("E n d   P o s t - P r o c e s s i n g");
   }
 
   List_Delete(DofData_L) ;
